*COMDECK ZONEPH
C-----------------------------------------------------------------------:-------
C INCLUDE MODULE: ZONEPH
C-----------------------------------------------------------------------:-------
C
C.....COMMON BLOCKS PERTAINING TO PHYSICAL CONDITIONS IN ONE ZONE
C
C
C.....INPUT SWITCHES
C
C     LOGICAL:   MEANING IF TRUE
C     -------    ---------------
C     LININP     INPUT VARIABLES AVAILABLE IN STANDARD UNITS (CGS)
C     LOGINP     INPUT VARIABLES AVAILABLE AS LOGARITHMS
C     LSTATE     EQUATION OF STATE IS REQUIRED
C     LKAPPA     OPACITY IS REQUIRED
C     LNUKE      NUCLEAR ENERGY GENERATION RATE IS REQUIRED
C     LRATES     NUCLEAR REACTION RATES (DX/DT) ARE REQUIRED
C     LDEL       ENERGY TRANSPORT GRADIENT IS REQUIRED (DLNT/DLNP)
C     LDERIV     DERIVATIIVES OF ALL QUANTITIES ARE REQUIRED
C
      COMMON /PHREQD/ LININP, LOGINP,
     1                LSTATE, LKAPPA, LNUKE,  LRATES, LDEL,
     2                LDERIV
C
C     LOGICAL:   MEANING IF TRUE
C     -------    ---------------
C     LPSVOK     EOS VARIABLES SET
C     LPSLOK     EOS LOGARITHMS SET
C     LPSDOK     EOS DERIVATIVES SET
C     LPSLST     FLAG TO USE LAST EOS FOR STARTING VALUES
C     LPKVOK     OPACITY VARIABLES SET
C     LPKLOK     OPACITY LOGARTHMS SET
C     LPKDOK     OPACITY DERIIVATIVES SET
C     LPNVOK     ENERGY GEN VARIABLES SET
C     LPLVOK     ENERGY GEN LOGARITHMS SET
C     LPDVOK     ENERGY GEN DERIIVATIVES SET
C
      COMMON /PHFLAG/ LPSVOK, LPSLOK, LPSDOK, LPSLST,
     1                LPKVOK, LPKLOK, LPKDOK,
     2                LPNVOK, LPNLOK, LPNDOK
C
C-------------------------------------------------------------------------------
C
C.....EQUATION OF STATE
C
C     CGS     LOG     STATE VARIABLES:   
C     ---     ---     ---------------
C     PTOT    PTL     TOTAL PRESSURE (ALL CONTRIBUTIONS)
C     PGAS    PGL     TOTAL GAS PRESSURE (PION + PELEC + PCOUL)
C     PRAD    PRL     RADIATION PRESSURE
C     PION    PIL     PGAS COMPONENT DUE TO KINETIC ENERGY OF IONS
C     PCOUL   PCL     PGAS COMPONENT DUE TO COULOMB INTERACTION OF IONS 
C     PELEC   PEL     PGAS COMPONENT DUE TO KINETIIC ENERGY OF ELECTRONS
C     T       TL      TEMPERATURE
C     RO      ROL     MASS DENSITY
C     NE      NEL     NUMBER DENSITY OF ELECTRONS
C     NP      NPL     NUMBER DENSITY OF PARTICLES
C     NI      NIL     NUMBER DENSITY OF IONS
C     U       UL      INTERNAL ENERGY DENSITY
C     ETA             DEGENERACY PARAMETER
C     BETA            RELATIVITY PARAMETER
C
      COMMON /STATEV/ PTOT,   PGAS,   PRAD,   PION,   PCOUL,  PELEC,
     1                T,      RO,     NE,     NP,     NI,     U,
     2                ETA,    BETA
      COMMON /STATEL/ PTL,    PGL,    PRL,    PIL,    PCL,    PEL,
     1                TL,     ROL,    NEL,    NPL,    NIL,    UL
      COMMON /STATED/ CHIRO,  CHIT,   CP,     CV,     DELAD,
     1                GAMMA,  GAMMA1, GAMMA2, GAMMA3,
     2                DRODP,  DRODT,  DUDP,   DUDT,   DDADDP, DDADDT,
     3                DNEDP,  DNEDT,  DMUDP,  DMUDT
      COMMON /STATEF/ LZION,  LSAHA,  LFION,  LNDEG,  LPDEG,  LFDEG,
     1                        LCDEP
C
C-------------------------------------------------------------------------------
C
C.....OPACITIES
C
C     CGS    LOG     OPACITY VARIABLE:
C     ---    ---     ----------------
C     OPAC   OPL     TOTAL STELLAR OPACITY - ALL SOURCES
C     OPES   OPEL    RADIATIVE OPACITY DUE TO ELECTRON SCATTERING
C     OPRAD  OPRL    RADIATIVE OPACITY DUE TO F-F AND B-F COLLISIONS
C     OPCON  OPCL    CONDUCTIVE OPACITY
C     DOPDT          DERIVATIVE WRT TEMPERATURE
C     DOPDRO         DERIVATIVE WRT DENSITY
C
C     LOGICAL        MEANING IF TRUE:
C     -------        ---------------
C     LOPRCH         RADIATIVE OPACITY FROM CHRISTY APPROX
C     LOPRCS         RADIATIVE OPACITY FROM IBEN FIT TO COX-STEWART TABLES
C     LOPCCN         CONDUCTIVE OPACITY FROM IBEN FIT TO CANUTO TABLES
C     LOPCHL         CONDUCTIVE OPACITY FROM IBEN FIT TO HUBBARD-LAMPE TABLES
C
      COMMON /OPACV/  OPAC,   OPES,   OPRAD,  OPCON
      COMMON /OPACL/  OPACL,  OPEL,   OPRL,   OPCL
      COMMON /OPACD/  DOPDT,  DOPDRO
      COMMON /OPACF/  LOPRCH, LOPRCS, LOPCCN, LOPCHL
C
C-------------------------------------------------------------------------------
C
C.....THE TEMPERATURE GRADIENT
      COMMON /DELLV/  DEL,    DELRAD
      COMMON /DELLL/  DELLN,  DELRL
      COMMON /DELLD/  DDLDP,  DDLDT,  DDLDL
      COMMON /DELLF/  LDELL
C
C-------------------------------------------------------------------------------
C
C.....ENERGY GENERATION RATES
      COMMON /EPSV/   EPS,    EPSPP,  EPSCN,  EPS3A,  EPSCHE, EPSNU
      COMMON /EPSL/   EPSL
      COMMON /EPSD/   DEPDT,  DEPDRO, EV,     EN
      COMMON /EPSF/   LEPS
C
C-------------------------------------------------------------------------------
C
C.....TOTAL DERIVATIVES
      COMMON /TOTALD/ DDRODP, DDRODT, DDUDP,  DDUDT,  DDOPDP, DDOPDT,
     1                DDEPDP, DDEPDT
C
C===============================================================================
