MODULE SFIT_GENERAL

  !+
  !  
  !  Program:
  !     SFIT_GENERAL
  !
  !  Language:
  !     Fortran95
  !
  !  Purpose:
  !=     Provide general text services for "sfit"
  !
  !  Contains:
  !     SUBROUTINE SFIT_ANNOUNCE     startup messages
  !     SUBROUTINE SFIT_HELP         general online help
  !     SUBROUTINE SFIT_VALID        current valid configurations
  !
  !  Author:
  !     C.S.Jeffery (CSJ) (Armagh Observatory, 1999- )
  !
  !  History:
  !    10-Feb-2005: CSJ
  !       extracted from SFIT main program
  !
  !-

CONTAINS

  SUBROUTINE SFIT_ANNOUNCE ( PROG__ID, PROG__VERS, PROG__DATE, PROG__URL )

    CHARACTER(LEN=*) :: PROG__ID, PROG__VERS, PROG__DATE, PROG__URL 

    WRITE (*,*) 
    WRITE (*,*) '  SSS    FFFF  III  TTTTT    '
    WRITE (*,*) ' S       F      I     T      '
    WRITE (*,*) '  SSS    FFF    I     T      '
    WRITE (*,*) '     S   F      I     T      '
    WRITE (*,*) '  SSS    F     III    T      '
    WRITE (*,*) 
    WRITE (*,*) ' Program: ',PROG__ID
    WRITE (*,*) ' Version: ',PROG__VERS
    WRITE (*,*) ' Date   : ',PROG__DATE
    WRITE (*,*) ' URL    : ',PROG__URL 
    WRITE (*,*)
    WRITE (*,*) ' Principal Author: Simon Jeffery, Armagh Observatory'
    WRITE (*,*)
    WRITE (*,*) ' Contributing Authors'
    WRITE (*,*) '          Philip Dufton, Queens University Belfast'
    WRITE (*,*) '          Vincent Woolf, Armagh Observatory (1999-2002)'
    WRITE (*,*) '          Glenn Hawe, Armagh Observatory (2002)'
    WRITE (*,*) '          Mairead Skelly, Armagh Observatory (2002)'
    WRITE (*,*) '          Chris Winter, Armagh Observatory (2006)'
    WRITE (*,*) 
    WRITE (*,*) ' For validity, type VALID '
    WRITE (*,*) ' For brief help, type HELP, or see: '
    WRITE (*,*) PROG__URL
    WRITE (*,*)

    IF ( PROG__VERS(1:3) == "2.1" ) THEN
      WRITE (*,*) ' SFIT 2.1 is an alpha test version. '
      WRITE (*,*)
      WRITE (*,*) ' Version Notes. '
      WRITE (*,*) ' 1. Several debug statements are still included. '
      WRITE (*,*) ' 2. There is a known bug where LSYNT overwrites the value of &
           & SELNO in SP_SYNTH '
      WRITE (*,*) '    I could not find the reason; nor were there any adverse consequences '
      WRITE (*,*) ' 3. The genetic algorithm method for "solve" has been included '
      WRITE (*,*) ' 4. "solve" now creates an output file containing fit parameters for &
           & each solution. '
      WRITE (*,*) '    The file extension is ".sfit_pars" '
      WRITE (*,*)
    ENDIF

    IF ( PROG__VERS(1:3) == "2.2" ) THEN
      WRITE (*,*) ' SFIT 2.2 is a beta test version. '
      WRITE (*,*)
      WRITE (*,*) ' Version Notes. '
      WRITE (*,*) ' 0. Extensive testing during 2003 October/November has continued '
      WRITE (*,*) '    to remove several bugs '
      WRITE (*,*) ' 4. "solve" and "synth" create output files containing fit parameters for &
           & each solution. '
      WRITE (*,*) '    The file extensions are ".slv_pars" and ".syn_pars" respectively.'
      WRITE (*,*)
      WRITE (*,*) ' 5. New commands provided for setting individual parameters ' 
      WRITE (*,*) ' 6. Summary file (SFIT.SUM) provided '
      WRITE (*,*)
    ENDIF

    IF ( PROG__VERS(1:3) == "2.5" ) THEN
      WRITE (*,*) ' Development version. '
      WRITE (*,*)
      WRITE (*,*) ' Notes. '
      WRITE (*,*) ' 1. A new suite of commands for setting parameters is introduced '
      WRITE (*,*) ' 2. Defining and playing of macros is introduced (macro, play) '
    ENDIF

    IF ( PROG__VERS(1:3) == "2.6" ) THEN
      WRITE (*,*) ' Development version. '
      WRITE (*,*)
      WRITE (*,*) ' Notes (2.5). '
      WRITE (*,*) ' 1. A new suite of commands for setting parameters is introduced '
      WRITE (*,*) ' 2. Defining and playing of macros is introduced (macro, play) '
      WRITE (*,*) ' Notes (2.6). '
      WRITE (*,*) ' 1. New continuum region setting commands (cont_regs, cont_regd, ...) '
      WRITE (*,*) ' 2. Structural changes to model arrays: '
      WRITE (*,*)  '       AMOD,ABIN removed, MOD and MOD2 allowed different dimensionality '
    ENDIF

    IF ( PROG__VERS(1:3) == "2.7" ) THEN
      WRITE (*,*) ' Notes (2.7) New commands:'
      WRITE (*,*) ' cont_reg_set, cont_reg_disp, cont_reg_clear '
      WRITE (*,*) ' data_mask_set, data_mask_disp, data_mask_clear '
      WRITE (*,*) ' data_flux_units, data_flux_scale '
      WRITE (*,*) ' method_formal '
    ENDIF

    IF ( PROG__VERS(1:3) == "3.0" ) THEN
      WRITE (*,*) ' Notes (3.0) New commands:'
    ENDIF

    WRITE (*,*)

    WRITE (*,*)
    WRITE (*,*) ' Please report all bugs, with associated input data '
    WRITE (*,*)

  END SUBROUTINE SFIT_ANNOUNCE


  SUBROUTINE SFIT_VALID 
    WRITE (*,*)
    WRITE (*,*) ' Current Validity '
    WRITE (*,*)
    WRITE (*,*) ' mode / method   / data  : status '
    WRITE (*,*)
    WRITE (*,*) ' FLX / CHISQ     / LORES :   --  '
    WRITE (*,*) ' FLX / AMOEBA    / LORES :   --  '
    WRITE (*,*) ' FLX / LEVENBURG / LORES :   --  '
    WRITE (*,*) ' FLX / GENETIC   / LORES :   --  '
    WRITE (*,*)
    WRITE (*,*) ' SLV / CHISQ     / LORES :   ok  14/10/04 '
    WRITE (*,*) ' SLV / AMOEBA    / LORES :   ok  14/10/04 '
    WRITE (*,*) ' SLV / LEVENBURG / LORES :   ok ?14/10/04 '
    WRITE (*,*) ' SLV / GENETIC   / LORES :   ??   '
    WRITE (*,*)
    WRITE (*,*) ' SLV / CHISQ     / HIRES :   ??   '
    WRITE (*,*) ' SLV / AMOEBA    / HIRES :   ok  21/10/04   '
    WRITE (*,*) ' SLV / LEVENBURG / HIRES :   ok  21/10/04   '
    WRITE (*,*) ' SLV / GENETIC   / HIRES :   ??   '
    WRITE (*,*)
    WRITE (*,*) ' SYN / CHISQ     / HIRES :   --  '
    WRITE (*,*) ' SYN / AMOEBA    / HIRES :   ??   '
    WRITE (*,*) ' SYN / LEVENBURG / HIRES :   ok  20/10/04 '
    WRITE (*,*) ' SYN / GENETIC   / HIRES :   --  '
    WRITE (*,*)
  END SUBROUTINE SFIT_VALID

  SUBROUTINE SFIT_HELP ( COMMAND )

    CHARACTER*(*) :: COMMAND

    IF ( COMMAND == ' ' ) THEN
      WRITE (*,*)
      WRITE (*,*) ' SFIT currently has two [three] major modes of operation ...: '
      WRITE (*,*) '  slv (solve) -- fitting normalixed spectra in grid '
      WRITE (*,*) '  syn (synth) -- optimizing abundances in model spectrum '
      WRITE (*,*) ' [flx (flux)  -- fitting observed flux distributions and photometry]'
      WRITE (*,*)
      WRITE (*,*) ' ... and four major methods of solution:'
      WRITE (*,*) '  chisq       -- brute force minimum chi^2 in model grid  '
      WRITE (*,*) '  levenburg   -- levenburg-marquardt chi^2 minimization '
      WRITE (*,*) '  amoeba      -- downhill simplex method '
      WRITE (*,*) '  genetic     -- optimisation using a genetic algorithm ' 
      WRITE (*,*)
      WRITE (*,*) ' Commands related to different modes and methods are prefixed appropriately '
      WRITE (*,*) '  slv_   -- SLV parameters '  
      WRITE (*,*) '  syn_   -- SYN parameters ( to be implemented )'  
      WRITE (*,*) '  flx_   -- FLX parameters ( to be implemented )'  
      WRITE (*,*) '  method_  -- define solution method and control parameters '
      WRITE (*,*) '  data_  -- define obs data and parameters '
      WRITE (*,*) '  cont_  -- define how normalisation is done '
      WRITE (*,*) '  spc_   -- defining how spectrum works ( to be implemented )'
      WRITE (*,*) 
      WRITE (*,*) ' Other commands '
      WRITE (*,*) '  model      -- set up a grid of model spectra '
      WRITE (*,*) '  save-grid  -- save a grid of model spectra '
      WRITE (*,*) '  read_grid  -- read a grid of model spectra '
      WRITE (*,*) '  show <grp> -- display groups of parameters: '
      WRITE (*,*) '                <grp> = [all, meth, data, cont, ...] '
      WRITE (*,*) '  ...  '
      WRITE (*,*) 
      WRITE (*,*) ' Type "help <command>" for further information (to be implemented ) '
      WRITE (*,*) 
    END IF

  END SUBROUTINE SFIT_HELP

END MODULE SFIT_GENERAL
