*
*     Programme to transfer ATLAS6 models to QUB format
*     using tau_4000 scale
*
      PROGRAM ATLAS4000


*
*      Double precision version
*
      IMPLICIT REAL*8 (A-H,O-Z)

*
*      Common blocks
*
      INCLUDE 'common_all.f'
      COMMON/ATLAS6/M6(100),TAU6(100),T6(100),PG6(100),
     *              NE6(100),PE6(100),OP6(100),NP6
      COMMON/ATLAS6B/ TEFF6,GRAV6,Y6

      REAL*8 M6,NE6,X(3),Z(3)

      CHARACTER*80 MODEL

*
*      Debug 
*
      BUGOT=.FALSE.

*
*      Define important input/output streams
*
      NREAD=5
      NWRITE=6
      NDISC=7
      NMODEL=8
      NATOM=11
      NDEBUG=99

*
*      Ensure QUB opacity routines are used 
*
      NOPTYP = 0

*
*       Define Debug output file if appropriate
*
      IF(BUGOT.EQ..TRUE.) THEN
        OPEN(UNIT=NDEBUG,FILE='ATLAS_QUB.DBG',STATUS='NEW')
      ENDIF


*
*       Read in Atomic Data
*
      CALL ATDATA

*
*      Define optical depth scale
*
      NDEPTH=25
      TAU(1)=1.E-4
      TAU(2)=3.1E-4
      DO 1 I=3,25
      I3=I-3
    1 TAU(I)=1.E-3*(10**(0.2*I3))

*
*      Read in ATLAS6 model
*
   11 CALL ATLAS_RD

*
*      Calculate composition
*      AHE is reset to fractional number of helium atoms by COMPOS
*
      AHE=Y6
      CALL COMPOS

      IF(BUGOT) THEN
        WRITE(NDEBUG,*) 'COMPOS call OK'
      ENDIF
*
*      Calculates electron pressure and optical depth at 4000 A
*
      RLAMB=4000.
      NOPTYP = 0
      DO I=1,NP6
        PE6(I)=1.38033E-16*NE6(I)*T6(I)
        OP6(I)=OPAC(T6(I),PG6(I),PE6(I),RLAMB,CAP,SIG,NP6,NOPTYP)
      ENDDO


*
*      Optical depth scale at 4000A
*
      TAU6(1)=OP6(1)*M6(1)
      DO I=2,NP6
        TAU6(I)=TAU6(I-1)+0.5*(OP6(I)+OP6(I-1))*(M6(I)-M6(I-1))
      ENDDO

*
*     Debug Output
*
      IF(BUGOT) THEN
        WRITE(NDEBUG,900) TEFF6,GRAV6,Y6,(TAU6(I),M6(I),T6(I),
     *                    PG6(I),PE6(I),OP6(I),I=1,NP6)
      ENDIF

*
*      Interpolate onto QUB scale
*
      TEFF=TEFF6
      GRAV=10.**GRAV6
      Y=Y6

      IST=3
      DO  I=1,NDEPTH

*
*     Find nearest points in ATLAS6 model
*
        DO J=IST,NP6
         IF(TAU6(J).GT.TAU(I))GOTO 10
        ENDDO
        J=NP6

   10   IST=J
        X(1)=TAU6(J-2)
        X(2)=TAU6(J-1)
        X(3)=TAU6(J)

*
*      Temperature
*
        Z(1)=T6(J-2)
        Z(2)=T6(J-1)
        Z(3)=T6(J)
        CALL POLY(X,Z,TAU(I),TEMP(I))

*
*      Gas pressure
*
        Z(1)=PG6(J-2)
        Z(2)=PG6(J-1)
        Z(3)=PG6(J)
        CALL POLY(X,Z,TAU(I),PG(I))

*
*      Electron pressure
*
        Z(1)=PE6(J-2)
        Z(2)=PE6(J-1)
        Z(3)=PE6(J)
        CALL POLY(X,Z,TAU(I),PE(I))

*
*      Opacity
*
        OP(I)=OPAC(TEMP(I),PG(I),PE(I),RLAMB,CAP,SIG)

      ENDDO

*
*      Output Model
*
      WRITE(NWRITE,200)
      READ(NREAD,100)MODEL

*
*      Output to screen
*
      WRITE(NWRITE,202)TEFF,GRAV,Y,
     1            (TAU(I),TEMP(I),PG(I),PE(I),OP(I),I=1,NDEPTH)

*
*      Output file
*
      OPEN(UNIT=NDISC,STATUS='NEW',FILE=MODEL,FORM='FORMATTED')
      WRITE(NDISC,201)TEFF,GRAV,Y
      WRITE(NDISC,201)(TAU(I),TEMP(I),PG(I),PE(I),OP(I),I=1,NDEPTH)
      CLOSE(UNIT=NDISC)

*
*      Check for continuation
*
      WRITE(NWRITE,203)
      READ(NREAD,*)NCONT
      IF (NCONT.EQ.0) GOTO 11

*
*      Format statements
*
  100 FORMAT(A80)

  200 FORMAT(' FILENAME FOR MODEL OUTPUT (+DIRECTORY IF DIFFERENT)')
  201 FORMAT(1P5E12.5)
  202 FORMAT('TEFF=',F10.0,4X,'GRAV=',1PE11.4,4X,'(He/H)=',0PF6.3//
     11H ,8X,'TAU',8X,'TEMP',10X,'PG',10X,'PE',8X,'OPAC'//(1P5E12.4))
  203 FORMAT(' ANOTHER MODEL (0=YES)')
 
  900 FORMAT('ATLAS6 MODEL'//
     *'TEFF=',F10.0,4X,'GRAV=',1PE11.4,4X,'(He/H)=',0PF6.3//
     *1H ,8X,'TAU',8X,'MASS',8X,'TEMP',10X,'PG',10X,'PE',8X,'OPAC'//
     *(1P6E12.4))

      STOP
      END

