*+   CHR_CTOI - Read an integer number from a character string.
      SUBROUTINE CHR_CTOI (STRING,IVALUE,STATUS)
*    Description :
*     Read an integer number from a character string.
*    Invocation :
*     CALL CHR_CTOI (STRING, IVALUE, STATUS)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*           String from which an attempt will be made to read
*           an integer number.
*     IVALUE  =  INTEGER (Returned)
*           Value read from the input string.
*     STATUS = INTEGER (Modified)
*           Variable holding the status value.
*           If this variable is not SAI__OK on input, then the routine
*           will return without action.
*           If the routine fails to complete, STATUS will be set
*           to SAI__ERROR.
*    Method :
*     Construct I-format and decode using Fortran 77 READ.
*    Authors :
*     Jack Giddings.               (UCL::JRG)
*     A C Davenhall.               (ROE::ACD)
*    History :
*     19-MAR-1982:  Original.                             (UCL::JRG)
*      2-OCT-1984:  Documentation improved.               (ROE::ACD)
*      3_OCT_1988:  Documentation improved.               (RAL::AJC)
*     16-AUG-1989:  Use SAE_PAR                           (RAL::AJC)
*     22-jan-1990:  Eliminate overlapping substring       (RAL::DLT)
*                   assignment
*    Type definitions :
      IMPLICIT NONE
*    Global constants :
      INCLUDE 'sae_par.f'
*    Import :
      CHARACTER
     :  STRING*(*)      ! string to be decoded.
*    Export :
      INTEGER
     :  IVALUE          ! Value decoded.
*    Status :
      INTEGER
     :  STATUS          ! Status flag.
*    External references :
      INTEGER
     :  CHR_LEN         ! String length.
*    Local variables :
      CHARACTER
     :  FORMAT*8,       ! Fortran 77 format string.
     :  COUNT*3         ! Character count
      INTEGER
     :  NCHAR,          ! Character count.
     :  CODE            ! Code from READ/WRITE
*-

      IF (STATUS .EQ. SAI__OK) THEN

         NCHAR = CHR_LEN(STRING)
         WRITE (COUNT, '(I3)', IOSTAT=CODE) NCHAR
         IF (CODE .NE. 0) THEN
            STATUS = SAI__ERROR
         ELSE
            FORMAT = '(I'//COUNT//')'
            READ (STRING(1:NCHAR),FORMAT, IOSTAT=CODE) IVALUE
            IF (CODE .NE. 0) THEN
               STATUS = SAI__ERROR
            END IF
         END IF

      END IF

      END
