*+ CHR_CTOL - Read a logical value from a character string.
      SUBROUTINE CHR_CTOL (STRING, LVALUE, STATUS)
*    Description :
*     The string is decoded as a boolean constant.
*     TRUE, T, YES, Y and FALSE, F, NO, N are recognized regardless of case.
*     Other strings will result in STATUS being set to SAI__ERROR.
*    Invocation :
*     CALL CHR_CTOL (STRING, LVALUE, STATUS)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*           String to be decoded.
*     LVALUE  =  LOGICAL (Returned)
*           Decoded value.
*     STATUS = INTEGER (Modified)
*           Variable holding the status value.
*           If this variable is not SAI__OK on input, then the routine
*           will return without action.
*           If the routine fails to complete, STATUS will be set
*           to SAI__ERROR.
*    Method :
*     Check that the used length of the string is .le. 5. If so
*     convert the string to upper case and check against the valid
*     values setting LVALUE appropriately.
*     If the string is not a recognized logical value, set status to
*     SAI__ERROR
*    Authors :
*     Jack Giddings.         (UCL::JRG)
*     A C Davenhall.         (ROE::ACD)
*     A J Chipperfield.      (RAL::AJC)
*    History :
*     19-MAR-1982:   Original.                          (UCL::JRG)
*     19-NOV-1984:   Documentation improved.            (ROE::ACD)
*     03-SEP-1988:   Documentation improved.            (RAL::AJC)
*     21-DEC-1988:   Rewrite for efficiency.            (RAL::AJC)
*     03-MAR-1989:   Use correct length of string       (RAL::AJC)
*     16-AUG-1989:   Use SAE_PAR                        (RAL::AJC)
*    Type definitions :
      IMPLICIT NONE
*    Global constants :
      INCLUDE 'sae_par.f'
*    Import :
      CHARACTER
     :  STRING*(*)           ! String to be decoded.
*    Export :
      LOGICAL
     :  LVALUE               ! Decoded value.
*    Status :
      INTEGER
     :  STATUS               ! Status flag.
*    External references :
      INTEGER
     :  CHR_LEN              ! Used length of string
*    Local variables :
      INTEGER SLEN           ! Used length of string
      CHARACTER*5 LSTRING    ! Local string - upper case
*-

      IF (STATUS .EQ. SAI__OK) THEN

         SLEN = CHR_LEN (STRING)
*       Check that used length is in range
         IF (SLEN .LE. 5) THEN
*          If length OK, convert to upper case
            LSTRING = STRING(1:SLEN)
            CALL CHR_UCASE (LSTRING)
*          Check for TRUE
            IF ((LSTRING .EQ. 'Y') .OR.
     :          (LSTRING .EQ. 'YES') .OR.
     :          (LSTRING .EQ. 'T') .OR.
     :          (LSTRING .EQ. 'TRUE')) THEN
               LVALUE = .TRUE.

*          If not TRUE, check for false
            ELSEIF ((LSTRING .EQ. 'N') .OR.
     :          (LSTRING .EQ. 'NO') .OR.
     :          (LSTRING .EQ. 'F') .OR.
     :          (LSTRING .EQ. 'FALSE')) THEN
               LVALUE = .FALSE.

*          On invalid strings, return SAI__ERROR
            ELSE
               STATUS = SAI__ERROR
            ENDIF

         ELSE
            STATUS = SAI__ERROR
         ENDIF

      ENDIF

      END
