*+  CHR_DELIM - Locate substring with given delimiter character.
      SUBROUTINE CHR_DELIM( STRING, DELIM, INDEX1, INDEX2 )
*    Description :
*     The input string, STRING, is examined to see if it contains a
*     substring delimited by the character DELIM. 
*     The indices are returned as INDEX1 and INDEX2. \\\\
*     If no occurrence of DELIM is found, or if the only occurrence of
*     DELIM is the last character of the string, then the indices are 
*     returned pointing to the whole of the input string. \\\\
*     If only one occurrence of DELIM is found and it is not the last
*     character in the string, then INDEX1 will point to this position 
*     and INDEX2 will point to the last character in the string. \\\\
*     If there are more than two occurrences of DELIM then INDEX1 will 
*     point to the first occurrence and INDEX2 to the last occurrence.
*    Invocation :
*     CALL CHR_DELIM( STRING, DELIM, INDEX1, INDEX2 )
*    Arguments :
*     STRING = CHARACTER*(*) (Given)
*           String to be searched.
*     DELIM = CHARACTER (Given)
*           The substring delimiting character.
*     INDEX1 = INTEGER (Returned)
*           Index to the first occurrence of the delimiter or to the
*           first character in the string.
*     INDEX2 = INTEGER (Returned)
*           Index to the last occurrence of the delimiter or to the
*           last character in the string.
*    Method :
*     Get the declared size of the string.
*     Find the index to the delimiter in the whole of the string.
*     If the delimiter character not found or the delimiter character
*     is the last character of the string then
*        Set the first index to 1 and the second index to the
*        declared size of the string.
*     Else
*        Set the first index to the delimiter index
*        Set the current positon in the string to the first index.
*        Repeat
*           Get index to the delimiter character in the substring
*           from the current position index plus one to the end of
*           the string.
*           if the delimiter index is not zero then
*              Set the second index to the current position plus the
*              delimiter index.
*           Endif
*        Until the current delimiter index is zero or the current
*        position is equal to or exceeds the declared string length.
*        If no second occurrence was found then
*           Second index is set equal to declared string length.
*        Else
*           Second index is set equal last occurrence position.
*        Endif
*     Endif
*    Authors :
*     Dave Baines. (ROE::ASOC5)
*    History :
*     28-Jun-1984:  Rather more Vax specific version         (ROE::ASOC5)
*      1-Sep-1988:  Use LEN instead of CHR_SIZE 
*                   Remove INCLUDE 'SAE_PAR'                 (RAL::AJC)
*      3-Oct-1988:  Improve documentation.                   (RAL::AJC)
*    Type Definitions :
      IMPLICIT NONE ! switch off default typing
*    Import :
      CHARACTER*(*)
     :  STRING ! string to be sarched
      CHARACTER
     :  DELIM ! delimiter character
*    Export :
      INTEGER
     :  INDEX1, ! index to first occurrence of delimiter
     :  INDEX2  ! index to second occurrence of delimiter
*    External references :
      INTEGER INDEX ! index to character in string
*    Local variables :
      INTEGER
     :  SIZE, ! declared size of passed string
     :  DELIND, ! index to delimiter character
     :  CURPOS  ! current position index into string
*-

*    get declared size of passed string
      SIZE = LEN( STRING )

*    find index to delimiter character in whole of passed string
      DELIND = INDEX( STRING, DELIM )

      IF( ( DELIND .LE. 0 ) .OR. ( DELIND .EQ. SIZE ) )THEN

*       delimiter character not found or delimiter character is
*       last character in string
*       set first index to 1 and second index to declared size of string
         INDEX1 = 1
         INDEX2 = SIZE
      ELSE

*       set first index to delimiter index
         INDEX1 = DELIND

*       set current position in string to first index
         CURPOS = INDEX1

*       repeat until no further occurnces of delimiter are found or
*       string finishes
         DO WHILE( ( DELIND .GT. 0 ) .AND. ( CURPOS .LT. SIZE ) )

*          get index to delimiter character in substring
*          from character after current delimiter to end of string
            DELIND = INDEX( STRING( CURPOS+1:SIZE ), DELIM )

*          check delimiter found
            IF( DELIND .GT. 0 ) THEN

*             reset current position
               CURPOS = CURPOS + DELIND
            ENDIF
         ENDDO

*       set second index
         IF( CURPOS .EQ. INDEX1 ) THEN

*          no second occurrence found
            INDEX2 = SIZE
         ELSE

*          set second index to last occurrence position
            INDEX2 = CURPOS
         ENDIF
      ENDIF

      END
