*+  CHR_FANDL - Find the indices of the first and last non-blank characters.

      SUBROUTINE CHR_FANDL ( STRING, INDEX1, INDEX2 )

*    Description :
*     Find the indices of the first and last non-blank characters in
*     the given string. If the string is all spaces then the
*     first index points to the end of the string and the last index
*     is set to zero. i.e. INDEX1 > INDEX2.
*     If the string has no length then zero is returned for the indices.
*    Invocation :
*     CALL CHR_FANDL( STRING, INDEX1, INDEX2 )
*
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*             Input string.
*     INDEX1  =  INTEGER (Returned)
*             Position of first non-blank character.
*     INDEX2  =  INTEGER (Returned)
*             Position of last non-blank character.
*
*    Authors :
*     Nick Eaton.  ( DUVAD::NE )
*     A J Chipperfield.  ( RLVAD::AJC )
*
*    History :
*        Aug 1988: Original
*     27-Feb-1989: Revise and use CHR_LEN - improved speed. (RLVAD::AJC)
*    endhistory
*
*    Type Definitions :
      IMPLICIT NONE

*    Import :

*     Input string
      CHARACTER * ( * ) STRING

*    Export :

*     Index of first non-blank character in string
      INTEGER INDEX1

*     Index of last non-blank character in string
      INTEGER INDEX2

*    External routines :
      INTEGER CHR_LEN

*    Local variables :
      LOGICAL SPACES
*-

*   Find the used length of the input string and check it is not zero
      INDEX2 = CHR_LEN( STRING )

      IF (INDEX2 .LE. 0) THEN
*      None used
         INDEX2 = 0
*      Set INDEX1 to declared length of string
         INDEX1 = LEN( STRING )
*      Correct for null string
         IF ( INDEX1 .LE. 0 ) THEN
            INDEX1 = 0
         ENDIF

      ELSE

*      The string is normal.
*      Find the position of the first non-blank character.
*      Use a flag SPACES to indicate if the search is still to continue.
         INDEX1 = 1
         SPACES = .TRUE.
         DO WHILE( SPACES .AND. ( INDEX1 .LE. INDEX2 ) )
            IF ( STRING( INDEX1:INDEX1 ) .EQ. ' ' ) THEN
               INDEX1 = INDEX1 + 1
            ELSE
               SPACES = .FALSE.
            ENDIF
         ENDDO

      ENDIF

      END
    
