*+  CHR_FIWE - Find next end of word
      SUBROUTINE CHR_FIWE (STRING, INDEX, STATUS)
*    Description : 
*     Find the next end of word, signified by the following character
*     being a word delimiter (space, tab or comma). Note that the
*     start of the next word is not found before looking for the
*     next word delimiter so it is possible for INDEX to remain
*     unchanged and indeed to point to a word delimiter rather than a
*     true end of a word. This routine is expected to be used in
*     conjunction with CHR_FIWS.
*    Invocation :
*     CALL CHR_FIWE (STRING, INDEX, STATUS)
*    Parameters : 
*     STRING=CHARACTER*(*)(Given)
*           The string to be searched.
*     INDEX=INTEGER(Modified)
*           The given value is the character position within STRING
*           at which searching is to start.
*           If INDEX is less than 1, the search starts at position 1.
*           The returned value is the character position preceding the
*           next word delimiter. 
*           If INDEX already points to a character preceding a delimiter,
*           it will not change.
*           If no delimiter is found, INDEX will point to the end of 
*           the string, and STATUS will be set.
*     STATUS=INTEGER(Modified)
*           Variable holding the status value.
*           If this variable is not SAI__OK on entry, then the
*           routine will return without action.
*           If the next delimiter not found before the end of the
*           string, STATUS is set to CHR__ENDOFSENT.
*    Method : 
*     <description of how the application works> 
*    Authors : 
*     Jon Fairclough.  (JACH::JHF)
*    History : 
*     28-Oct-1988:  Original
*     16-Aug-1989:  Name changed FIND_WORD_END to FIWE.
*                   Change variable names for consistency with CHR.
*                   Improve prologue documentation.
*                   Change to ADAM__OK to SAI__OK.  (RLVAD::AJC)
*    Type Definitions : 
      IMPLICIT NONE 
*    Global constants : 
      INCLUDE 'sae_par.f'
      INCLUDE 'chr_err.f'
*    Import:
      CHARACTER*(*) STRING
      INTEGER INDEX
*    Export:
*     <declarations for variables output from routine>
*    Status : 
      INTEGER STATUS 
*    External references : 
*     <declarations for external function references> 
*    Global variables : 
*     <any INCLUDE files for global variables held in named COMMON> 
*    Local Constants : 
*     <local constants defined by PARAMETER> 
*    Local variables : 
      INTEGER LENGTH
*    Internal References : 
*     <declarations for internal functions> 
*    Local data : 
      CHARACTER TAB
*     PARAMETER (TAB=CHAR(9)) 
*- 
      IF (STATUS .NE. SAI__OK) RETURN
      TAB=CHAR(9) 
*
*    Begin
*
      LENGTH = LEN (STRING)
      IF (INDEX+1 .GE. LENGTH) GOTO 999
      IF (INDEX .LT. 1) INDEX = 1
      DO WHILE (STRING(INDEX+1:INDEX+1) .NE. ' ' .AND.
     :          STRING(INDEX+1:INDEX+1) .NE. TAB .AND.
     :          STRING(INDEX+1:INDEX+1) .NE. ',') 
         IF (INDEX .GE. LENGTH) GOTO 999
         INDEX = INDEX + 1
      ENDDO
      RETURN
*
999   CONTINUE
      INDEX = LENGTH
      STATUS   = CHR__ENDOFSENT
*
*    End
*
      END
