*+ CHR_HTOI - Read an integer from a hex string.
      SUBROUTINE CHR_HTOI (STRING, IVALUE, STATUS)
*    Description :
*     Read an integer from a hex string. 
*    Invocation :
*     CALL CHR_HTOI (STRING, IVALUE, STATUS)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*           String to be decoded.
*     IVALUE  =  INTEGER (Returned)
*           Value decoded from the given string.
*     STATUS = INTEGER (Modified)
*           Variable holding the status value.
*           If this variable is not SAI__OK on input, then the routine
*           will return without action.
*           If the routine fails to complete, STATUS will be set
*           to SAI__ERROR.
*    Method :
*     Construct a Z-format and decode using a Fortran 77 internal READ
*     into the integer argument.
*    Authors :
*     Jack Giddings.        (UCL::JRG)
*     A C Davenhall.        (ROE::ACD)
*    History :
*     19-MAR-1982:   Original.                        (UCL::JRG)
*     19-NOV-1984:   Documentation improved.          (ROE::ACD)
*     03-OCT-1988:   Documentation improved.          (RAL::AJC)
*     16-AUG-1989:   Use SAE_PAR                      (RAL::AJC)
*     22-JAN-1990:   Eliminate overlapping substring
*                    assignment                       (RAL::DLT)
*    Type definitions :
      IMPLICIT NONE
*    Global constants :
      INCLUDE 'sae_par.f'
*    Import :
      CHARACTER
     :  STRING*(*)           ! String to be decoded.
*    Export :
      INTEGER
     :  IVALUE               ! Decoded value.
*    Status :
      INTEGER
     :  STATUS               ! Status flag.
*    External references :
      INTEGER
     :  CHR_LEN              ! string length.
*    Local variables :
      CHARACTER
     :  FORMAT*6,            ! Fortran 77 format string.
     :  COUNT*3              ! Character count
      INTEGER
     :  NCHAR,               ! Character count.
     :  CODE                 ! Return status from READ/WRITE.
*-

      IF (STATUS .EQ. SAI__OK) THEN

         NCHAR = CHR_LEN(STRING)
         WRITE (COUNT, '(I3)', IOSTAT=CODE) NCHAR
         IF (CODE .NE. 0) THEN
            STATUS = SAI__ERROR
         ELSE
            FORMAT = '(Z'//COUNT//')'
            READ (STRING(1:NCHAR),FORMAT, IOSTAT=CODE) IVALUE
            IF (CODE .NE. 0) THEN
               STATUS = SAI__ERROR
            END IF
         END IF

      END IF

      END
