*+ CHR_ISNAM - Determine whether a string is a valid name.
      LOGICAL FUNCTION CHR_ISNAM (STRING)
*    Description :
*     Determine whether the string is a valid name.  That is,
*     whether it starts with an alphabetic character and continues
*     with alphanumeric or underscore characters.
*    Invocation :
*     LOGICAL = CHR_ISNAM (STRING)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*           String that is to be tested.
*    Result :
*     CHR_ISNAM = LOGICAL
*           Returns .TRUE. if the string is a valid name;
*           returns .FALSE. otherwise.
*    Method :
*     Use CHR_LEN to determine the stringing length.
*     Check that the first character is alphabetic with CHR_ISALF.
*     Check any remaining characters with CHR_ISALM.
*    Authors :
*     Jack Giddings.               (UCL::JRG)
*     A C Davenhall.               (ROE::ACD)
*    History :
*     3-JAN-1983:    Original.                          (UCL::JRG)
*     16-NOV-1984:   Documentation improved and method  (ROE::ACD)
*                    simplified.
*    Type definitions :
      IMPLICIT NONE          ! Force Total Declarations.
*    Import :
      CHARACTER
     :  STRING*(*)           ! Stringing to be checked.
*    External references :
      INTEGER
     :  CHR_LEN              ! String length.
      LOGICAL
     :  CHR_ISALF,           ! Flag; whether alphabetic?
     :  CHR_ISALM            ! Flag; whether alphanumeric?
*    Local variables :
      INTEGER
     :  SIZE,                ! String size.
     :  INDEX                ! Current position in STRING.
*-

      SIZE = CHR_LEN(STRING)

      IF (SIZE .GT. 0) THEN

*
*       String is non-blank.

         CHR_ISNAM = CHR_ISALF(STRING(1:1))
         INDEX = 0

         DO WHILE (CHR_ISNAM  .AND.  INDEX .LT. SIZE)
            INDEX=INDEX+1
            CHR_ISNAM = CHR_ISALM(STRING(INDEX:INDEX))
         END DO

      ELSE

*
*       String is of 0 size, (or less), or totally blank. 
*       Set the result to false.

         CHR_ISNAM = .FALSE.
      END IF

      END
