*+  CHR_LDBLK - Remove leading blanks from a string.
      SUBROUTINE CHR_LDBLK (STRING)
*    Description :
*     Subroutine to remove the leading blanks from a character string.
*     The space at the end of string, created by shunting up the
*     non-blank characters, is filled with blanks.
*    Invocation :
*     CALL CHR_LDBLK (STRING)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Updated)
*           String from which the leading blanks are to be removed.
*    Method :
*     If the string is not = to blank then
*       find the position of the first non-blank characters
*       if there are some blanks to be removed,  then
*         shunt up the string to remove the leading blanks.
*         copy extra blanks into the end of the string.
*       end if
*     end if
*    Bugs :
*     None known.
*    Authors :
*     A C Davenhall.      (ROE::ACD)
*     A J Chipperfield.   (RAL::AJC)
*    History :
*     20/6/84:  Original version.                        (ROE::ACD)
*     30/6/84:  Modified to update a single string       (ROE::ACD)
*               rather than generating a new one. This
*               was done to be consistent with the other
*               CHR_ routines.
*     16/11/84: Name changed from CHR_LDBLNK to          (ROE::ACD)
*               CHR_LDBLK to conform to SSE convention.
*     03/10/88: Improve documentation                    (RAL::AJC)
*     13/06/89: Revise for speed. Use declared length of
*               string as CHR_LEN is expensive.
*               Also remove initial check for all spaces;
*               it costs every time and only if the string
*               is blank and .GT. about 20 chars.        (RAL::AJC)
*    Type Definitions :
      IMPLICIT NONE
*    Import-export :
      CHARACTER
     :  STRING*(*)  ! String to have leading blanks removed.
*    Local variables :
      INTEGER
     :  START,      ! Position of first non-blank character in string.
     :  STOP,       ! Declared length of string.
     :  CURORG,     ! Original (unshifted) current position in string.
     :  CURSHF,     ! Shifted current position in the string.
     :  LENCPY      ! No. of characters to be shifted.
*-

*    Find first non-space
      START = 1
      STOP = LEN( STRING )
      DOWHILE ((STRING(START:START).EQ.' ').AND.(START.LE.STOP))
         START = START + 1
      END DO

*    START now points to the start of the string
*    Check if there are any leading blanks to be removed.

      IF (START.GT.1) THEN
*
*       Shunt up the non-blank characters to the start of the
*       string. Do whole string as its just as expensive to find the
*       used length.

         LENCPY = STOP - START + 1
         DO CURSHF=1,LENCPY
            CURORG=CURSHF+START-1
            STRING(CURSHF:CURSHF)=STRING(CURORG:CURORG)
         END DO

*
*       Remove the characters at the end of the string that are
*       now duplicated.

         STRING(LENCPY+1:STOP)=' '

      END IF
 
      END
