*+ CHR_MOVE - Move one string into another (ignoring trailing blanks).
      SUBROUTINE CHR_MOVE (STR1,STR2)
*    Description :
*     The string STR1 (or as much of it as there is room for) is 
*     copied into STR2 beginning at position 1.
*     Trailing blanks in STR1 are ignored.
*    Invocation :
*     CALL CHR_MOVE (STR1, STR2)
*    Arguments :
*     STR1  =  CHARACTER*(*) (Given)
*           Input string.
*     STR2  =  CHARACTER*(*) (Modified)
*           Output string.
*    Method :
*     Find the total size of the output string.
*     Find the length (excl. trail blanks) of the input string.
*     Copy as much as possible of the input string into the
*     output string.
*    Authors :
*     Sid Wright.                 (UCL::SLW)
*     A C Davenhall.              (ROE::ACD)
*    History :
*     07-Mar-1983:   Original.                     (UCL::SLW)
*     11-Nov-1984:   Documentation improved.       (ROE::ACD)
*     13-Sep-1988:   Use LEN instead of CHR_SIZE
*                    Improve documentation         (RAL::AJC)
*    Type definitions :
      IMPLICIT NONE         ! Force Total Declarations.
*    Import :
      CHARACTER
     :  STR1*(*)            ! Input string.
*    Import-Export :
      CHARACTER
     :  STR2*(*)            ! Output string.
*    External references :
      INTEGER
     :  CHR_LEN             ! String length (ignoring trailing blanks).
*    Local variables :
      INTEGER
     :  SIZE2,              ! Size of STR2.
     :  LEN2                ! Length of STR2.
*-

      SIZE2 = LEN(STR2)
      LEN2 = MIN(CHR_LEN(STR1), SIZE2)
      STR2(1:LEN2) = STR1(1:LEN2)

      END

