*+ CHR_PUTC - Copy one string into another at given position
      SUBROUTINE CHR_PUTC(CVALUE,STRING,LENGTH)
*    Description :
*     The string CVALUE (or as much of it as there is room for) is 
*     copied into the part of STRING beginning at position LENGTH+1.
*     LENGTH is updated to indicate the end position of the copy of 
*     CVALUE within STRING after this operation.
*     If no copying is done, LENGTH is remains unchanged.
*     The sizes of CVALUE and STRING are based on the declared Fortran
*     77 size given by the intrinsic LEN function.
*    Invocation :
*     CALL CHR_PUTC (CVALUE, STRING, LENGTH)
*    Arguments :
*     CVALUE  =  CHARACTER*(*) (Given)
*           Input string to be copied.
*     STRING  =  CHARACTER*(*) (Modified)
*           String into which the input string is to be copied.
*     LENGTH  =  INTEGER (Modified)
*           Position pointer within STRING.
*    Authors :
*     Jack Giddings.      (UCL::JRG)
*     A C Davenhall.      (ROE::ACD)
*    History :
*     03-JAN-1983:  Original.                           (UCL::JRG)
*     02-OCT-1984:  Documenation improved.              (ROE::ACD)
*     13-SEP-1988:  Documentation improved              (RAL::AJC)
*     24-FEB-1989:  Check on string sizes               (RAL::AJC)
*    Type definitions :
      IMPLICIT NONE    ! Force Total Declarations.
*    Import :
      CHARACTER
     :  CVALUE*(*)     ! Input string.
*    Import-Export :
      CHARACTER
     :  STRING*(*)     ! Output string.
      INTEGER
     :  LENGTH         ! Position pointer within STRING.
*    Local variables :
      INTEGER
     :  SIZE2,         ! Size of STRING.
     :  SIZE1          ! Size of CVALUE.
*-

*    Get size of target string
      SIZE2 = LEN (STRING)

*    Check pointer is within string
      IF (LENGTH .LT. SIZE2) THEN
*       Get length that can be copied
         SIZE1 = MIN(LEN(CVALUE), SIZE2-LENGTH)
         IF (SIZE1 .GT. 0) THEN
*          Copy the string
            STRING(LENGTH+1:LENGTH+SIZE1) = CVALUE(1:SIZE1)
*          Update pointer
            LENGTH = LENGTH + SIZE1
         ENDIF

      ENDIF

      END
 
