*+  CHR_TERM - Terminate string by padding out with blanks.
      SUBROUTINE CHR_TERM( LENGTH, STRING )
*    Description :
*     The given string, STRING, is terminated to a length of LENGTH
*     characters by filling the remainder of its declared length
*     with blanks.
*    Invocation :
*     CALL CHR_TERM( LENGTH, STRING )
*    Arguments :
*     LENGTH = INTEGER (Given)
*           Required length for the string, must be positive
*           and not greater than the declared length of the string.
*     STRING = CHARACTER*(*) (Modified)
*           String to be terminated.
*    Method :
*     If required length is not negative then
*        Get declared length of the given string.
*        If the required length is not longer than or equal to the
*        declared length then
*           Pad to the end of the string with blanks.
*        Endif
*     Endif
*    Authors :
*     Jack Giddings. (UCL::JRG)
*     Dave Baines.   (ROE::ASOC5)
*    History :
*     03/01/1983 : Original version        (UCL::JRG)
*     28/06/1984 : Documentation increased (ROE::ASOC5)
*      2/09/1988 : Replace CHR_SIZE with LEN 
*                  Improve the Description    (RAL::AJC)
*     25/01/1990 : Eliminate redundant EXTERNAL (RAL::DLT)
*    Type Definitions :
      IMPLICIT NONE        ! switch off default typing
*    Import :
      INTEGER
     :  LENGTH             ! required length of the string
*    Import-Export :
      CHARACTER*(*)
     :  STRING             ! string to be terminated
*    Local variables :
      INTEGER
     :  SIZE               ! declared length of the string
*-

*    check that required length is not negative
      IF( LENGTH .GE. 0 ) THEN

*       get declared length of given string
         SIZE = LEN ( STRING )

*       check that required length is not greater than or equal to
*       declared length
         IF( LENGTH .LT. SIZE ) THEN

*          pad to the end of the string with blanks
            STRING( LENGTH+1:SIZE ) = ' '
         ENDIF
      ENDIF

      END
