*+  CHR_UPPER - Give upper case equivalent of a character.
      CHARACTER*1 FUNCTION CHR_UPPER( ACHAR )
*    Description :
*     If the given character is a lower case character, this
*     function returns the upper case equivalent. Otherwise the
*     given character will be returned as the function value.
*    Invocation :
*     CHARACTER = CHR_UPPER( ACHAR )
*    Arguments :
*     ACHAR = CHARACTER (Given)
*           Character for which the upper case equivalent is required.
*    Result :
*     CHR_UPPER = CHARACTER
*           Upper case equivalent of input character if input character
*           is a lower case letter; otherwise set to the input character.
*    Method :
*     Get ascii value of given character.
*     If given character is not a lower case letter then
*        Set returned value to given character.
*     Else
*        Returned value is set to upper case equivalent.
*     Endif
*    Authors :
*     Dave Baines. (ROE::ASOC5)
*    History :
*     29/06/1984 : Original version (ROE::ASOC5)
*     25/01/1990 : Use ICHAR instead of integer parameters because of
*                  DECstation compiler bugs.  (RAL::DLT)
*    Type Definitions :
      IMPLICIT NONE  ! switch off default typing
*    Import :
      CHARACTER
     :  ACHAR        ! character for which an upper case equivalent is required
*    External references :
      INTEGER ICHAR  ! ascii value of given character
      CHARACTER CHAR ! character corresponding to given ascii value
*    Local variables :
      INTEGER
     :  ASCII        ! value of given input character
*-

*    get ascii value of given character
      ASCII = ICHAR( ACHAR )

*    check for given character not a lower case letter
      IF( ASCII .LT. ICHAR('a') ) THEN

*       returned value is set to given character
         CHR_UPPER = ACHAR

      ELSEIF( ASCII .GT. ICHAR('z') ) THEN

*       returned value is set to given character
         CHR_UPPER = ACHAR

      ELSE

*       returned value is set to upper case equivalent of given character
*       add on the offset between upper and lower case characters
         CHR_UPPER = CHAR(ASCII+ICHAR('A')-ICHAR('a'))
      ENDIF

      END
