
      SUBROUTINE CBDIFF (NORDER, CBCOFF,CBDCFF, CBXMIN,CBXMAX)

!+
!     PROGRAM:      CBDIFF
!     PURPOSE:      DETERMINE THE COEFFICIENTS IN THE CHEBYSHEV-SERIES 
!                   REPRESENTATION OF THE DERIVATIVE OF A POLYNOMIAL
!                   GIVEN IN CHEBYSHEV SERIES FORM
!
!     REFERENCES:   NAG LIBRARY MARK 12 (METHOD 1)
!                   NUMERICAL RECIPES   (METHOD 2)
!
!=    Determines coefficients in the Chebyshev-series representation of the derivative of a polynomial.
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      22.02.88  C.S.JEFFERY / ST.ANDREWS
!
!
!  NORDER - Order of Chebyshev series
!  CBCOFF - Array of Chebyshev polynomial coefficients (dimension > NORD+1)
!  CBDCFF - Array of Chebyshev polynomial coefficients for derivative
!  CBXMIN - minimum value of range for Chebysehv-series representation
!  CBXMAX - maximum value of range for Chebysehv-series representation
!-

      IMPLICIT REAL    (A-H,O-Z)
      IMPLICIT INTEGER (I-N)

      INTEGER NORDER
      REAL    CBCOFF(*), CBDCFF(*), CBXMIN, CBXMAX
!.
!.....SOME USEFUL CONSTANTS
      K   = NORDER
      KP1 = K + 1
!
!  ...SPECIAL CASES
      CBDCFF(KP1) = 0.
      CBDCFF(K)   = 2*K*CBCOFF(KP1)
!  ...GENERAL CASE
      IF (KP1.GE.3) THEN
         DO 11 J = K-1,1,-1
            CBDCFF(J) = CBCOFF(J+2) + 2*J*CBCOFF(J+1)
11          CONTINUE
         ENDIF
!  ...NORMALISE TO INTERVAL XMAX-XMIN
      CC = 2. / (CBXMAX-CBXMIN)
      DO 12 J = 1,KP1
         CBDCFF(J) = CBDCFF(J) * CC
12       CONTINUE

!.....QUIT
      END
