
      SUBROUTINE FTRAN (X,Y,N, DF,KL,KH, FN,FR,FI,FF,WR,WI,WW)

!+
!=    COMPUTES THE FOURIER TRANSFORM, POWER SPECTRUM AND WINDOW TRANSFORM OF Y(X),
!     WHERE Y(X) IS A FUNCTION DEFINED BY UNEQUALLY SPACED DATA.
!
!     REF: DEEMING,\.J., 1975. ASTROPHYS. + SP. SCI. 36, 137.
!
!     INPUT:  X    ARRAY OF X VALUES
!             Y    ARRAY OF Y VALUES
!             N    NUMBER OF DATA POINTS
!             DF   FREQUENCY INTERVAL FOR TRANSFORM
!             KL   LOWER FREQUENCY LIMIT (/DF)
!             KH   UPPER FREQUENCY LIMIT (/DF)
!
!     OUTPUT: FN   ARRAY OF FREQUENCY POINTS
!             FR   REAL PART OF FOURIER TRANSFORM
!             FI   IMAGINARY PART OF FOURIER TRANSFORM
!             FF   NORMALISED AMPLITUDE OF F.T. (POWER) SPECTRUM)
!             WR   REAL PART OF WINDOW TRANSFORM
!             WI   IMAGINARY PART OF WINDOW TRANSFORM
!             WW   NORMALISED AMPLITUDE OF W.T. (POWER WINDOW)
!-

      DIMENSION X(*),Y(*),FN(*),FR(*),FI(*),FF(*),WR(*),WI(*),WW(*)
      DATA PI/3.1415926/
!.
      DO 20 K = KL,KH
         FN(K) = K * DF
         FR(K) = 0.
         FI(K) = 0.
         WR(K) = 0.
         WI(K) = 0.
         B = 2. * PI * K * DF
         DO 10 I = 1,N
            A = B * X(I)
            C = COS(A)
            S = SIN(A)
            FR(K) = FR(K) + Y(I) * C
            FI(K) = FI(K) + Y(I) * S
            WR(K) = WR(K) + C
            WI(K) = WI(K) + S
   10    CONTINUE
         FF(K) = (FR(K)*FR(K) + FI(K)*FI(K)) / (N*N)
         WW(K) = (WR(K)*WR(K) + WI(K)*WI(K)) / (N*N)
   20 CONTINUE
      END
