      SUBROUTINE GRID_LOC (G,NG,V,NV)

!+
!=    SUBROUTINE GRID_LOC EXECUTES A BRUTE FORCE SEARCH IN AN ARRAY (G)
!     OF (NG) MONOTONICALLY INCREASING VALUES FOR THE CELL CONTAINING A
!     SPECIFIED VALUE (V). THE CELL IS IDENTIFIED BY A POINTER (NV) SUCH THAT: 
!                        G(NV) <= V < G(NV+1)
!
!     THIS ALGORITHM IS OPTIMISED FOR REPEATED USE WITH A SLOWLY VARYING
!     FUNCTION V, THE PREVIOUS VALUE OF NV IS USED TO SEED THE SEARCH. 
!
!     INPUT:
!     G          ARRAY (VARIABLE LENGTH) CONTAINING GRID VALUES
!     NG         NUMBER OF VALID ENTRIES IN ARRAY G
!     V          VALUE OF VARIABLE FOR WHICH CELL ID IS REQUIRED
!     NV         SEED FOR GRID SEARCH
!
!     OUTPUT:
!     NV         INDEX FOR ARRAY CELL CONTAINING V
!
!-

      REAL, DIMENSION(*), INTENT(IN) :: G
      INTEGER, INTENT(IN)            :: NG
      REAL, INTENT(IN)               :: V
      INTEGER, INTENT(INOUT)         :: NV
!.

      NV = MIN (MAX (NV,1), NG)
      IF (V < G(1)) THEN
         NV = 0

      ELSEIF (V > G(NG)) THEN
         NV = NG 

      ELSEIF (V < G(NV)) THEN
         DO N=NV,1,-1
           IF (V > G(N)) EXIT
         ENDDO
         NV = N

      ELSEIF (V > G(NV+1)) THEN
         DO N=NV+1,NG,+1
           IF (V < G(N)) EXIT
         ENDDO
         NV = N - 1
      ENDIF

      END
