
      SUBROUTINE LINEQT (A,X,B,N,AW,BW,IR,JC,IFAIL)

!+
!     PROGRAM:     LINEQT
!     TYPE:        SUBROUTINE
!     LANGUAGE:    FORTRAN 77
!     PURPOSE:     
!=                 SOLVE A SYSTEM OF SIMULTANEOUS LINEAR EQUATIONS A.X = B
!                                                                  = -   -
!     METHOD:      GAUSSIAN ELIMINATION WITH TOTAL SCALED PIVOTTING
!     AUTHOR:      C.S.JEFFERY
!     VERSION:     II
!     PLACE:       UNIVERSITY OF ST ANDREWS
!     DATE:        1980
!     HISTORY:     VERSION I - HONEYWELL 360
!                  VERSION II - VAX 11/785
!
!     ARGUMENTS:
!        A         ARRAY CONTAINING COEFFICIENT MATRIX
!        X         VECTOR CONTAINING SOLUTION
!        B         VECTOR CONTAINING SCALAR COEFFICIENTS
!        N         DIMENSION OF ARRAYS AND NUMBER OF SIMULTANEOUS EQUATIONS
!        AW        WORK AREA ARRAY
!        BW        WORK AREA VECTOR
!        IR        WORK AREA VECTOR
!        JC        WORK AREA VECTOR
!        IFAIL     FLAG INDICATING PROGRAM STATUS
!-

      IMPLICIT REAL (A-H,O-Z)
      INTEGER IR(N), JC(N), IFAIL
      REAL A(N,N),AW(N,N),X(N),B(N),BW(N),FACT,ELMAX,SCALE
!.
!.....INITIALISE ARRAYS
      IFAIL = 0
      DO 10 I = 1,N
      IR(I) = I
      JC(I) = I
      BW(I) = B(I)
      DO 10 J = 1,N
   10 AW(I,J) = A(I,J)

!.....FOR EACH ROW OF THE MATRIX
      DO 100 K = 1,N-1

!..      SELECT PIVOT
         ELMAX = 0.
         DO 30 I = K,N
         DO 30 J = K,N
            IF (ABS(AW(IR(I),JC(J))).GT.ABS(ELMAX)) THEN
               IMAX = I
               JMAX = J
               ELMAX = AW(IR(I),JC(J))
            ENDIF
   30    CONTINUE
         IF (ABS(ELMAX).LT.1.E-10) THEN
            IFAIL = 1
            WRITE(6,'('' *** LINEQT *** BAD MATRIX INVERSION'')')
            RETURN
         ENDIF

!..      MOVE PIVOT TO TOP LEFT CORNER
         I = IR(K)
         IR(K) = IR(IMAX)
         IR(IMAX) = I
         J = JC(K)
         JC(K) = JC(JMAX)
         JC(JMAX) = J

!..      ELIMINATE KTH COEFF IN K,N EQNS
         JK = JC(K)
         IK = IR(K)
         SCALE = AW(IK,JK)
         DO 50 LI = K+1,N
            IL = IR(LI)
            FACT = AW(IL,JK) / SCALE
            AW(IL,JK) = FACT
            BW(IL) = (BW(IL) - FACT*BW(IK)) / SCALE
            DO 50 LJ = K+1,N
               JL = JC(LJ)
               AW(IL,JL) = (AW(IL,JL) - FACT*AW(IK,JL)) / SCALE
   50    CONTINUE

!..      SCALE KTH EQN TO MAKE DIAGONAL ELEMNTS = 1 (DET(AW)=1)
         BW(IK) = BW(IK) / SCALE
         DO 70 LJ = K,N
            JL = JC(LJ)
            AW(IK,JL) = AW(IK,JL) / SCALE
   70    CONTINUE
  100 CONTINUE

!.....CHECK FOR SINGULAR MATRIX
      IF (ABS(AW(IR(N),JC(N))).LT.1.E-10) THEN
         IFAIL = 1
         WRITE(6,'('' *** LINEQT *** BAD MATRIX INVERSION'')')
         RETURN
      ENDIF

!.....BACK SUBSTITUTION
      DO 150 KK = 1,N
         K = N + 1 - KK
         X(JC(K)) = BW(IR(K))
         IF (K.NE.N) THEN
            DO 140 J = K+1,N
  140       X(JC(K)) = X(JC(K)) - AW(IR(K),JC(J)) * X(JC(J))
         ENDIF
         X(JC(K)) = X(JC(K)) / AW(IR(K),JC(K))
  150 CONTINUE

!.....QUIT
      END
