
      SUBROUTINE LOC_3D (XX,NX, YY,NY, ZZ,NZ, X,Y,Z, J,K,L, IL)

!+
!=    SEARCHES FOR A CELL CONTAINING THE POINT (X,Y,Z) IN A 3D GRID
!     DEFINED BY THE ORDINATE ARRAYS XX, YY AND ZZ. THE GRID MUST
!     OF COURSE BE RECTANGULAR WITH XX, YY AND ZZ CONTAINING MONOTONICALLY
!     VARYING VALUES. THE CELL-SIZE SHOULD PREFERABLY BE NEARLY UNIFORM.
!     GIVEN THESE ASSUMPTIONS IT IS POSSIBLE TO PREDICT A NEW CELL POSITION,
!     TO ADJUST THE ARRAY INDICES APPROPRIATELY, AND TO TEST FOR SUCCESS.
!     AN ITERATIVE APPROACH SHOULD BE RAPIDLY CONVERGENT. ALL 3 ORDINATES
!     CAN BE TREATED SIMULTANEOUSLY
!
!     THE START VALUES OF THE POINTERS MAY BE VERIFIED BY SETTING IL > 0
!
!     THE TREATMENT OF THE UPPER BOUNDARY CAN BE TREATED IN TWO WAYS:
!     IF ANY ORDINAND EXCEEDS ITS CORRESPONDING DOMAIN, THEN THE CELL-
!     POINTER IS SET TO: ITS MAXIMUM VALUE     (EG NX)   IF IL = 2
!                 OR TO: MAXIMUM VALUE MINUS 1 (EG NX-1) IF IL < 2
!
!     INPUT:
!     XX, YY, ZZ     VARIABLE LENGTH ARRAYS CONTAINING ORDINATE VALUES
!     NX, NY, NZ     INTEGERS SPECIFYING LENGTH OF ORDINATE ARRAYS
!      X,  Y,  Z     3-SPACE POSITION TO BE IDENTIFIED
!     IL             FLAG INDICATING TREATMENT OF UPPER BOUNDARIES
!
!     OUTPUT:
!      J,  K,  L     3-SPACE COORDINATES OF GRID-CELL CORNER
!
!-

      REAL      XX, YY, ZZ,  X, Y, Z
      INTEGER   NX, NY, NZ,  J, K, L,  IL
      DIMENSION XX(*), YY(*), ZZ(*)
!.

    
!     FUDGE TO KEEP NUMBERLINE STRAIGHT IN A DOWNWARD SEARCH
      DATA INFTY / 10000 /
      II(A) = INT(A+INFTY) - INFTY



!     CHECK FOR LEGAL STARTING VALUES
      IF (IL.GT.0) THEN
         J = MAX (1, MIN (NX-1,J) )
         K = MAX (1, MIN (NY-1,K) )
         L = MAX (1, MIN (NZ-1,L) )
      ENDIF

!     USE PREDICT/CORRECT METHOD TO LOCATE GRID CELL
      DO 100 I = 1,10
         DJ = (X-XX(J)) / (XX(J+1)-XX(J))
         DK = (Y-YY(K)) / (YY(K+1)-YY(K))
         DL = (Z-ZZ(L)) / (ZZ(L+1)-ZZ(L))
         MJ = II(DJ)
         MK = II(DK)
         ML = II(DL)
!        TEST FOR CORRECT CELL (IE NO CHANGES REQUIRED)
         M1 = MJ*MJ + MK*MK + ML*ML
         IF (M1.EQ.0) GOTO 110
         J1 = MAX (1, MIN (NX-1,J+MJ) )
         K1 = MAX (1, MIN (NY-1,K+MK) )
         L1 = MAX (1, MIN (NZ-1,L+ML) )
!        TEST FOR CONVERGED ITERATION (IE NO CHANGES ALLOWED, EG GRID BOUNDARY)
         M2 = (J1-J)**2 + (K1-K)**2 + (L1-L)**2
         IF (M2.EQ.0) GOTO 110
         J  = J1
         K  = K1
         L  = L1
100      CONTINUE
      WRITE (6,*) ' ** LOC_3D ** NON-CONVERGENCE OF GRID-SEARCH '

!     POINT TO TOP EDGE IF X|Y|Z ABOVE RANGE
110   IF (IL.EQ.2) THEN
         IF (X.GT.XX(NX)) J = NX
         IF (Y.GT.YY(NY)) K = NY
         IF (Z.GT.ZZ(NZ)) L = NZ
      ENDIF

      END
