SUBROUTINE MTH_MEDIAN(X, MED, NX)

  !+ 
  ! Program:
  !    MTH_MEDIAN
  !
  ! Purpose:
  !=   Compute the median of values contained in a 1d array X
  !
  ! Authors:
  !    D.R.Morgan (DRM) (Armagh Observatory)
  !
  ! History:
  !    18-MAR-2005: DRM
  !       original version
  !-

  IMPLICIT NONE

  ! Subroutine Arguments
  REAL, DIMENSION(*), INTENT(IN) :: X       ! Array of input values
  INTEGER, INTENT(IN)    	 :: NX      ! Number of valid entries in X
  REAL, INTENT(OUT)              :: MED     ! Median value of X

  ! Local Arrays
  REAL, ALLOCATABLE, DIMENSION(:) :: TX      ! Work array
  INTEGER                         :: MID
  INTEGER                         :: NXD2


  !.

  !  Create temporary array
  ALLOCATE (TX(NX))
  TX = X(1:NX)

  !  Sort temporary array using subroutine sort
  CALL SORT1 (TX,NX)

  MID = INT(NX*0.5)

  !  Find middle element of sorted temporary array
  IF (MOD(NX,2) == 0) THEN
    NXD2 = NX*0.5
    MED = TX(NXD2)
  ELSE
    NXD2 = NX-1*0.5
    MED = (TX(NXD2) + TX(NXD2+1))*0.5
  END IF

!  Clean up
   DEALLOCATE (TX)

 END SUBROUTINE MTH_MEDIAN
