      SUBROUTINE PNROOT (NORDER, PNCOFF, VALUE, GUESS, ROOT)

!+
!     PROGRAM:      PNROOT
!     PURPOSE:      
!=                  FIND THE ROOT OF THE EQUATION F(X) = Y NEAREST TO X=GUESS 
!                   WHERE F(X) IS A POLYNOMIAL OF THE FORM:
!                   F(X) = A0 + A1*X + A2*X*X + ... + AN*X**N
!     METHOD:       NEWTON-RAPHSON ITERATION
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      05.02.88  C.S.JEFFERY / ST.ANDREWS
!
!     INPUT:        NORDER       ORDER OF POLYNOMIAL (EQ QUADRATIC = 2)
!                   PNCOFF       COEFFICIENTS OF POLYNOMIAL
!                   VALUE        VALUE OF FUNCTION AT WHICH X IS REQUIRED (Y)
!                   GUESS        INITIAL GUESS FOR ROOT OF F(X) = Y
!
!     OUTPUT:       ROOT         SOLUTION : F(ROOT) = Y
!
!     NOTE:         DIMENSION OF PNCOFF MUST BE .GE. NORDER+1
!-

      INTEGER       NORDER
      REAL          PNCOFF(*), VALUE, GUESS, ROOT
!  ...WORK AREA FOR DERIVATIVE OF F(X)
      REAL          DPNDX(20)
!.

!  ...GET FIRST DERIVATIVE OF THE POLYNOMIAL
      CALL PNDIFF (NORDER, PNCOFF, DPNDX)

!  ...PERFORM NEWTON-RAPHSON ITERATION
      XN = GUESS
      DO 1 I = 1,20
         CALL PNEVAL (NORDER,   PNCOFF, XN, FX)
         FX = FX - VALUE
         CALL PNEVAL (NORDER-1, DPNDX,  XN, DF)
         DX = FX / DF
         IF (ABS(DX).LE.XN*1.E-3) GO TO 2
         XN = XN - DX
1        CONTINUE

!  ...FINAL CORRECTION
2     ROOT = XN - DX

!  ...QUIT
      END
