      SUBROUTINE PNTRAN (NORDER, PNCOFFX, PNCOFFY, X0)

!+
!     PROGRAM:      PNTRAN
!     PURPOSE:      
!=                  TRANSFORM THE POLYNOMIAL F(X) IN TERMS OF F(Y),
!                   WHERE Y = X - X0 ,  AND
!                   F(X) = A0 + A1*X + A2*X*X + ... + AN*X**N
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      08.02.88  C.S.JEFFERY / ST.ANDREWS
!
!     INPUT:        NORDER       ORDER OF POLYNOMIAL (EG QUADRATIC = 2)
!                   PNCOFFX      COEFFICIENTS OF POLYNOMIAL
!                   X0           CONSTANT OF TRANSFORMATION
!
!     OUTPUT:       PNCOFFY      COEFFICIENTS OF TRANSFORMED POLYNOMIAL
!
!     NOTE:         DIMENSION OF PNCOFF MUST BE .GE. NORDER+1
!
!-
      IMPLICIT NONE
!  ...VARIABLES PASSED AS ARGUMENTS
      INTEGER       NORDER
      REAL          PNCOFFX(*), PNCOFFY(*), X0
!  ...VARIABLES USED INTERNALLY
      INTEGER       I,J,K,  II,JJ,  IERR
      REAL          COEFF, TERM
!  ...EXTERNAL FUNCTIONS
      INTEGER       NCR
!.

!  ...LOOP OVER ORDERS COLLECTING TRANSFORMED COEFFICIENTS
      DO 3 I = 1,NORDER
         II = I - 1
!     ...EVALUATE FIRST TERM IN SERIES
         COEFF = PNCOFFX(I)
!     ...COLLECT SUBSEQUENT TERMS
         DO 2 J = I+1,NORDER+1
            JJ = J - 1
            TERM = NCR (JJ,II, IERR) * PNCOFFX(J)
            DO 1 K = 1,JJ-II+1
1              TERM = TERM * X0
2           COEFF = COEFF + TERM
3        PNCOFFY(I) = COEFF

!  ...PICK UP LAST COEFFICIENT
      PNCOFFY(NORDER+1) = PNCOFFX(NORDER+1)

!  ...QUIT
      END
