
      SUBROUTINE SINELS (X,Y,ND,F, A0,A1,B1,D1,PHI, EA0,EA1,EB1,ED1,EPH,
     1                   RMS, IFAIL)

!+
!=                 SUBROUTINE SINELS FITS A SINE FUNCTION TO PERIODIC DATA
!                  WITH A GIVEN FREQUENCY USING THE METHOD OF LEAST SQUARES.
!
!                  AUTHOR.  C.S.JEFFERY          DATE.  7.MAY.1986
!
!                  INPUT:    X         ARRAY OF X - VALUES
!                            Y         ARRAY OF Y - VALUES
!                            ND        NUMBER OF DATA VALUES
!                            F         FREQUENCY OF FIT
!
!                  OUTPUT:   A0,A1,B1  COEFFICIENTS OF FIT -
!                                      A0 + A1.COS(2.PI.F.X)
!                                         + B1.SIN(2.PI.F.X)
!                            A0,D1,PHI COEFFICIENTS OF FIT -
!                                      A0 + D1.SIN(2.PI.F.X + PHI)
!                            EA0,EA1,EB1,ED1,EPH           ERRORS
!                            RMS       R.M.S. DEVIATION OF FIT
!                            IFAIL     ERROR FLAG FROM SUB. LINEQT
!-
      IMPLICIT REAL (A-H,O-Z)
      DIMENSION X(*), Y(*)
      DIMENSION S(3,3), U(3,3), T(3), C(3)
      DATA PI  /3.141592654/
      DATA PI2 /6.283185308/
!.

!.....SET COEFFICIENTS TO ZERO
        DO 200 J = 1,3
           T(J) = 0
           DO 200 K = 1,3
  200      S(K,J) = 0

!.....EVALUATE SUMMATIONS
        F2PI =  F * 6.283185305
      DO 250 L = 1,ND
         XX = F2PI * X(L)
         ALPHA = COS(XX)
         BETA  = SIN(XX)
         C(1) = 1.0
         C(2) = ALPHA
         C(3) = BETA
         DO 240 J = 1,3
            S(J,1) = S(J,1) + C(J)
            S(J,2) = S(J,2) + C(J)*ALPHA
            S(J,3) = S(J,3) + C(J)*BETA
            T(J)   = T(J)   + C(J)*Y(L)
  240    CONTINUE
  250 CONTINUE

!.....SOLVE FOR  Y = A0 + A1.COS(2PI.FX) + B1.SIN(2PI.FX)

      CALL MXINV (S,U,3,3,IFAIL)
      CALL MXSUBS (T,C)
      A0 = C(1)
      A1 = C(2)
      B1 = C(3)

!.....TRANSFORM TO  Y = A0 + D1.SIN(2PI.FX + PHI)
      D1 = SQRT (A1*A1 + B1*B1)
      TH1 = ASIN (A1/D1)
      TH2 = ACOS (B1/D1)
      PHI = ABS (TH1) / 6.283185305
      IF (TH1.GT.0.0) THEN
         IF (TH2.LT.1.57079632) THEN
            PHI = PHI
         ELSE
            PHI = 0.5 - PHI
         ENDIF
      ELSE
         IF (TH2.LT.1.57079632) THEN
            PHI = 1.0 - PHI
         ELSE
            PHI = 0.5 + PHI
         ENDIF
      ENDIF

!.....ERROR ANALYSIS
      SUM = 0
      DO 400 L = 1,ND
         FIT = A0 + A1*COS(F2PI*X(L)) + B1*SIN(F2PI*X(L))
         SUM = SUM + (Y(L)-FIT)**2
  400 CONTINUE
      RMS = SQRT ( SUM/ND )
      VRN = SUM / FLOAT(ND-3)
      EA0 = SQRT (U(1,1)*VRN)
      EA1 = SQRT (U(2,2)*VRN)
      EB1 = SQRT (U(3,3)*VRN)
      ED1 = SQRT ((EA1*A1)**2 + (EB1*B1)**2) * 2. / D1
      EPH = SQRT ((EA1*B1)**2 + (EB1*A1)**2) * 2. / D1**2
      EPH = EPH / PI2

      END
