      SUBROUTINE SMOOTH (X, Y, NPTS, FWHM, WORK)

!+
!     PROGRAM:       SMOOTH
!     PURPOSE:       
!=                   CONVOLVE Y(X) WITH GAUSSIAN GIVEN FWHM
!     AUTHOR:        C.S.JEFFERY
!     DATE:          7 AUG 87
!     LANGUAGE:      FORTRAN 77
!     VERSION:       I
!     MACHINE:       MICROVAX
!
!-
      REAL    X(*), Y(*), WORK(*)
      REAL    FWHM
      INTEGER NPTS
!.
!.....WEIGHTS FOR GAUSSIAN SMOOTHING FUNCTION
!     SIGMA  = FWHM / 2.3548
!     SIGMA2 = SIGMA ** 2
      WTGAUS (A) = EXP (-0.5 * A*A / SIGMA2 )


!.....GET SOME CONSTANTS
      SIGMA  = FWHM / 2.3548
      SIGMA2 = SIGMA * SIGMA
      SIG3   = 3.0 * SIGMA

!.....GET SOME X-LIMITS
      N  = NPTS
      X1 = X(1)
      XN = X(N)

!.....LOOP OVER X VALUES
      DO 500 I = 1,N

!.....COMPUTE THE CONVOLUTION INTEGRAL FOR EACH X-VALUE
!     ...(EXECUTED AS A (3-POINT) SUMMATION OVER +/-3*SIGMA)
         WORK(I) = 0.
         WEIGHT  = 0.
         XV = X(I)
         XL = XV - SIG3
         XH = XV + SIG3
         XL = MAX (XL,X1)
         XH = MIN (XH,XN)
         DO 200 J = 2,N-1
            XJ = X(J)
            IF (XJ.GT.XL .AND. XJ.LT.XH) THEN
               BX1 = XV - X(J-1)
               BX2 = XV - XJ
               BX3 = XV - X(J+1)
               WT1 = WTGAUS (BX1)
               WT2 = WTGAUS (BX2)
               WT3 = WTGAUS (BX3)
               BY1 = Y(J-1) * WT1
               BY2 = Y( J ) * WT2
               BY3 = Y(J+1) * WT3
               BQD = 0.25 * ((BY1+BY2)*(BX1-BX2) + (BY2+BY3)*(BX2-BX3))
               BWT = 0.25 * ((WT1+WT2)*(BX1-BX2) + (WT2+WT3)*(BX2-BX3))
               WORK(I) = WORK(I) + BQD
               WEIGHT  = WEIGHT   + BWT
            ENDIF
200      CONTINUE
         WORK(I) = WORK(I) / WEIGHT

500   CONTINUE

!.....WRITE THE SMOOTHED FUNCTION BACK INTO Y(X)
      DO 600 I = 1,N
         Y(I) = WORK(I)
600   CONTINUE

!.....QUIT
      END
