      INTEGER FUNCTION NR_JULDAY (ID,IM,IY)

!+
!  Program:
!     NR_JULDAY
!
!  Purpose:
!=    Return the Julian day number for a given date
!
!  Program type:
!     INTEGER function subprogram
!
!  Arguments given:
!     ID (Integer)   Day
!     IM (Integer)   Month
!     IY (Integer) Year
!
!  Description: 
!     This routine returns the Julian Day Number which begins at noon
!     of the calendar date specified by day ID, month IM, and year IY.
!     Positive year signifies AD, negative BC. Remember that the year
!     after 1 BC was 1 AD.
!
!  Reference:
!     Press et al. 1989. Numerical Recipes, p. 10
!
!  Authors:
!     C.S.Jeffery (CSJ) St Andrews
!
!  History:
!     original version:
!        03-JAN-1995 CSJ
!
!  Bugs:
!     This program is rubbish. It returns 1 for JD 0 (Jan 1 4713 BC)
!     and is several days in error for contempoarary dates.
!
!-

!  Gregorian calendar was adopted on Oct 15, 1582.
      PARAMETER ( IGREG = 15+31*(10+12*1582) )

!.

!  Avoid year 0 (did not exist).
      IF (IY.EQ.0) WRITE(*,*) 'nr_julday: There is no Year Zero'

!  Keep numbers straight for years BC.
      IF (IY.LT.0) IY = IY + 1

      IF (IM.GT.2) THEN
         JY = IY
         JM = IM + 1
      ELSE
         JY = IY - 1
         JM = IM + 13
      ENDIF

      NR_JULDAY = INT(365.25*JY) + INT(30.6001*JM) + ID + 1720995

!  Test for conversion to Gregorian calendar.
      IF (ID+31*(IM+12*IY).GE.IGREG) THEN
         JA = INT(0.01*JY)
         NR_JULDAY = NR_JULDAY+2-JA+INT(0.25*JA)
      ENDIF


      END

