FUNCTION NR_LOCATE (XX,X)

  !+
  !  Program:
  !     NR_LOCATE

  !  Purpose:
  !=    Find location of viven variable x in a table xx

  !  Type:
  !     Fortran90 FUNCTION subprogram

  !  Description:
  !     Given an array xx(1:N), and give a value x, returns a value j such that
  !     x is between xx(j) and xx(j+1). xx must be monotonic, either increasing or decreasing.
  !     j = 0 or j = N is returned to indicate that x is out of range.
  !
  !     Note: has same functionality, but better syntax,  as  ../mth/locate.f 

  !  Arguments given:
  !     XX (real)           
  !     X  (real)      

  !  Arguments returned:

  !  Reference:
  !     Press et al. 1999, Numerical Recipes in Fortran 90, p. 1045

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     10-NOV-2012: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  ! Numerical recipes type
  ! USE NRTYPE

  ! Safe typing
  IMPLICIT NONE

  ! Function arguments
  DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XX
  DOUBLE PRECISION, INTENT(IN) :: X

  ! Function result
  INTEGER :: NR_LOCATE

  ! Local variables
  INTEGER :: N, JL,JM,JU
  LOGICAL :: ASCND

  !.

  N = SIZE(XX)
  ASCND = ( XX(N) >= XX(1) )

  ! Initialize upper and lower limits
  JL = 0
  JU = N+1

  ! loop until N is located
  DO
     IF ( JU-JL <=1 ) EXIT 
     JM = (JU+JL)/2   ! compute a midpoint
     IF ( ASCND .EQV. (X >= XX(JM)) ) THEN ! replace eiher upper or lower limit
        JL = JM
     ELSE 
        JU = JM
     ENDIF
  ENDDO

  ! Set output, take care with endpoints
  IF ( X == XX(1) ) THEN
     NR_LOCATE = 1
  ELSEIF ( X == XX(N) ) THEN
     NR_LOCATE = N-1
  ELSE
     NR_LOCATE = JL
  ENDIF

END FUNCTION NR_LOCATE

