SUBROUTINE NR_D_HEAPSORT ( N, M, K, A )

  !+
  !=    Sorts an array A(N,M) into ascenidng numerical order on A(1:N,K)
  !     while maintaining the rows in the same correspondece.
  !-

  IMPLICIT NONE


  !     Arguments:
  !     

  INTEGER, INTENT(IN) ::  N ! 1st dimension of A  (sorted)
  INTEGER, INTENT(IN) ::  M ! 2nd dimension of A 
  INTEGER, INTENT(IN) ::  K ! Index of A on which to sort 
  DOUBLE PRECISION, DIMENSION (N,M), INTENT(INOUT) :: A ! Array to be sorted


  !     Local Varaibles:
  !     
  DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: R  ! Temporary row holder
  INTEGER :: I, J, L, IR   ! Local pointers

  !.

  !     Set up storage
  ALLOCATE ( R(M) )


  !     The index L will be decremented from its initial value down to 1 
  !     during the "hiring" (heap creation) phase. Once it reaches 1, the 
  !     index IR will be decremented from its iunitial value down to 1 during 
  !     the "retiremnet-and-promotion" (heap selection) phase. 
  L = N/2 + 1
  IR = N

  DO 

    !     In hiring phase
    IF ( L > 1 ) THEN
      L = L - 1
      R = A(L,:)

      !     In retirement and promotion phase. 
    ELSE
      R = A(IR,:)           ! Clear a space at end of array
      A(IR,:) = R           ! Retire the top of heap into it
      IR = IR - 1           ! Decrease the size of the corporation.
      IF (IR == 1 ) THEN    ! Done with the last promotion
        A(1,:) = R          ! The least competent worker of all!
        EXIT
      ENDIF
    ENDIF
    ! Whether in the hiring or promotion phase, 
    ! we here set up to sift down element R 
    ! to its proper level.
    I = L
    J = L + L

    DO WHILE (J <= IR )
      IF ( J < IR ) THEN
        !     Compare to the better underling
        IF ( A(J,K) < A(J+1,K) ) J = J + 1 
      ENDIF
      IF ( R(1) < A(J,K) ) THEN ! Demote R
        A(I,:) = A(J,:)
        I = J
        J = J + J
      ELSE                  ! This is R's level. Set J to terminate sift-down
        J = IR + 1
      ENDIF
    ENDDO

    A(I,:) = R              ! Put R into its slot. 

  ENDDO

END SUBROUTINE NR_D_HEAPSORT

