      SUBROUTINE NR_D_SPLIE2 (X1A,X2A,YA,M,N,Y2A)

!+
!
!  NR_D_SPLIE2
!
!     (double precision version)
!
!     Purpose:
!=    Construct 1D cubic spline and derivatives of tabulated data.
!
!     Description:
!     Given an M by N tabulated function YA, and tabulated independent
!     variables X1A (M vlaues) and X2A (N values), this routine constructs
!     one-dimensiona natural cubic splines of the rows of YA and returns the
!     second derivatives in the array Y2A.
!
!  Press et al. 1989. Numerical Recipes
!
!  Simon Jeffery 
!  2000 February
!
!-

!  Subroutine Arguments.
      DOUBLE PRECISION X1A(M)    ! tabulated indebendent variable 1 (given)
      DOUBLE PRECISION X2A(N)    ! tabulated independent variable 2 (given)
      DOUBLE PRECISION YA(M,N)   ! tabulated function               (given) 
                              ! and spline coefficients       (returned)
      DOUBLE PRECISION Y2A(M,N)  ! second derivatives            (returned)
      INTEGER  M, N

!  Local arrays.
      PARAMETER (NN=100)
      DOUBLE PRECISION YTMP(NN),Y2TMP(NN)

!.

      DO J=1,M
        DO K=1,N
          YTMP(K)=YA(J,K)
        ENDDO
        CALL NR_D_SPLINE (X2A,YTMP,N,1.E30,1.E30,Y2TMP)
        DO K=1,N
          Y2A(J,K)=Y2TMP(K)
        ENDDO
      ENDDO

      END
