SUBROUTINE OP_BFIN

  !+
  !
  !  Name:
  !  OP_BFIN
  !
  !  Purpose: 
  != Reads in Opacity Project data for continuous opacities.
  !  a modified version of op_bfopin.f90
  !
  !  History:
  !     21-Mar-97: Created (PMH)
  !     12-Feb-98: modified for Sterne 2 (PMH)
  !     02-Mar-99: cleaned up (CSJ)
  !     05-Nov-03: modified (NTB)
  !      1-APR-20: added Zr,Y,Sr  (CSJ)
  !     21-Oct-20: added Ge  (CSJ)
  !     25-Nov-20: Revision commenced
  !
  !
  !-

  ! Global variables
  USE LTE_WLGRID
  USE OP_PRECISIONS
  USE OP_GLOBAL
  USE,INTRINSIC :: IEEE_ARITHMETIC

  IMPLICIT NONE 

  DOUBLE PRECISION, EXTERNAL :: TAP_IONPT
  
  INTEGER :: I, J, K, IX, N

  !  Local variables.
  INTEGER :: NSC, ISC, ISCX              !  no of levels in ion (+counters)
  INTEGER :: NZ                          !  no of protons in atom
  INTEGER :: NE                          !  no of electron in target ion
  INTEGER :: ISLP, IS, IL, IP            !  state identifier (slp,s,l,p)
  INTEGER :: ILV, NS                     !  state identifier (n)
  REAL    :: XCHIC                       !  energy of level (computed)

  INTEGER :: NXC                         !  no of wavelengths in level (+counters)
  REAL    :: OPXWL(OP_MAXNX)      !  OP x-s wavelengths (A)
  REAL    :: OPXXS(OP_MAXNX)      !  OP x-s (cm^2)

  REAL    :: XWT,XCHI
  REAL    :: XWL, XS, XSQ(OP_MAXNW)
  REAL    :: XCHIG             !  energy of ground state wrt continuum
  REAL    :: XWLIM             ! inisation limit for level (in A)
  REAL    :: ACC               ! internal accuracy of calculation (ignore)

  INTEGER :: ALPK                      !  atomic number
  INTEGER :: ALPI                      !  electron number on ion
  CHARACTER(LEN=4) :: INSLP            !  ILV + ISLP
  
  INTEGER :: NQM1 ! is the running state offset counter usde to store in OP_NQ

  !           IONLIMIT controls lower energy limit used for cross-sections
  !           1 = use ionisation limit, 0 = use cross-section data limit
  REAL    :: IONLIMIT=1.0
   
  INTEGER :: N1                        ! dummy holder 
  
  !! File name holders and unit numbers
  ! locator for OP data
  CHARACTER(LEN=7), PARAMETER :: PATH_OP = 'OPDATA/'       
  CHARACTER(LEN=7), PARAMETER :: PATH_FE = 'FEDATA/'       
  CHARACTER(LEN=2) :: ION              ! used to construct filename
  CHARACTER(LEN=2) :: ATOM             ! used to construct filename
  ! filenames
  CHARACTER(LEN=33) :: PFILE           !  file holding energy level data
  CHARACTER(LEN=33) :: XFILE           !  file holding x-section data
  CHARACTER(LEN=33) :: FEFILE          !  file holding NORAD format x-sections
  CHARACTER(LEN=2)  :: OPFORM          !  flag for data format 'OP' or 'FE'
  ! CHARACTER(LEN=37) :: XSOUT         !  file for transformed x-sections
  !                                    !  not currently used, but envisage building a binary for all the OP x-sections
  ! unit numbers for files
  INTEGER :: PF=12, XF=13
  ! status flags for files
  INTEGER :: PFEXT, XFEXT

  
  !  Local constants.
  REAL(KIND=QQ), PARAMETER :: RINV = 911.2670505D0 ! 1/R (A)
  REAL(KIND=QQ), PARAMETER :: REV = 13.605693122D0  ! 1 Ryd (eV)



  ! Store the wavelength array for the OP xsections
  OP_NW = LL
  ALLOCATE(OP_WL(OP_NW))
  ALLOCATE(OP_NU(OP_NW))
  OP_WL(1:LL) = WL(1:LL)

  ! Announce programme
  WRITE (*,*)
  WRITE (*,*) 'OP_BFIN:'
  WRITE (*,'(A4,I5,A3,5F7.1,A3,3F8.1)') &
       ' nw ',op_nw,' wl ',op_wl(1:5),'...',op_wl(ll-2:ll)
  WRITE (*,*) 'nz ne nx  chi_I  chi_0 chi_nx ( slp n,... ., ' 

  OP_XSQ = 0.0
  NQM1 = 0  ! zero the state offset counter
  
  ! Set up array of selected atoms to be used.
  ! Array used for indirect referencing of OP arrays. 
  !
  ! Iron Project data for TiI, CrI, and NiII are available,
  ! and can be switched on by adding those atoms here.
  !
  OP_NATOMS = 22
  OP_SATOMS(1:OP_NATOMS) = (/ 1,2,3,4,5,6,7,8,9,10,11,12,13,14, &
                              16, 18, 20, 26, 32, 38,39,40 /)

  !  Loop over atoms
  atoms: DO OP_IAS = 1,OP_NATOMS
     K    = OP_SATOMS(OP_IAS)       ! pick up atomic number from OP_SATOMS.   
     ALPK = OP_ALPZ(K)              ! atomic number of current atom - should = k !?!

     IF (OP_ALPIMIN(k) == 0) CYCLE atoms  ! skip read if no data for this element

     ! Loop over ions
     ions: DO J = OP_ALPIMIN(K),OP_ALPIMAX(K)
        ALPI = ALPK-J+1              ! ion no. (j) -> elec. no.
        OPFORM = 'OP'                ! default opacity type

        !!  Open files to be read for current atom/ion

        ! Select Iron project data files
        ! We have used those files in which the BE is giving as +ve relative to ground state
        ! In some cases (ti01,ni02) we have used the .ptpx files instead of the .px files
        ! - having renamed them
        ! For '.px' files the levels are not sorted by energy, but by configuration, so lowest levels are
        ! not all in the first 50. We will need an algorithm to select. 
        ! 2020.xii.1
        ! For '.pxx' files, this has been corrected and levels are sorted by energy, but only for Fe. 
        !
        IF ( K == 22 .AND. J == 1 ) THEN  ! Ti I
           OPFORM = 'FE'
           WRITE (ION, '(I2.2)') J
           FEFILE = PATH_FE // 'ti' // ION // '.pxx'

        ELSEIF ( K == 24 .AND. J == 1 ) THEN  ! Cr I
           OPFORM = 'FE'
           WRITE (ION, '(I2.2)') J
           FEFILE = PATH_FE // 'cr' // ION // '.pxx'

        ELSEIF ( K == 26 .AND. J < 6 ) THEN   ! Fe I - V
           OPFORM = 'FE'
           WRITE (ION, '(I2.2)') J
           FEFILE = PATH_FE // 'fe' // ION // '.pxx'

        ELSEIF ( K == 28 .AND. J == 2 ) THEN  ! Ni II
           OPFORM = 'FE'
           WRITE (ION, '(I2.2)') J
           FEFILE = PATH_FE // 'ni' // ION // '.pxx'
        ENDIF

        ! Open FE opacity files
        IF ( OPFORM == 'FE' ) THEN

           PFEXT = 0 
           OPEN(UNIT=XF,FILE=FEFILE,STATUS='OLD',IOSTAT=PFEXT)
           IF (PFEXT > 0) CYCLE ions
           CALL SKIP_HEADER (XF,'--------------------')
           READ (XF,*) NZ, NE, NS
           !IF (NS > 0) OPFORM = 'F2'
           !CALL SKIP_TARGET (XF,NS)
 
       
        ! Open OP opacity files   
        ELSE
           WRITE (ATOM,'(I2.2)') ALPK
           WRITE (ION, '(I2.2)') ALPI
           PFILE = PATH_OP // 'p' // ATOM // ION // '.op'
           XFILE = PATH_OP // 'x' // ATOM // ION // '.op'
           PFEXT = 0 ; XFEXT = 0
           OPEN (unit=PF,file=PFILE,status='OLD',IOSTAT=PFEXT)
           IF (PFEXT > 0) CYCLE ions
           OPEN (unit=XF,file=XFILE,status='OLD',IOSTAT=XFEXT)
        ENDIF

        !  Save position in x-section arrays
        NSC=0
        XWLIM = 0.D0
        OP_NQ(J:OP_MAXNI,K)            = NQM1  ! Set the state offset counters
        OP_NQ(1:OP_MAXNI,K+1:OP_MAXNA) = NQM1  ! Fill up array to avoid gaps

        ! reset energy level
        ISLP = 0; IS = 0; IL = 0; IP = 0; ILV = 0
        
        ! loop over states
        state: DO ISC = 1,OP_MAXNS+1
           
              
           ! Read the energy level data (one per line)
           !! reads in slp, ilv, statistical weight and ionization potential(ryd)
           !IF ( OPFORM      == 'F2' ) READ (XF,*) NZ, NE, NS
           !IF ( OPFORM      == 'F2' ) CALL SKIP_TARGET (XF,NS)
           IF ( OPFORM == 'OP' ) READ (PF,*) ISLP,ILV,XWT,XCHI
           IF ( OPFORM == 'FE'  ) READ (XF,*) IS, IL, IP, ILV
           IF ( OPFORM == 'FE'  ) ISLP =  100*IS + 10*IL + IP
           
           ! Trap end of file or x-section limit
           IF ( ((ISLP == 0) .AND. (IS+IL+IP+ILV == 0)) .OR. (NSC == OP_MAXNS) ) THEN  
              OP_NS(j,k) = NSC            ! store the number of x-sections for this ion
              NQM1 = OP_NQ(j,k) + NSC     ! increment offset counter for next level by number of x-sections
              EXIT state                  ! go to next ion   
           ENDIF
           
           ! Augment level counter
           NSC=NSC+1
           
           ! Read in state data from the cross-section file (format dependent) 
           IF ( OPFORM == 'OP' ) READ (XF,*) ISCX, NZ, NE, ISLP, ILV, XCHIC, NXC 
           IF ( OPFORM == 'FE' ) READ (XF,*) N1, NXC
           IF ( OPFORM == 'FE' ) READ (XF,*) XCHIC, ACC
           !IF ( OPFORM == 'F2' ) READ (XF,*) XCHIC, NXC
           !IF ( OPFORM == 'F2' ) READ (XF,*) ACC

           !! Level identifier
           OP_ISLP(NSC,J,K) = ISLP
           OP_ILV (NSC,J,K) = ILV
           
           !! Statistical weight
           IF ( OPFORM == 'FE' ) XWT = IS*((2*IL) +1)     ! stat. weight
           OP_XWT(NSC,j,k) = XWT    
           
           !! Ionisation limit (convert from Rydberg to Angstrom)
           IF ( OPFORM == 'FE' ) XCHI = XCHIC     ! Use theoretical energy for FE data 
           XWLIM = - RINV / XCHI * IONLIMIT       !  ionisation limit (in Angstrom)
           ! Store ionisation limit from input file unless over-ride applies
           IF ( OP_XWLIM(NSC,J,K) < 0.01 .OR. IEEE_IS_NAN(OP_XWLIM(NSC,J,K)) ) OP_XWLIM(NSC,J,K) = XWLIM
           
           ! Excitation potentials  (convert from Rydberg to eV)
           ! 
           ! OP give all energies as -ve relative to continuum, hence these are -ve binding energies. 
           IF ( OPFORM == 'OP' ) XCHI = -XCHI
           
           ! For OP store the ground-state as the ionization energy but don't overwrite FEDATA ground states !
           IF (NSC == 1 .AND. OPFORM == 'OP') OP_XCHIG(J,K)  = XCHI            
           ! IRON energies are given as Binding energies (+ve) : pre-tabulate  ionization energies (in OP_GLOBAL)
           OP_XCHI(NSC,j,k) = (OP_XCHIG(J,K) - XCHI) * REV  
           
           ! Last level / EOF detected
           IF (NXC == 0) CYCLE state
           
           ! Read the x-section
           WRITE (INSLP,'(I1,I3)') ILV,ISLP      ! Why are we doing this ? book keeping ?
           DO IX = NXC, 1, -1                    ! loop over wavelengths
              READ (XF,*) XWL, XS
              OPXWL(IX) = RINV/XWL               ! Rydberg -> Angstrom
              OPXXS(IX) = 1.e-18*XS              ! Mbarns -> cm^2
           ENDDO
           
           !! taking into account the duplicate wavelengths 
           !! (shift wavelengths slightly so no overlapping points with different xsec)
           DO IX = 1,NXC-1
              IF (ABS(OPXWL(IX+1)-OPXWL(IX)) .LT. 1.D-35) THEN
                 IF (OPXXS(IX+1) .LT. 1.D-35) THEN
                    OPXWL(IX+1)=OPXWL(IX+1)*1.00001
                 ELSEIF (OPXXS(IX) .LT. 1.D-35) THEN
                    OPXWL(IX)  =OPXWL(IX)  *0.999995
                 ELSE
                    OPXWL(IX)  =OPXWL(IX)  *0.999995
                    OPXWL(IX+1)=OPXWL(IX+1)*1.000005
                 ENDIF
              ENDIF
           ENDDO

           
           !           Find average OP cross-section value across 
           !           Sterne wavelength interval
           IF (NXC > 0) CALL OP_XSEC (NXC,OPXWL,OPXXS,OP_XWLIM(NSC,J,K),LL,WL,XSQ)
           OP_XSQ(1:LL,OP_Q(ISC,J,K)) = XSQ(1:LL)
           !write (*,*) wl
           !write (*,*) opxwl(1:nxc),opxxs(1:nxc)
           !write (*,*) isc,j,k,op_xsq(:,isc,j,k)
           !stop
 
        ENDDO state                           ! end state loop


        
        ! Tidy up the data files
        IF (OPFORM == 'OP') CLOSE(PF)
        CLOSE(XF)

        ! Report x-sections for each ion
        IF (NSC > 0 ) &
             WRITE (*,'(3I3,3F7.2,2X,11(1X,I3,I2))') K, (K-J+1), NSC, &
             OP_XCHIG(J,K)*REV, OP_XCHI(1,J,K), OP_XCHI(NSC,J,K), &
             (OP_ISLP(I,J,K),OP_ILV(I,J,K),I=1,MIN(10,NSC-1)), &
              OP_ISLP(NSC,J,K),OP_ILV(NSC,J,K)
         
     ENDDO ions                                  ! end ion loop

  
  ENDDO atoms                                    ! end atom loop


  ! report total number of x-sections
  WRITE (*,'(A4,I5)') ' nq ',nqm1+nsc
  FLUSH(6)
  
  !  Interpolate x-s at reference wavelength for opt.depth scale
  CALL OP_INXS 


CONTAINS

  SUBROUTINE SKIP_HEADER ( UNIT, STRING )

    ! Read opening lines of input file 'UNIT'
    ! until a line matching 'STRING' is found
    
    INTEGER           :: UNIT
    CHARACTER*(*)     :: STRING

    INTEGER           :: NCH
    CHARACTER(LEN=20) :: TEST

    NCH = LEN(STRING)
    DO
       READ (UNIT,*) TEST
       IF (TEST(1:NCH) == STRING) EXIT
    ENDDO

  END SUBROUTINE SKIP_HEADER


  SUBROUTINE SKIP_TARGET ( UNIT, NS )

    ! FEDATA type 2 files have NS target energies
    ! listed against each x-section at 5 energies per line
    ! Need to skip over these
    
    INTEGER           :: UNIT
    INTEGER           :: NS

    IF (NS > 0) THEN
       DO I = 1,CEILING(FLOAT(NS)/5.0)
          READ (UNIT,*)
       ENDDO
    ENDIF

  END SUBROUTINE SKIP_TARGET
  
END SUBROUTINE OP_BFIN
