      SUBROUTINE BLAZE (NW,MO, LL,MM,RR)

!+
!     PROGRAM:      BLAZE
!     PURPOSE:      
!=                  COMPUTE BLAZE FUNCTION FOR SPECIFIED ORDER AND WAVELENGTH
!
!     NOTES:    (1) IBLTYP    : DEFINES THE TYPE OF BLAZE FUNCTION TO BE USED.
!                   IBLTYP = 1: A SINC**2 FUNCTION
!                   IBLTYP = 2: 9TH ORDER CHEBYSHEV POLYNOMIALS
!               (2) RR IS A NUMBER BETWEEN 0 AND 1. RECTIFICATION FOR DETECTOR
!                   RESPONSE AND STELLAR CONTINUUM SLOPE IS CONSIDERED TO BE A
!                   SEPARATE FUNCTION
!               (3) IF IBLTYP IS DEFINED ... THIS ROUTINE ASSUMES THAT VALID
!                   DATA EXIST IN THE DEFINITION OF THE BLAZE FUNCTION /BLDECK/
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      12.01.88  C.S.JEFFERY / ST.ANDREWS
!     >> ASSUMES SINC*SINC FUNCTION FOR BLAZE
!     2.0           microVAX      03.02.88  C.S.JEFFERY / ST.ANDREWS
!     >> ALLOWS 9TH ORDER POLYNOMIAL FOR EACH ORDER
!     2.1           microVAX      29.03.88  C.S.JEFFERY / ST.ANDREWS
!     >> CORRECTION TO ABOVE FOR CONTINUUM OF STANDARD STAR
!        REMOVED 14.04.88
!        SPECTRA ARE ASSUMED TO BE NORMALISED PRIOR TO BLAZE CORRECTION
!     2.2           microVAX      29.03.88  C.S.JEFFERY / ST.ANDREWS
!     >> ADDITION TO ALLOW DERAIVATIVES TO BE CALCULATED BY DBLDX
!
!     INPUT:        NW      WAVELENGTH POINTER
!                   MO      ORDER POINTER
!
!     OUTPUT:       MM      ORDER NUMBER
!                   LL      WAVELENGTH
!                   RR      BLAZE FUNCTION
!-

      INCLUDE 'implct.f'
      INCLUDE 'ecdeck.f'
      INCLUDE 'blazer.f'
!.

!     TRANSLATE INPUT
      M = MO
      N = NW

!     DETERMINE ORDER NUMBER (M)
      OM     = MECORD(M)
!     IDENTIFY WAVELENGTH (LAMDA)
      WL     = ECSLAM(M) + NW*ECDLAM(M)

!     SINC**2 BLAZE FUNCTION
      IF (IBLTYP.EQ.1) THEN

!     ...SPECIFY R(LAMDA,M)
         XX     = OM * (1.0 - BLWCEN(M) / WL)
         YY     = PI * BLA(M) * XX
!     ...AVOID SINGULARITY FOR LAMC(M) = WL
         IF (ABS(YY).GT.1.E-3) THEN
            RB     = SIN (YY) ** 2 / YY ** 2
         ELSE
            RB     = 1.0
         ENDIF

!     9TH ORDER CHEBYSHEV POLYNOMIALS
      ELSEIF (IBLTYP.EQ.2) THEN

!        BLAZE UNDEFINED FOR THIS ORDER, SET BLAZE = 1
         IF (.NOT.LBLF(M)) THEN
            RB = 1.0
            GOTO 990
         ENDIF

!        IF THIS IS A NEW ORDER ...
         IF (M.NE.MSAVE) THEN
!           GET CHEBYSHEV COEFFICENTS
            CALL MXRGET (BLCHEB, CBCOFF, 10,32, M)
            CBXMIN = BLXMIN(M)
            CBXMAX = BLXMAX(M)
            CBWCEN = BLWCEN(M)
!           GET DERIVATIVE (IN CASE ITS NEEDED)
            CALL CBDIFF (9,CBCOFF,CBDCFF,CBXMIN,CBXMAX)
            MSAVE = M
         ENDIF

!        EVALUATE BLAZE FUNCTION
         XX = WL - CBWCEN
         CALL CBEVAL (9,CBCOFF,CBXMIN,CBXMAX, XX, RB)

!     NO OTHER BLAZE FUNCTION TYPES ARE CURRENTLY SUPPORTED

      ELSE
         RB = 1.0
      ENDIF

!     SET OUTPUT ARGUMENTS

990   MM = OM
      LL = WL
      RR = RB
      END
