      SUBROUTINE BLEDIT

!+
!     PROGRAM:      BLEDIT
!     PURPOSE:      
!=                  ADJUST THE BLAZE FUNCTION MANUALLY
!     REFERENCES:   
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      11.03.88  C.S.JEFFERY / ST.ANDREWS
!-

      INCLUDE 'implct.f'
      INCLUDE 'ecdeck.f'
      INCLUDE 'cadeck.f'
      INCLUDE 'spdeck.f'


!     CONTROL CODES
      CHARACTER*1   CCODE

!     ORDER TRACKING
      INTEGER       NORDER
      INTEGER       NORDLO
      INTEGER       NORDHI

!     ORDER CHARACTERISTICS
      REAL          SL
      REAL          DL
      INTEGER       NL

!     LOCAL BLAZE FUNCTION ARRAYS
      REAL          CBCOFF(10)
      REAL          EBLCF(10,32)
      REAL          EBLWC(32)
      REAL          EBLFM(32)
      REAL          EBLXM(32)
      REAL          EBLXP(32)
      LOGICAL       LEBLF(32)

!     PLOT PARAMETERS (DO NOT USE IF DECK 'DISPGX' IS ACTIVATED)
      CHARACTER*3   CHC
      CHARACTER*79  HEADER, XLABEL, YLABEL
      REAL          DXMIN, DXMAX, DYMIN, DYMAX

!     EXTERNAL FUNCTIONS USED
      CHARACTER*20  CHFRMI
!.


!     VERIFY GRAPHICS DEVICE IS SET UP

      CALL GX_OK (IOK)
      IF (IOK.EQ.0) THEN
         CALL MESSGE ('BLEDIT: * Error * No graphics device')
         RETURN
      ENDIF

!     SET UP START VALUES REQUIRED BY THIS ROUTINE ONLY

!     SET UP THE ORDER MAPPING
      NORDLO = MIN(MECORD(1),MECORD(NECORD))
      NORDHI = MAX(MECORD(1),MECORD(NECORD))
      NORDER = NORDLO

!     ZERO LOCAL FITTING ARRAYS
      CALL MXZERO (EBLCF, 10,32)
      DO 10 N = 1,NECORD
         EBLWC(N) = 0.
         EBLFM(N) = 0.
         EBLXM(N) = 0.
         EBLXP(N) = 0.
10       LEBLF(N) = LF

!     COPY EXISTING FIT INTO LOCAL WORK AREA
      DO 20 N = 1,NECORD
         IF (LBLF(N)) THEN
            DO 11 K = 1,10
11             EBLCF(K,N) = BLCHEB(K,N)
            EBLWC(N) = BLWCEN(N)
            EBLFM(N) = BLFMAX(N)
            EBLXM(N) = BLXMIN(N)
            EBLXP(N) = BLXMAX(N)
            LEBLF(N) = LT
         ENDIF
20       CONTINUE

!     WARN USER ABOUT LIVE KEY-BOARD
      CALL IOHEAD ('Blaze function polynomial editting')
      CALL KEYWRN
      CALL IOFOOT


!     SELECT ORDER FOR FIT

100   CALL MESSGE (' <')
      CALL KEYCHK (CCODE,'=.+-EHQ')

!     HELP
      IF (CCODE.EQ.'H') THEN
         CALL MESSGE (' Select echelle order / plotting mode')
         CALL MESSGE ('   =      Specify order by number')
         CALL MESSGE ('   +      Next order')
         CALL MESSGE ('   -      Back to previous order')
         CALL MESSGE ('   .      Repeat current order')
         CALL MESSGE ('   E      Exit')
         CALL MESSGE ('   Q      Quit')
         CALL MESSGE ('   H      Help')

!     EXIT
      ELSEIF (CCODE.EQ.'E') THEN
         GOTO 900

!     ABORT
      ELSEIF (CCODE.EQ.'Q') THEN
         GOTO 999

!     SPECIFY ORDER BY NUMBER
      ELSEIF (CCODE.EQ.'=') THEN
         CALL GETINT ('Specify order number', NORDER)
         GOTO 150

!     PROCEED TO NEXT ORDER
      ELSEIF (CCODE.EQ.'+') THEN
         NORDER = NORDER + 1
         NORDER = MAX (NORDLO,NORDER)
         GOTO 150

!     REPEAT CURRENT ORDER
      ELSEIF (CCODE.EQ.'.') THEN
         GOTO 150

!     GO BACK TO PREVIOUS ORDER
      ELSEIF (CCODE.EQ.'-') THEN
         NORDER = NORDER - 1
         NORDER = MIN (NORDER,NORDHI)
         GOTO 150

!     NO MORE OPTIONS
      ENDIF
      GOTO 100


!     CHECK ORDER EXISTS BEFORE DOING ANYTHING SILLY

150   CONTINUE
      IF (NORDER.LT.NORDLO .OR. NORDER.GT.NORDHI) THEN
         CALL MESSGE ('* WARNING * nonexistent order')
         CALL OUTINT ('Limits for order number are', NORDLO)
         CALL OUTINT ('and',                         NORDHI)
         GOTO 100
      ENDIF

!     GET ORDER POINTER FROM ORDER NUMBER
      J = ECXORD (NORDER) 

!     COLLECT THE POLYNOMIAL COEFFICIENTS FOR THIS ORDER
155   WCEN = EBLWC(J)
      FMAX = EBLFM(J)
      XMIN = EBLXM(J)
      XMAX = EBLXP(J)
      DO 157 K = 1,10
157      CBCOFF(K) = EBLCF(K,J)

!     COPY THE ORDER INTO THE SPECTRUM ARRAYS
160   SL   = ECSLAM(J)
      DL   = ECDLAM(J)
      NL   = NECWAV(J)
      DO 161 I = 1,NL
         SPW(I) = SL + I * DL
         SPF(I) = ECSTAK(I,J)
161      CONTINUE
      NSPW = NL
      SPHEAD(1) = ECHEAD(1:10) // ' : Order ' // CHFRMI (NORDER, 3)

!     GET ARRAY LIMITS
      SPWMIN = SPW(1)
      SPWMAX = SPW(NSPW)
!        FLUX LIMITS DERIVED AVOIDING END OF ORDER EFFECTS
      CALL MINMAX (SPF, 10,NSPW-10, SPFMIN,SPFMAX)

!     NOW CALCULATE SOME DISPLAY PARAMETERS

!     THE X-RANGE
      XLABEL = 'Wavelength (A)'
      DXMIN  = SL +      DL
      DXMAX  = SL + NL * DL
      XRANGE = DXMAX - DXMIN

!     THE Y-RANGE
      YLABEL = 'Flux (counts)'
      DYMIN  = SPFMIN - (SPFMAX-SPFMIN) / 10.
      DYMAX  = SPFMAX + (SPFMAX-SPFMIN) / 10.
      YRANGE = DYMAX - DYMIN

!     THE HEADER
      CHC = CHFRMI (NORDER,3)
      HEADER = ECHEAD (1:10) // ' : Order ' // CHC

!     PLOT THE FRAME
      CALL GX_CLR
      CALL GX_RSPC ( 0.2,   0.1,   0.8,   0.9 )
      CALL GX_QSPC (DXMIN, DYMIN, DXMAX, DYMAX)
      CALL GX_AXES (HEADER, XLABEL, YLABEL)

!     PLOT THE CURRENT ORDER
!     GROSS SPECTRUM + BLAZE FUNCTION
      CALL GX_QPLT (NSPW, SPW, SPF)
!     NOW COPY BLAZE FUNCTION, SKIPPING BY 10
170   N = 0
      ILO = (XMIN + WCEN - SL) / DL
      IHI = (XMAX + WCEN - SL) / DL
      DO 171 I = ILO,IHI,10
!        COMPUTE BLAZE FUNCTION
         X = SL + I * DL
         CALL CBEVAL (9,CBCOFF, XMIN,XMAX, X-WCEN, R)
!        STORE BLAZE FUNCTION
         N = N + 1
         SPW(N) = X
         SPF(N) = R * FMAX
171      CONTINUE
!     PLOT THE BLAZE FUNCTION
      NSPW = N
      CALL GSPLI (2)
      CALL GX_QPLT (NSPW, SPW, SPF)
!
!     PLOT WCEN, FMAX AND THE FIT LIMITS
      CALL GSPLI (4)
      XINC = XRANGE / 10.
      YINC = YRANGE / 10.
!        WCEN
      CALL GX_QMOV (WCEN, DYMIN + 2.*YINC)
      CALL GX_QDRW (WCEN, DYMIN + 3.5*YINC)
      CALL GX_QMOV (WCEN-1., DYMIN + 3.*YINC)
      CALL GX_QDRW (WCEN, DYMIN + 3.5*YINC)
      CALL GX_QDRW (WCEN+1., DYMIN + 3.*YINC)
!        FMAX
      CALL GX_QMOV (DXMIN + 2.*XINC, FMAX)
      CALL GX_QDRW (DXMIN + 3.*XINC, FMAX)
      CALL GX_QMOV (DXMAX - 3.*XINC, FMAX)
      CALL GX_QDRW (DXMAX - 2.*XINC, FMAX)
!        XMIN/XMAX
      CALL GX_QMOV (XMIN+WCEN, DYMIN+2.5*YINC)
      CALL GX_QDRW (XMIN+WCEN, DYMIN+YINC)
      CALL GX_QDRW (XMAX+WCEN, DYMIN+YINC)
      CALL GX_QDRW (XMAX+WCEN, DYMIN+2.5*YINC)
      CALL GSPLI (1)

!     I THINK WE'VE PLOTTED EVERYTHING
      CALL GX_FLSH
      GOTO 200


!     NOW EDIT THE FIT

!     GET THE KEYSTROKE COMMAND
200   CALL MESSGE (' <+')
      CALL KEYCHK (CCODE,'CEHPQRTW')
201   CONTINUE

!     HELP
      IF (CCODE.EQ.'H') THEN
         CALL MESSGE (' Edit the fit manually ')
         CALL MESSGE ('   C    Change continuum slope (a0,a1)  ')
         CALL MESSGE ('   W    Change Wcen ')
!        CALL MESSGE ('   F    Change Fmax ')
         CALL MESSGE ('   T    Trim range ')
         CALL MESSGE ('   P    Replot data and fit ')
         CALL MESSGE ('   R    Recover start values')
         CALL MESSGE ('   E    Exit ')
         CALL MESSGE ('   Q    Quit ')
         CALL MESSGE ('   H    Help ')

!     QUIT
      ELSEIF (CCODE.EQ.'Q') THEN
         CALL MESSGE ('Blaze function coefficients not changed')
         GOTO 100

!     EXIT
      ELSEIF (CCODE.EQ.'E') THEN
         EBLWC(J)   = WCEN
         EBLFM(J)   = FMAX
         EBLXM(J)   = XMIN
         EBLXP(J)   = XMAX
         DO 205 K = 1,10
205         EBLCF(K,J) = CBCOFF(K)
         GOTO 100

!     REDRAW
      ELSEIF (CCODE.EQ.'P') THEN
         GOTO 160

!     RECOVER THE STARTING POLYNOMIAL COEFFICIENTS FOR THIS ORDER
      ELSEIF (CCODE.EQ.'R') THEN
         GOTO 155

!     MODIFY CONTINUUM SLOPE
      ELSEIF (CCODE.EQ.'C') THEN
         GOTO 300

!     MODIFY CENTRAL WAVELENGTH
      ELSEIF (CCODE.EQ.'W') THEN
         CALL GX_QCUR (W,F)
         WCEN = W
         GOTO 170

!     MODIFY FLUX MAXIMUM
!     NOW REDUNDANT ... WOULD HAVE TO BE INCLUDED AS LINEAR SHIFT IN
!                       LINEAR POLYNOMIAL COEFFICIENT IF REQUIRED>
      ELSEIF (CCODE.EQ.'F') THEN
         CALL GX_QCUR (W,F)
         FMAX = F
         GOTO 170

!     TRIM THE VALID RANGE
      ELSEIF (CCODE.EQ.'T') THEN
         GOTO 400

!     THAT'S ALL
      ENDIF
      GOTO 200


!     MODIFY THE BLAZE FUNCTION

!     USE TWO POINTS TO ADJUST POLYNOMIAL COEFFICIENTS
300   CALL MESSGE ('Mark two continuum points on data')
      CALL GX_QCUR (W1,F1)
      CALL GX_QCUR (W2,F2)

!     CHECK FOR NULL
      IF (ABS(W1-W2).LT.1.0) THEN
         CALL MESSGE ('BLEDIT: * Error * points must be well separated')
         GOTO 200
      ENDIF

!     CALCULATE THE CURRENT BLAZE FUNCTION VALUES
!      NB USE OF NORMALISED INDEPENDENT VARIABLE
      XMIN = EBLXM(J)
      XMAX = EBLXP(J)
      XMXM = XMAX - XMIN
      W1X  = W1-WCEN
      W1XX = ((W1X-XMIN) + (W1X-XMAX)) / XMXM
      CALL CBEVAL (9,CBCOFF,XMIN,XMAX, W1X, F1X)
      W2X = W2-WCEN
      W2XX = ((W2X-XMIN) + (W2X-XMAX)) / XMXM
      CALL CBEVAL (9,CBCOFF,XMIN,XMAX, W2X, F2X)

!     EVALUATE THE RESIDUALS AFTER REMOVAL OF (A0 + A1.X)
      A0X = CBCOFF(1)
      A1X = CBCOFF(2)
      C1  = F1X - (A0X + A1X * W1XX)
      C2  = F2X - (A0X + A1X * W2XX)

!     AND WHAT SHOULD THEY HAVE BEEN?
      D1 = F1 - C1
      D2 = F2 - C2

!     BACKSUBSTITUTE TO RECOVER NEW COEFFICIENTS
      A1  = (D1 - D2) / (W1XX - W2XX)
      A0  = D1 - A1 * W1XX

      CBCOFF(1) = A0
      CBCOFF(2) = A1

!     NOW GO BACK TO REPLOT ORDER WITH REVISED FIT
      GOTO 170


!     TRIM THE BLAZE FUNCTION

!     USE TWO POINTS TO ADJUST POLYNOMIAL COEFFICIENTS
400   CALL MESSGE ('Indicate new limits for fit')
      CALL GX_QCUR (W1,F1)
      CALL GX_QCUR (W2,F2)
      IF (W1.GT.W2) THEN
         W  = W2
         W2 = W1
         W1 = W
      ENDIF

!     CHECK FOR NULL AND EXTRAPOLATION
      IF (ABS(W1-W2).LT.1.0) THEN
         CALL MESSGE ('BLEDIT: * Error * points must be well separated')
         GOTO 200
      ELSEIF (W1-WCEN.LT.XMIN .OR. W2-WCEN.GT.XMAX) THEN
         CALL MESSGE ('BLEDIT: * Warning * cannot extend range of fit')
         W1 = MAX (W1, XMIN + WCEN)
         W2 = MIN (W2, XMAX + WCEN)
      ENDIF

!     COPY FIT INTO CURRENT ARRAYS WITH TRIMMED X VALUES
      XLO = W1 - WCEN
      XHI = W2 - WCEN
      ILO = 10. * XLO
      IHI = 10. * XHI
      DO 410 I = ILO,IHI
         X = FLOAT (I) / 10.
         CALL CBEVAL (9,CBCOFF,XMIN,XMAX, X,Y)
         IX = 1 + I - ILO
         CX(IX) = X
         CY(IX) = Y
410      CONTINUE
      NCX = IX

!     FIT NEW CHEBYSHEV SERIES TO THIS FUNCTION
      CALL CBLSQF (CX,CY,NCX, 9, CBCOFF,XMIN,XMAX)

!     NOW GO AND PLOT THE NEW FIT
      GOTO 170


!     NORMAL EXIT

900   CONTINUE
      CALL GX_CLR

!     ZERO PERMANENT ARRAYS SO NO UNDESIRED RESIDUALS ARE CARRIED OVER
      CALL MXZERO (BLCHEB, 10,32)
      CALL MXZERO (BLPOLY, 10,32)

      DO 910 I = 1,NECORD
!        COPY POLYNOMIAL FIT TO PERMANENT ARRAYS
         IF (LEBLF(I)) THEN
            BLWCEN(I) = EBLWC(I)
            BLFMAX(I) = EBLFM(I)
            DO 905 J = 1,10
905            BLCHEB(J,I) = EBLCF(J,I)
            LBLF(I)   = LT
            BLXMIN(I) = EBLXM(I)
            BLXMAX(I) = EBLXP(I)
!        MORE ZEROING
         ELSE
            BLWCEN(I) = 0.
            BLFMAX(I) = 0.
            LBLF(I)   = LF
         ENDIF
910   CONTINUE

!     AND QUIT THIS SUBROUTINE
      CALL LINEFD (1)
      CALL MESSGE ('End of Echelle Blaze function editing')
      CALL IOFOOT
      RETURN


!     ABORT EXIT

999   CALL MESSGE ('BLEDIT: Abnormal termination')
      CALL MESSGE ('        Blaze function unchanged')
      END
