
      SUBROUTINE DBLDX (NW,MO, DRDX, DRDK)

!+
!     PROGRAM:      DBLDX
!     PURPOSE:      
!=              COMPUTE DERIVATIVE OF BLAZE FUNCTION (WRT TO X)
!
!     NOTES:    (1) MUST BE PRECEDED BY A CALL TO BLAZE
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      29.03.88  C.S.JEFFERY / ST.ANDREWS
!
!     INPUT:        NW      WAVELENGTH POINTER
!                   MO      ORDER POINTER
!
!     OUTPUT:       DRDX    DERIVATIVE OF BLAZE WRT TO ORDER VARIABLE
!                   DRDK    DERIVATIVE OF BLAZE WRT TO WCEN / MORDER
!-

      INCLUDE 'implct.f'
      INCLUDE 'ecdeck.f'
      INCLUDE 'blazer.f'
!.

!     TRANSLATE INPUT
      M = MO
      N = NW

!     SINC**2 BLAZE FUNCTION
      IF (IBLTYP.EQ.1) THEN

!        AVOID SINGULARITY FOR LAMC(M) = LL
         IF (ABS(YY).GT.1.E-3) THEN
            DRDX = (YY * SIN (2*YY) - 2 * (SIN (YY))**2 )
     1                  / (YY**3) * PI * BLA(M)
            DRDK = - DRDX / WL
         ELSE
            DRDX = 0.
            DRDK = 0.
         ENDIF

!     9TH ORDER CHEBYSHEV POLYNOMIALS
      ELSEIF (IBLTYP.EQ.2) THEN

!        BLAZE UNDEFINED FOR THIS ORDER, BLAZE=1, DERIVATIVES=0.
         IF (.NOT.LBLF(M)) THEN
            DRDX = 0.
            DRDK = 0.
            RETURN
         ENDIF

!        DBLDX MUST BE PRECEDED BY A CALL TO BLAZE
         IF (M.NE.MSAVE) THEN
            DRDX = 0.
            DRDK = 0.
            RETURN
         ENDIF

!       EVALUATE DERIVATIVE OF CHEBYSHEV
         CALL CBEVAL (8,CBDCFF,CBXMIN,CBXMAX, XX, DRDX)

!       AND THE DERIVATIVE OF THE BLAZE FUNCTION WRT K
         DRDK = - DRDX / OM

!     NO OTHER BLAZE FUNCTION TYPES ARE CURRENTLY SUPPORTED

      ELSE
         DRDX = 0.
         DRDK = 0.
      ENDIF

      END
