      SUBROUTINE ECOVLP

!+
!     PROGRAM:      ECOVLP
!     PURPOSE:      
!=                  AUTOMATIC PROCEDURE TO DEFINE OVERLAP REGIONS
!
!     REFERENCES:   
!
!     METHODS:      0) DEFINE REGION WIDTH 'DELTAL' CENTRED ON FLUX CROSSOVER
!                   1) DEFINE REGION WIDTH 'DELTAL' CENTRED ON ACTUAL OVERLAP
!                   2) DEFINE OVERLAPS FROM VALID ZONES FOR CHEBYSHEVS
!
!     NOTES:        
!                   1) ASSUME ALL ORDERS SCRUNCHED WITH SAME DL
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      12.01.88  C.S.JEFFERY / ST.ANDREWS
!
!     INPUT:        DELTAL  WIDTH (IN WAVELENGTH UNITS) OF OVERLAP ZONE
!                   MODE    METHOD SELECTOR SWITCH (SEE ABOVE)
!-


      INCLUDE 'implct.f'
      INCLUDE 'ctdeck.f'
      INCLUDE 'ecdeck.f'



      INTEGER   MODE
      REAL      DELTAL

      INTEGER   IX, IY, MX, MY
      INTEGER   NXLO, NXUP, NXC, NYLO, NYUP, NYC
      REAL      WLO, WUP,  XMEAN, YMEAN
!.


!     GET MODE OF OVERLAP DEFINITION

      IF (NARG.GE.1) THEN
         MODE = IFRMCH (ARG(1))
!        DEFAULTS DEPEND UPON TYPE OF BLAZE FUNCTION
         IF (MODE.GT.1 .AND. IBLTYP.EQ.1) THEN 
            MODE = 0
         ENDIF
      ELSE
         CALL GETINT ('ECOVLP: Enter mode for defining overlap',MODE)
      ENDIF

!     GET OVERLAP INTERVAL (PROMPT IF REQUIRED)

      DELTAL = 1.0
      IF (NARG.GE.2) THEN
         DELTAL = RFRMCH (ARG(2))
      ELSEIF (MODE.EQ.0 .OR. MODE.EQ.1) THEN
         CALL GETFPV ('ECOVLP: Enter width of overlap interval',DELTAL)
      ENDIF



!     EQUAL OVERLAPS CENTRED ON FLUX CROSSOVER

      IF (MODE.EQ.0) THEN

!        LOOP OVER ORDERS
         DO 20 N = 1,NECORD-1
            MX = N
            MY = N + 1

!           DEFINE UPPER AND LOWER LIMITS OF ACTUAL OVERLAP
            WLO = ECSLAM(MY) +              ECDLAM(MY)
            WUP = ECSLAM(MX) + NECWAV(MX) * ECDLAM(MX)
!           GET POINTERS TO OVERLAP LIMITS (ASSUMING EQUAL INCREMENTS ECDLAM)
            NYLO = 1
            NYUP = (WUP - WLO) / ECDLAM(MY) + 1
            NXUP = NECWAV(MX)
            NXLO = NXUP - (NYUP - NYLO)

!           NOW DEFINE CENTRE OF OVERLAP

!           RUN THROUGH OVERLAP REGION
            DO 12 IX = NXLO+3,NXUP-3
               IY = IX + NYLO - NXLO
!              COMPUTE RUNNING MEANS
               XMEAN = 0.
               YMEAN = 0.
               DO 11 J = -2,2,1
                  XMEAN = XMEAN + ECSTAK (IX+J, MX)
                  YMEAN = YMEAN + ECSTAK (IY+J, MY)
 11               CONTINUE
               XMEAN = XMEAN / 5.
               YMEAN = YMEAN / 5.
!              TEST MEANS FOR CROSSOVER
               IF (YMEAN.GE.XMEAN) THEN
                  NXC  = IX
                  GOTO 15
               ENDIF
 12            CONTINUE

!           ERROR, CROSSOVER NOT FOUND
            CALL MESSGE ('ECOVLP: ** Warning ** Crossover not found')
            CALL MESSGE ('ECOVLP: Adopting default values (Mode 1)')
            NXC  = NINT ((NXUP + NXLO) / 2.)

!           FOUND IT
 15         CONTINUE

!           SO SAVE THE OVERLAP REGION IN COMMON
            WCEN   = ECSLAM(MX) + NXC * ECDLAM(MX)
            WOVMAX = WCEN + DELTAL / 2.0
            WOVMIN = WCEN - DELTAL / 2.0

!           COLLECT ECHELLE SPECTRUM DEFINITION
            SLMX   = ECSLAM(MX)
            DLMX   = ECDLAM(MX)
            SLMY   = ECSLAM(MY)
            DLMY   = ECDLAM(MY)

!           COMPUTE INDICES OF OVERLAP LIMITS (AND ALLOW ONE PIXEL)
            NXUPOV(MX) = (WOVMAX - SLMX) / DLMX  -  1
            NXLOOV(MX) = (WOVMIN - SLMX) / DLMX  +  1
            NYUPOV(MY) = (WOVMAX - SLMY) / DLMY  -  1
            NYLOOV(MY) = (WOVMIN - SLMY) / DLMY  +  1

!           THAT'S ALL FOR ORDER MX
 20         CONTINUE



!     EQUAL OVERLAPS CENTRED ON TOTAL OVERLAP ZONE

      ELSEIF (MODE.EQ.1) THEN

!        LOOP OVER ORDERS
         DO 30 N = 1,NECORD-1
            MX = N
            MY = N + 1

!           DEFINE UPPER AND LOWER LIMITS OF ACTUAL OVERLAP
            WUP    = ECSLAM(MX) + NECWAV(MX) * ECDLAM(MX)
            WLO    = ECSLAM(MY) +              ECDLAM(MY)
            WCEN   = (WLO + WUP) / 2.0

!           IDENTIFY UPPER AN LOWER LIMITS OF OVERLAP
            WOVMAX = WCEN + DELTAL
            WOVMIN = WCEN - DELTAL

!           COLLECT ECHELLE SPECTRUM DEFINITION
            SLMX   = ECSLAM(MX)
            DLMX   = ECDLAM(MX)
            SLMY   = ECSLAM(MY)
            DLMY   = ECDLAM(MY)

!           COMPUTE INDICES OF OVERLAP LIMITS (AND ALLOW ONE PIXEL)
            NXUPOV(MX) = (WOVMAX - SLMX) / DLMX  -  1
            NXLOOV(MX) = (WOVMIN - SLMX) / DLMX  +  1
            NYUPOV(MY) = (WOVMAX - SLMY) / DLMY  -  1
            NYLOOV(MY) = (WOVMIN - SLMY) / DLMY  +  1

!           THAT'S ALL FOR ORDER MX
 30         CONTINUE



!     OVERLAP DEFINED BY VALIDITY OF THE BLAZE FUNCTION POLYNOMIALS

      ELSEIF (MODE.EQ.2) THEN

!        LOOP OVER ORDERS
         DO 40 N = 1,NECORD-1
            MX = N
            MY = N + 1

!           IDENTIFY UPPER AN LOWER LIMITS OF OVERLAP
            WOVMAX = BLXMAX(MX) + BLWCEN(MX)
            WOVMIN = BLXMIN(MY) + BLWCEN(MY)

!           COLLECT ECHELLE SPECTRUM DEFINITION
            SLMX   = ECSLAM(MX)
            DLMX   = ECDLAM(MX)
            SLMY   = ECSLAM(MY)
            DLMY   = ECDLAM(MY)

!           COMPUTE INDICES OF OVERLAP LIMITS (AND ALLOW ONE PIXEL)
            NXUPOV(MX) = (WOVMAX - SLMX) / DLMX  -  1
            NXLOOV(MX) = (WOVMIN - SLMX) / DLMX  +  1
            NYUPOV(MY) = (WOVMAX - SLMY) / DLMY  -  1
            NYLOOV(MY) = (WOVMIN - SLMY) / DLMY  +  1

!           ALL FOR ORDER MX
 40         CONTINUE

      ENDIF


!     SET DEFAULTS FOR THE START AND END OF FIRST AND LAST ORDERS

      NYLOOV(1)      = NYLOOV(2)
      NYUPOV(1)      = NYUPOV(2)
      NXLOOV(NECORD) = NXLOOV(NECORD-1)
      NXUPOV(NECORD) = NXUPOV(NECORD-1)

!     SET INDICATOR FOR DEFINITION OF OVERLAP ZONES
      NWAVOV = NXUPOV(1) - NXLOOV(1)


!     WRITE OUT OVERLAP POINTERS
      IF (NARG.EQ.3 .AND. ARG(3).EQ.'PRINT') THEN
         WRITE (6,9998)
         DO 50 N = 1,NECORD
         WRITE (6,9999) N, MECORD(N), ECSLAM(N), ECDLAM(N), NECWAV(N),
     1                  NYLOOV(N), NYUPOV(N), NXLOOV(N), NXUPOV(N)
50       CONTINUE
      ENDIF


9998  FORMAT ('   N  IORD     W0     DL   NWAV  NYLO  NYUP  NXLO  NXUP')
9999  FORMAT (2I5,F10.3,F6.3,5I6)



!     EXIT
      END
