      SUBROUTINE INTERP

!+
!     PROGRAM:      INTERP
!     PURPOSE:      
!=                  INTERPOLATE A FUNCTION ON EXISTING DATA ON A NEW GRID
!     REFERENCES:   
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0           microVAX      28.10.87  C.S.JEFFERY / ST.ANDREWS
!
!     INPUT:        
!
!     OUTPUT:       
!-

      INCLUDE 'ctdeck.f'
      INCLUDE 'spdeck.f'


!     LOCAL VARIABLES DEFINING THE INTERPOLATION GRID AND RESULTING FIT
      CHARACTER*10 STYLE
      REAL         X(NSP), Y(NSP), DXINT, XIMIN, XIMAX
      INTEGER      NIPTS,  NIFRQ,  NIMAX, I,     IERR,  NORDER

!     WORK ARRAYS REQUIRED BY LSQFIT
      REAL         AA(11), ERR(11), RMS, CH2, RML, FRT, R2B

!     WORK ARRAYS REQUIRED BY L2FRES
      INTEGER      NW, MAX, NK
      PARAMETER   (NW=50, NQ=13*NSP+6)
      REAL         WX(NW),WF(NW),WG(NW),WD(NW),WT(NW)
      REAL         WR(NSP), WW(NSP), WS(NQ)
!.


!     GET TYPE OF INTERPOLATION
      IF (NARG.EQ.0) THEN
1        CALL GETCHV ('INTERP: interpolation type (? for help) ',STYLE)
         IF (STYLE.EQ.'?') THEN
            CALL MESSGE ('        LINEAR : Linear join the dots')
            CALL MESSGE ('        POLY   : Polynomial join the dots')
            CALL MESSGE ('        LSQ    : Least squares polynomial')
            CALL MESSGE ('        SPLINE : Cubic with knots (L2FRES)')
            CALL MESSGE ('        HERMITE: Hermite polynomials (INTEP)')
            GO TO 1
         ENDIF
      ELSE
         STYLE = ARG(1)
      ENDIF

!     DEFINE THE INTERPOLATION GRID (EXTRAPOLATION IS FORBIDDEN)
      IF (NARG.LT.2) THEN
         CALL GETFPV ('INTERP: Interval for interpolation grid',DXINT)
      ELSE
         DXINT = RFRMCH (ARG(2))
      ENDIF

!     SNEAKY BIT FOR FIXING GRID TO SENSIBLE SCALE (WE HOPE)
      XIMIN = INT (DXMIN)
      NIFRQ = INT (1.0/DXINT)
      DO 201 I=0,NIFRQ,1
         XI = XIMIN + I*DXINT
         IF (XI.GE.DXMIN) THEN
            XIMIN = XI
            GO TO 202
         ENDIF
201   CONTINUE
202   XIMAX = INT (DXMAX)
      NIMAX = INT ((DXMAX-DXMIN)/DXINT + 1.)
      DO 206 I=1,NIMAX,1
         XI = XIMIN + I*DXINT
         IF (XI.GT.DXMAX) THEN
            XIMAX = XI - DXINT
            NIPTS = I
            GO TO 207
         ENDIF
206   CONTINUE
207   CONTINUE

!          ............................................
!          .     OUTPUT FROM THIS SECTION:            .
!          .     -------------------------            .
!          .     STYLE : INTERPOLATION STYLE          .
!          .     DXINT : INTERPOLATION INTERVAL       .
!          .     XIMIN : MINIMUM VALUE ON GRID        .
!          .     XIMAX : MAXIMUM VALUE ON GRID        .
!          .     NIPTS : NUMBER OF POINTS ON GRID     .
!          ............................................



!     DEFINE THE INTERPOLATION GRID
      DO 250 I = 1,NIPTS
250      X(I) = XIMIN + (I-1)*DXINT

!     LINEAR INTERPOLATION
      IF (STYLE(1:3).EQ.'LIN') THEN
         DO 301 I = 1,NIPTS
            CALL TABINT (NSPW, SPW, SPF, 1, X(I), Y(I))
301      CONTINUE

!     POLYNOMIAL INTERPOLATION (DEGREE N)
      ELSEIF (STYLE(1:3).EQ.'POL') THEN
         CALL GETINT ('INTERP: Order of polynomial (1-9)',NORDER)
         DO 401 I = 1,NIPTS
            CALL TABINT (NSPW, SPW, SPF, NORDER, X(I), Y(I))
401      CONTINUE

!     LEAST SQUARES POLYNOMIAL (DEGREE N) INTERPOLATION
      ELSEIF (STYLE(1:3).EQ.'LSQ') THEN

!        FIND THE ORDER AND DO THE FIT
         CALL GETINT ('INTERP: Order of polynomial (1-9)',NORDER)
         CALL LSQPOL (SPW, SPF, NSPW, NORDER,
     1                AA, ERR, RMS, CH2, RML, FRT, R2B)

!        REPORT DETAILS OF THE FIT
         CALL LSQRPT (NORDER, AA, ERR, RMS, CH2)

!        NOW CONSTRUCT THE FITTED FUNCTION USING EXPANDED POLYNOMIAL
         CALL POLEXP (NORDER, AA, X, Y, NIPTS)

!     HERMITE POLYNOMIAL FITTING
      ELSEIF (STYLE(1:3).EQ.'HER') THEN
         CALL INTEP (X(1), Y(1), SPW, SPF, NSPW, IERR)
         DO 601 I = 2,NIPTS
            CALL EINTEP (X(I), Y(I), SPW, SPF, NSPW, IERR)
601      CONTINUE

!     LEAST SQUARES KNOTTED CUBIC SPLINE INTERPOLATION
      ELSEIF (STYLE(1:3).EQ.'SPL') THEN
         MAX = MIN ( NSPW*0.66, FLOAT(NW) )
         CALL L2FRES (IERR, MAX, NK,WX,WF,WG,WD,WT,
     1        .TRUE.,NSPW,SPW,SPF, WR, .FALSE.,WW, .TRUE.,NIPTS,X,Y, WS)

!     NO MORE METHODS
      ENDIF

!     OVERWRITE THE WORK ARRAYS BACK INTO THE MAIN DATA ARRAYS
      DO 701 I = 1,NIPTS
         SPW(I) = X(I)
         SPF(I) = Y(I)
701   CONTINUE
      NSPW = NIPTS
      CALL SPSTAT

!     QUIT
      END
