      SUBROUTINE LPRADV (IOUT)

!+
!=    DETERMINE THE RADIAL VELOCITY OF THE DEFINED LINE PROFILE
!     RELATIVE TO THE REFERENCE WAVELENGTH 'LINWAV'
!-

      INCLUDE 'ctdeck.f'
      INCLUDE 'lpdeck.f'
      INCLUDE 'dispgx.f'
      INTEGER IOUT

      DATA CLIGHT /2.99773E5/
!.

!     CHECK INPUT DATA
      IF (NLPW.LT.4) THEN
         CALL MESSGE ('LPRADV: Insufficient data for parabola fit')
         RETURN
      ELSEIF (LPLABW.LE.0.0) THEN
         CALL MESSGE ('LPRADV: Lab wavelength undefined')
         RETURN
      ENDIF

!     FIT A PAARABOLA TO THE CURRENT LINE PROFILE
      CALL PRBOLA (LPW,LPF,NLPW, A,EA,B,EB,C,EC, X0,EX0,Y0,EY0, IS)

!     CONVERT OBSERVED WAVELENGTH TO RADIAL VELOCITY (NON-RELATIVISTIC)
      LPOBSW = X0 + LPWC
      LPOBSE = EX0
      LPRV   = CLIGHT * (LPOBSW - LPLABW) / LPLABW
      LPRVER = CLIGHT * EX0 / LPLABW

!     DISPLAY THE FIT
      IF (DPLTOK) THEN
         X = LPW(1)
         Y = A*X*X + B*X + C
         CALL GX_QMOV (X,Y)
         DO 100 I = 2,NLPW
            X = LPW(I)
            Y = A*X*X + B*X + C
            CALL GX_QDRW (X,Y)
100      CONTINUE
      ENDIF

!     PRINT OUT THE RESULTS
      IF (IOUT.GT.0) THEN
         CALL OUTFPV ('LPRADV: Laboratory wavelength          ',LPLABW)
         CALL OUTFPV ('LPRADV: Observed wavelength            ',LPOBSW)
         CALL OUTFPV ('LPRADV: Radial velocity of line (km/s) ',LPRV  )
         CALL OUTFPV ('LPRADV: Formal error in fit         +/-',LPRVER)
      ENDIF

!     QUIT
      END
