      SUBROUTINE rtf_fluxes ( tau,b,s,nd, j,h,k,f )

!+
!
!  Program:
!     rtf_fluxes
!
!  Purpose:
!=    calculate flux and moments at all points within atmosphere
!     Assumes diffusion approximation for tau > tau_diff
!
!-

!  Global constants
      USE TAPM_PAR

!  Global declaration
      IMPLICIT NONE

!  Local constants
      DOUBLE PRECISION tau_diff               ! Diffusion approx. tau>tau_diff
      PARAMETER ( tau_diff = 5.0D0 )

!  Subroutine Arguments
!    Import
      DOUBLE PRECISION tau(*)                 ! Optical depth
      DOUBLE PRECISION b(*)                   ! Planck function
      DOUBLE PRECISION s(*)                   ! Source function
      INTEGER nd                              ! Number of depths

!    Export
      DOUBLE PRECISION j(*)                   ! Mean intensity
      DOUBLE PRECISION h(*)                   ! Eddington flux
      DOUBLE PRECISION k(*)                   ! Second moment
      DOUBLE PRECISION f(*)                   ! Total flux

!  Local Variables
      INTEGER  d                              ! Depth quadrature index

!.

!   Loop over depths
      DO d = 1,nd

!     Integrate transfer equation for fluxes.      
        IF ( tau(d).LT.tau_diff ) THEN
          CALL RTF_LAM ( tau, s, nd, tau(d), j(d) )
          CALL RTF_PHI ( tau, s, nd, tau(d), f(d) )
          CALL RTF_XI  ( tau, s, nd, tau(d), k(d) )

!     Diffusion approximation.
        ELSE
          j(d) = s(d)
          IF (d .LT. nd ) THEN
            f(d) = 0.5*(b(d)-b(d-1))/(tau(d)-tau(d-1)) +
     :             0.5*(b(d+1)-b(d))/(tau(d+1)-tau(d))
            f(d) = f(d) * 4./3.
          ELSE
            f(d) = (b(d)-b(d-1))/(tau(d)-tau(d-1)) * 4./3.
          ENDIF
        ENDIF
        
        h(d) = f(d) / 4 * tapm__pi
      ENDDO

      END
