      SUBROUTINE rtf_integ (i1,i2,f,x,y)

!+
!
!  Program:
!     rtf_integ
!
!  Purpose:
!=    Integrate a function y=Int(f(x)) on an interval x = x(i1:i2). 
!     The x-intervals are assumed to be non-uniform. Trapezoidal
!     integration is used.
!
!-

!  Subroutine Arguments.
      INTEGER    I1, I2                           ! limits of integral
                                                  ! x(i1) : x (i2)
      DOUBLE PRECISION x(*)                       ! indep. variable
      DOUBLE PRECISION f(*)                       ! integrand
      DOUBLE PRECISION y(*)                       ! integral

!  Local variables.
      DOUBLE PRECISION  h                         ! quadrature steps
      INTEGER  i, ia, ib                          ! local loop indices

!.

!   Check for null integration
      y(i1) = 0.D0
      y(i2) = 0.D0
      IF (i2.EQ.i1) RETURN

!   Check for i2 > i1
      IF (i2.GT.i1) THEN
        ia = i1
        ib = i2
      ELSE
        ia = i2
        ib = i1
      ENDIF

!   Trapezoidal integration 
      DO i = ia+1,ib,1
        h = abs(x(i) - x(i-1))
        y(i) = h/2.*(f(i)+f(i-1))
        y(i) = y(i) + y(i-1)
      ENDDO

      END

