      SUBROUTINE rtf_lam ( t, f, n, tau, lambda )
!+
!
!  Program.
!     rtf-lam
!
!  Purpose.
!=    1-dimensional 'LAMBDA' operator for evaluating the mean intensity integral.
!
!-

!  Global declaration
      IMPLICIT NONE

!  Subroutine Arguments
!    Import
      DOUBLE PRECISION t(1)                 ! t
      DOUBLE PRECISION f(1)                 ! f(t)
      INTEGER n                             ! dimension of f(t)
      DOUBLE PRECISION tau                  ! evaluation of LAMBDA
!    Export
      DOUBLE PRECISION lambda               ! LAMBDA integral

!  Local Constants
      INTEGER    kd                         ! Quadrature dimensions
      PARAMETER (kd=100)

!  Local Variables
!     Depth quadrature
      DOUBLE PRECISION  s(kd)               ! integrand
      DOUBLE PRECISION  e(kd)               ! exponential integral (debug)
      INTEGER  d                            ! Depth quadrature index
      DOUBLE PRECISION  smax                ! maximum value of integrand
      DOUBLE PRECISION  tmax                ! limit of integral
      INTEGER  k                            ! array locator

!    Gauss-Legendre quadrature
      INTEGER nqmax                ! order of quadrature
      PARAMETER (nqmax = 64)
      INTEGER nq                   ! local quadrature order
      INTEGER nlast                ! previous quadrature order ?
      DOUBLE PRECISION qx(nqmax)   ! scaled quadrature x values
      DOUBLE PRECISION qw(nqmax)   ! quadrature weights
      INTEGER iq                   ! qaudratureloop index
      DOUBLE PRECISION xr, xm      ! quadrature range and median
      DOUBLE PRECISION pt, pf      ! interpolated function
      DOUBLE PRECISION df          ! derivative of interpolated function
      DOUBLE PRECISION et          ! exponential integral E(i,t)
      DOUBLE PRECISION sq          ! quadrature integrand

!  Functions called
      DOUBLE PRECISION rtf_expi
!.


!  Get quadrature absicssae and weights
      nq = 8
      IF (nq .NE. nlast ) CALL nr_gauleg ( -1.D0, 1.D0, qx, qw, nq )
      nlast = nq

!  Calculate integrand,
!  detect largest value and store upper limit to abscissa.
      smax = 0.
      tmax = t(n)
      DO d = 1,n
         e(d) = rtf_expi(1,abs(t(d)-tau))
         s(d) = f(d) * e(d)
         IF ( s(d).GT.smax ) THEN
           smax = s(d)
           tmax = t(min(d+1,n))
         ELSEIF ( s(d).GT.smax/100. ) THEN
           tmax = t(d)
         ELSEIF ( t(d).LT.tau+10. ) THEN
           tmax = t(d)
         ENDIF
      ENDDO

!  Calculate integrand
      sq = 0.D+0
      xm = 0.5 * (tmax)
      xr = 0.5 * (tmax)
      DO iq = 1,nq
         pt = xm + xr * qx(iq) 
         CALL rtf_snu (t,f,n,pt,pf)
         et = rtf_expi(1,abs(pt-tau))
         sq = sq + qw(iq) * pf * et
      ENDDO
      sq = sq * xr
              
!  Result
      lambda = 0.5 * sq

      END
