      SUBROUTINE rtf_qgauss ( x, f, nx, x1, x2, n, s )

!+
!  Program:
!     rtf_qgauss
!
!  Purpose:
!=    Carry out Gauss-Legendre quadrature on a function f(x)
!     over the interval x1 to x2. The value of f is interpolated
!     at the quadrature points. The order of the quadrature is
!     given by n
!
!-

!  Global declarations.
      IMPLICIT NONE

!  Subroutine Arguments.
!    Import
      DOUBLE PRECISION x(1)        ! x - values for which f is tabulated
      DOUBLE PRECISIOn f(1)        ! f(x)
      INTEGER          nx          ! number of x-values tabulated
      DOUBLE PRECISION x1          ! lower limit of integral
      DOUBLE PRECISION x2          ! upper limit of integral
      INTEGER n                    ! order of quadrature 
!    Export      
      DOUBLE PRECISION s           ! integral f(x1:x2)

!  Local constants.
      INTEGER nmax                 ! maximum dimension for work arrays
      INTEGER nqmax                ! order of quadrature
      PARAMETER (nqmax = 64)

!  Local variables.
!    Gauss-Legendre quadrature
      INTEGER nq                   ! local quadrature order
      INTEGER nlast                ! previous quadrature order ?
      DOUBLE PRECISION qx(nqmax)   ! scaled quadrature x values
      DOUBLE PRECISION qw(nqmax)   !        quadrature weights
      INTEGER i                    ! qaudratureloop index
      DOUBLE PRECISION px          ! adopted quadrature x-values
      DOUBLE PRECISION pf          ! interpolated f(px)
!    Scaled integrand
      DOUBLE PRECISION xr          ! x range of integral
      DOUBLE PRECISION xm          ! x midpoint of integral 
      INTEGER k                    ! local index for locating qx
      
      DATA nlast / 0 /

!.
      
!  Check quadrature limit
      nq = min(nqmax,n)

!  Get quadrature absicssae and weights
      IF (nq .NE. nlast ) THEN
         CALL rtf_gauleg ( -1.D0, 1.D0, qx, qw, nq )
         nlast = nq
      ENDIF 

!  Interval checking
      s = 0.D+0
      IF ( x2.LE.x1 ) RETURN

!  Initialization
      xm = 0.5 * (x1+x2)
      xr = 0.5 * (x2-x1)

!  Evaluate f(x) at quadrature x-values
      k = 1
      DO i = 1,nq
         px = xm + xr * qx(i) 
         IF ( px.GT.x(k+1) ) THEN
         DO WHILE ( px.GT.x(k+1) )
            k = min(k+1,nx)
         ENDDO
         ENDIF
         CALL rtf_poly (x(k),f(k),px,pf)
         s = s + qw(i) * pf
      ENDDO

!  Renormalization
      s = xr * s
      
      END
