      SUBROUTINE rtf_snu (x,y,n,xx,yy)

!+
!
!=    Use quadratic interpolation on n-point data in arrays
!     x and y to evaluate the function yy at the point xx.
!     If zero-xintervals are given, linear or 3-point averages are 
!     used. (Based on qub program 'poly').
!
!     It may be appropriate to use linear interpolation for this
!     program. Speed is very important.        csj...27/1/98
!
!-

!  Global Declarations.
      IMPLICIT REAL*8 (a-h,o-z)

!  Subroutine Arguments.
!     Input:
      DOUBLE PRECISION x(*)                      ! array of x values
      DOUBLE PRECISION y(*)                      ! y (x)
      INTEGER          n                         ! number of x-values
                                                 ! NB only first 3 values used
      DOUBLE PRECISION xx                        ! x point to interpolate y

!     Output:
      DOUBLE PRECISION yy                        ! interpolated y value
!
! Local variables.
      INTEGER i, lasti
      DATA lasti / 1 /

!.

      i = lasti
      IF (xx.LT.x(lasti)) i = 1
      DO WHILE (xx.GT.x(i+1).AND.i.LT.n-2)
         i = i+1
      ENDDO

      a=x(i)  -xx
      b=x(i+1)-xx
      c=x(i+2)-xx
      d=x(i)  -x(i+1)
      e=x(i+1)-x(i+2)
      f=x(i+2)-x(i)

!      quadratic interpolation/extrapolation
      IF (d*e*f.ne.0.0) THEN
        yy=-b*c/(d*f)*y(i)
     :     -a*c/(d*e)*y(i+1)
     :     -a*b/(e*f)*y(i+2)
      
!      linear interpolation/extrapolation
      ELSEIF (d.ne.0. .OR. e.ne.0.) THEN
        IF (d.eq.0.) yy=(b*y(i+2)-.5*c*(y(i)+y(i+1)))/e
        IF (e.eq.0.) yy=(c*y(i)-.5*a*(y(i+1)+y(i+2)))/f
        IF (f.eq.0.) yy=(a*y(i+1)-.5*b*(y(i)+y(i+2)))/d

!      three point average
      ELSE
        yy=(y(i)+y(i+1)+y(i+2))/3.0

      ENDIF
      
      END                                                               
