      SUBROUTINE rtf_trans (tau,s,nd, mu, i,f)


!+
!=    calculate the emergent intensity  (i)  and  flux (f) as a formal solution
!     of the radiative transfer equation 
!     given the source function (s) as a function of optical depth (tau)
!     for the case of a semi-infinite plane-parallel atmosphere.
!     i is calculated for a given obliquity (mu = cos(theta)).
!
!     D.Mihalas, Stellar Atmospheres, 2nd ed. (2-52), (8-11)
!-


      IMPLICIT   NONE
      INTEGER    kd
      PARAMETER (kd=100)
                                                 ! I/O list
      INTEGER    nd, d                            ! number of depth points
      REAL*8     tau(1)                           ! optical depth
      REAL*8     s(1)                             ! source function
      REAL*8     mu                               ! cosine angle of radiation
      REAL*8     i                                ! specific intensity (d=1)
      REAL*8     f                                ! emergent flux
                                                 ! 
                                                 ! local variables
      REAL*8     tt(kd),  ss(kd), amu             ! local copy if inputs
      INTEGER    ifail, n, d1, d2
      EXTERNAL   d01baz, rtf_fint, rtf_iint       ! integrand functions
      REAL*8     d01baf

      COMMON /rtf_source/ tt, ss, amu, n

!      Initialize variables for CALL statements

      INTEGER I48
!.
      I48=48


      d1 = 1
      d2 = 1
      DO d = 1,nd
        tt(d) = tau(d)
        ss(d) = s(d)
        IF (tt(d).LE.100.0) d2 = max (d,d2)
      ENDDO
      n = nd
      amu = mu

      ifail = 0
      i = d01baf (d01baz,tau(d1),tau(d2),I48,rtf_iint,ifail) / mu

      ifail = 0
      f = d01baf (d01baz,tau(d1),tau(d2),I48,rtf_fint,ifail) * 2



      END
