      SUBROUTINE avlol (bn,ca,sig,tau,flxn,n,z)

!+
!     Purpose:
!=    Calculate monochromatic source function and flux.
!
!     Description:
!     Calculate the monochromatic source function and flux by the method
!     of Avrett and Loeser. The integral operators are represented by matrices
!     and the source function is determined by iteration. This version is
!     adapted for use in line formation calculations. At large tau, S = B.
!
!     Z is an adjustable parameter for the expansion of the source function
!     and is normally set to 1. Only in the case of non-convergence should
!     it be changed.
!
!     Adapted from a program by U.Heber + D.Schoenberner
!     by C.S.Jeffery         9.May.1991
!
!-


!       Double precision version

      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
      DIMENSION tau(*), bn(*), ca(*), sig(*)
      DIMENSION g(50,50), h(50), s(50), s0(50), c(50), jn(50), 
     :          rho(50)
      REAL      jn

      DATA      ndebug / 0 /                      ! set > 0 to print details

!.

      IF (ndebug.gt.0) THEN
        IF (ndebug.eq.1) REWIND 89
        DO i = 1,n
          WRITE (89,1088) i,tau(i),ca(i),sig(i),bn(i)
 1088     FORMAT ( 1x,i2, 1p,5e18.10 )
        ENDDO
      ENDIF


!    Identify region of photosphere whence flux comes 
      emz = 1. - z
      epz = 1. + z
      n0 = 1
      DO WHILE (tau(n0).LT.5.E-4 .AND. n0.LT.n-10) 
        n0 = n0 + 1
      ENDDO
      nj = n0 + 9
      DO WHILE (tau(nj).LT.7.E+0 .AND. nj.LT.n)
        nj = nj + 1
      ENDDO



!    calculate matrix coefficients for Avrett-Loeser parabolae
      nm1 = nj - 1
      DO j = n0, nj
        CALL expi (ex2,ex3,ex4,ex5,tau(j))
        g(j,n0) = 1. - ex2/2.
        g(j,nj) = tau(j) + ex3/2.
        IF (j.EQ.n0) THEN
          h(n0) = 2.*ex3
          h(nj) = 4./3. - 2.*ex4
        ENDIF
        DO i = n0+1, nm1
          IF (i.LE.j) THEN
            ta = tau(j) - tau(i)
            CALL expi (e2,e3,e4,e5,ta)
            g(j,i) = (z*(e4-ex4)/tau(i)+0.5*(emz*e3-epz*ex3))/tau(i)
     :                - .5*ex2
            IF (j.EQ.n0) 
     :         h(i) = -4.*(z*(e5-ex5)/tau(i)+0.5*(emz*e4-epz*ex4))
     :                / tau(i)                            + 2.*ex3
          ELSE
            ta = tau(i) - tau(j)
            CALL expi (e2,e3,e4,e5,ta)
            g(j,i) = ((z*(2./3.-e4-ex4)+ta*(tau(i)-z*tau(j)))/tau(i)
     :               +0.5*(emz*e3-epz*ex3))/tau(i) - 0.5*ex2
            IF (j.EQ.n0) 
     :         h(i) = -4.*((z*(e5-ex5)+2./3.*(epz/2.*tau(i)-z*tau(j)))
     :               /tau(i)-0.5*(emz*e4+epz*ex4))/tau(i) + 2.*ex3
          ENDIF
        ENDDO
      ENDDO


!    1st approximation for source function (S = B)
      DO j = n0, nj
        rho(j) = sig(j)/(ca(j)+sig(j))
        s0(j) = bn(j)
      ENDDO

!    Iterate for accurate source function (S = (1-ro).B + ro.J)
      err = 1.0
      DO WHILE (err.GT.5.E-4)
        CALL parabel (n0,nj,s0,tau,c,z)
        DO j = n0, nj
          jn(j) = 0.0
          DO i = n0, nj
            jn(j) = jn(j) + g(j,i)*c(i) 
          ENDDO
          s(j) = (1.-rho(j))*bn(j) + rho(j)*jn(j) 
        ENDDO
        CALL relerr (s0,s,n0,nj,err)
        DO j = n0,nj
          s0(j) = s(j)
        ENDDO
      ENDDO


!    Calculate emergent flux 
      CALL parabel (n0,nj,s,tau,c,z)
      flxn = 0.0
      DO i = n0, nj
        flxn = flxn + h(i)*c(i)
      ENDDO


      IF (ndebug.gt.0) THEN
        DO i = n0,nj
          WRITE (89,1089) i,tau(i),ca(i),sig(i),rho(i),bn(i),jn(i),s(i)
 1089     FORMAT ( 1x,i2, 1p,7e10.2 )
        ENDDO
      ENDIF

      END
