      SUBROUTINE EQUIV

!+     
!=    Normalizes profile and calculates equivalent width        
!-

!  Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!.


!     Normalize fluxes

      IF ( NSOURCE == 3 ) THEN
        CONFL = 1.0
        FLUXN(1:NW)=FLUX(1:NW)/FLUXC(1:NW)


!     Symmetric profiles
      ELSEIF( NMODE.NE.10 .AND. NMODE.NE.11 .AND. SYMM ) THEN

        CONFL=CONFL/FLUX(NW)
        DO  I=1,NW
          FLUXN(I)=FLUX(I)/FLUX(NW)
        ENDDO

!     Asymmetric profile
      ELSEIF( (NMODE.NE.10 .AND. NMODE.NE.11 .AND. .NOT.SYMM) 
     :       .OR.  ((NMODE.EQ.10.OR.NMODE.EQ.11) .AND. 
     :       (NSOURCE .NE. 0
     :       .AND. NSOURCE .NE. 1
     :       .AND. NSOURCE .NE. 3
     :       .AND. NSOURCE .NE. 7)) ) THEN
        CONFL=2.0*CONFL/(FLUX(1)+FLUX(NW))
        F1=FLUX(1)
        FNW=FLUX(NW)

!     Normalises using linear interpolation of fluxes at wavelength extremes
        DO I=1,NW
          FLUXN(I)=FLUX(I)*(W(NW)-W(1))
     :                    /(F1*(W(NW)-W(I))+FNW*(W(I)-W(1)))
        ENDDO

!     Synthetic spectrum (intensities not required)
      ELSEIF ((NMODE.EQ.10.OR.NMODE.EQ.11) .AND. 
     :       (NSOURCE .EQ. 0
     :       .OR. NSOURCE .EQ. 1
     :       .OR. NSOURCE .EQ. 3)) THEN
        FLUXN(1:NW)=FLUX(1:NW)/FLUXC(1:NW)

      ENDIF




!     Calculates equivalent width

!     Trapezoidal integration
      EW=0.0
      DO I=2,NW

        EW=EW+0.5*(2.0-FLUXN(I)-FLUXN(I-1))*(W(I)-W(I-1))
      ENDDO
      EW=ABS(EW)

!     Symmetric profile
      IF(SYMM) EW=2.*EW


      END
