      SUBROUTINE GEN

!+
!=      Integrates the pressure structure
!       using a four-point Runge-Kutte method.
!-

!    Public constants
      USE SPC_CONSTANTS

!    Global variables
      USE SPC_GLOBAL

!    Implicit typing (denegrated)
!     Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

      REAL*8 K1,K2,K3,K4

!.
!       Check for logarithmic gravity

      IF(GRAV.LT.10.0) THEN
        GRAV=10.**GRAV
      ENDIF


!       Opacity for first depth point
!       Only electron scattering
!       an assume PE=0.5*PG

      OP(1)=CELSCT*(CNUMBM/AMU)


!       Grey approximation to radiation pressure

      PRT=1.92E-15*(TEFF**4)


!       Pressures at first depth point
!       AHE is fractional number of helium atoms
!       as COMPOS was called in GENERATE

      PG(1)=TAU(1)*(GRAV/OP(1)-PRT)
      PE(1)=ELPE(PG(1),TEMP(1),AHE,.5*PG(1))

      DO  I=1,NDEPTH-1


!         Radiation pressure set to grey value

        PR=PRT


!         Delta tau and mean temperature

        DELTA=TAU(I+1)-TAU(I)
        TEMEAN=0.5*(TEMP(I)+TEMP(I+1))


!         K1-K4 estimates integrand*delta at current values and
!             gas pressures PG1-PG3 and
!             electron pressures PE1-PE3

        K1=  DELTA*(GRAV/OP(I)-PR)
        PG1= PG(I)+0.5*K1
        PE1= ELPE(PG1,TEMEAN,AHE,0.5*PG1)


!         NB - last variable in OPAC is 0 for QUB opacities

        K2=  DELTA*(GRAV/OPAC(TEMEAN,PG1,PE1,RLAMB,CAP,SIG,I,0,1)-PR)
        PG2= PG(I)+0.5*K2
        PE2= ELPE(PG2,TEMEAN,AHE,0.5*PG2)

        K3=  DELTA*(GRAV/OPAC(TEMEAN,PG2,PE2,RLAMB,CAP,SIG,I,0,1)-PR)
        PG3= PG(I)+K3
        PE3= ELPE(PG3,TEMP(I+1),AHE,0.5*PG3)

        K4=  DELTA*(GRAV/OPAC(TEMP(I+1),PG3,PE3,RLAMB,CAP,SIG,I,0,1)-PR)


!         Calculate pressures and opacity at next depth point

        PG(I+1)=PG(I)+(K1+2.0*K2+2.0*K3+K4)/6.0

        PE(I+1)=ELPE(PG(I+1),TEMP(I+1),AHE,0.5*PG(I+1))

        OP(I+1)=OPAC(TEMP(I+1),PG(I+1),PE(I+1),RLAMB,CAP,SIG,I,0,1)

      ENDDO

      RETURN
      END
