      SUBROUTINE HE_II(PEX,TEMPX)

!+
!  HE_II
!
!=      Calculates He II line profiles
!       based on unified theory of
!             Schoening and Butler (AA Suppl 78,51, 1989)
!
!-

!    Public constants
      USE SPC_CONSTANTS

!  Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!  Temporary work space
      DIMENSION WORHE2(71)
      DIMENSION XVAR(3),YVAR(3),STE(3)

!.


!  Clear the work array
      WORK = 0.

!  Identify transition
      NTRAN=IDHE2

!  Logarithmic variables
      ELNUM=CELNUM*PEX/TEMPX
      ELNUM=LOG10(ELNUM) + 15.
      TEMPL=LOG10(TEMPX)

!  Constant for scaling opacity
      CONST=CLIGHT/(WAVHE2(NTRAN)*WAVHE2(NTRAN))

!  Find position in density and temperature grid
      DO ITEMP=2,NTHE2(NTRAN)-1
        IF(TEMPL.LE.THE2(ITEMP,NTRAN)) GOTO 10
      ENDDO
      ITEMP=NTHE2(NTRAN)-1

   10 DO IELNUM=2,NDHE2(NTRAN)-1
        IF(ELNUM.LE.ELHE2(IELNUM,NTRAN)) GOTO 11
      ENDDO
      IELNUM=NDHE2(NTRAN)-1

!  Debug output
   11 IF(BUGLN) THEN
        WRITE(NDEBUG,900)NTRAN,WAVHE2(NTRAN),CONST,
     *                   IELNUM,ELNUM,ITEMP,TEMPL
      ENDIF


!  Linear Temperature and density interpolation
!      DO J=1,NWHE2(NTRAN)
!
!        W1=(THE2(ITEMP,NTRAN)-TEMPL)/
!     *     (THE2(ITEMP,NTRAN)-THE2(ITEMP-1,NTRAN))
!        S1=W1*      SHE2(J,IELNUM-1,ITEMP-1,NTRAN)+
!     *     (1.0-W1)*SHE2(J,IELNUM-1,ITEMP  ,NTRAN)
!        S2=W1*      SHE2(J,IELNUM  ,ITEMP-1,NTRAN)+
!     *     (1.0-W1)*SHE2(J,IELNUM  ,ITEMP  ,NTRAN)
!
!        W1=(ELHE2(IELNUM,NTRAN)-ELNUM)/
!     *     (ELHE2(IELNUM,NTRAN)-ELHE2(IELNUM-1,NTRAN))
!
!        WORHE2(J)=W1*S1+(1.0-W1)*S2
!
!      ENDDO


!  Quadratic Temperature and density interpolation
!     ...solves "emission cores" in weak lines
!     ...badly formed wings of strong lines (Teff >= 45000 and log g > 4.5
!        probably due to structure of models -- small T inversions.  

      DO J=1,NWHE2(NTRAN)

        DO NEJ = IELNUM-1,IELNUM+1
          XVAR(1:3)=DBLE(THE2(ITEMP-1:ITEMP+1,NTRAN))
          YVAR(1:3)=SHE2(J,NEJ,ITEMP-1:ITEMP+1,NTRAN)
          CALL POLY(XVAR,YVAR,TEMPL,STE(NEJ-IELNUM+2))
        ENDDO
        XVAR(1:3)=DBLE(ELHE2(IELNUM-1:IELNUM+1,NTRAN))
        YVAR(1:3)=STE(1:3)
        CALL POLY (XVAR,YVAR,ELNUM,WORHE2(J))
!       prevent extrapolation ... 
!        IF ( ELNUM > ELHE2(NDHE2(NTRAN),NTRAN) ) WORHE2(J) = STE(3)

      ENDDO

!  Debug output
      IF(BUGLN) THEN
        WRITE(NDEBUG,901)(WHE2(I,NTRAN),WORHE2(I),I=1,NWHE2(NTRAN))
      ENDIF

      WLIM = WHE2(NWHE2(NTRAN),NTRAN)

!  Interpolation in wavelength
      DO I=1,NW

      DELW=ABS(W(I))

        IF (DELW.LE.WHE2(1,NTRAN)) THEN

!       Small Wavelength
          WORK(I)=10.0**WORHE2(1)

        ELSEIF (DELW.LE.WLIM) THEN

!       Large Wavelengths
          DO IWAV=2,NWHE2(NTRAN)
            IF(DELW.LE.WHE2(IWAV,NTRAN)) GOTO 12
          ENDDO
          IWAV=NWHE2(NTRAN)

!       Wavelength interpolation
   12     W1=(WHE2(IWAV,NTRAN)-DELW)/
     *       (WHE2(IWAV,NTRAN)-WHE2(IWAV-1,NTRAN))
          S1=W1*WORHE2(IWAV-1)+(1.0-W1)*WORHE2(IWAV)

          WORK(I)=10.0**S1

        ENDIF

      ENDDO


!  Debug output
      IF(BUGLN) THEN
        WRITE(NDEBUG,902)(W(I),WORK(I),LOG10(WORK(I)),I=1,NW)
      ENDIF


!  Scale profile
      WORK = CONST * WORK


!  Debug format statements

  900 FORMAT(' ***** HEII ***** TRANSITION AND PLASMA PARAMETERS'/
     *       ' NTRAN=',I3,5X,'WAVE=',F10.2,5X,'CONST=',1PE12.3,5X,
     *       ' IELNUM=',I3,5X,'ELNUM=',1PE10.3,
     *       ' ITEMP=',I3,5X,'TEMPL=',0PF10.3)
  901 FORMAT(' ***** HEII ***** PROFILE ON HE II WAVELENGTH GRID'/
     *(1P,9E12.5))
  902 FORMAT(' ***** HEII ***** WAVELENGTHS AND LINE OPACITY'/
     *(1P,9E12.5))

      END
