      SUBROUTINE HEIVOIGT (NW,W,WORK, ELNUM,EWID,CORR,DEL1,DEL2,DOPP )

!+
!   HEIVOIGT
!
!=  returns Voigt profiles for HeI lines, including triplet splitting.
!   Used at low densities when tabulated data is not available.
!   Necessary to check EWID, CORR and DEL are valid for low densities.
!
!   For triplets, del1, del2 give shifts for subordinate lines
!   the component strengths are distributed in the ratio 5:3:1.
!
!-

!  Double precision 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Subroutine Arguments.
      INTEGER          :: NW      ! (given) number of wavelengths in profile
      DOUBLE PRECISION, DIMENSION(*) :: W    ! (given) wavelengths in profile
      DOUBLE PRECISION, DIMENSION(*) :: WORK ! (returned) line opacity profile

      DOUBLE PRECISION :: ELNUM   ! electron density               (cm^-3)
      DOUBLE PRECISION :: EWID    ! electron damping width         (units??)
      DOUBLE PRECISION :: CORR    ! correction for ion broadening  ??
      DOUBLE PRECISION :: DEL1, DEL2  ! splitting in triplet
      DOUBLE PRECISION :: DOPP    ! thermal Doppler width          (A)

!.
      
!  Voigt profile
      AVO=ELNUM*EWID*(1.0+CORR)/DOPP
      DO I=1,NW
        VO=ABS(W(I)/DOPP)
        WORK(I)=VOIGT(AVO,VO)/DOPP
      ENDDO

!  Check for triplet
      IF(ABS(DEL1).GT.0.01) THEN
        DO I=1,NW
          V1 = ABS(W(I)-DEL1)/DOPP
          V2 = ABS(W(I)-DEL2)/DOPP
          WORK(I) = (5.0*WORK(I)
     :               + (3.0*VOIGT(AVO,V1)+VOIGT(AVO,V2))/DOPP)/9.0
        ENDDO
      ENDIF

      END SUBROUTINE HEIVOIGT
