SUBROUTINE INTEG_AVLO

  !+
  !=      Calculates the surface fluxes and the continuum optical depths
  !       where the line optical depth is approximately unity
  !-

  !  Global variables
  USE SPC_GLOBAL

  !  Double precision version
  IMPLICIT REAL*8 (A-H,O-Z)

  !  Local variables.
  DOUBLE PRECISION, DIMENSION(KD) :: CAP, SIG, BNU, TEG, ALQ
  DOUBLE PRECISION, DIMENSION(KD) :: OPC, TAUN,CAC

  !  Local temp variables for opacity interface (real)
  INTEGER  :: JD      ! Depth index
  INTEGER  :: IWL     ! OP Wavelenth index
  REAL     :: TTJ     ! Gas temperature
  REAL     :: PGJ     ! Gas pressure
  REAL     :: PEJ     ! Electron pressure
  REAL     :: WCJ     ! Wavelength
  REAL     :: CAPJ    ! bf+ff opacity
  REAL     :: SIGJ    ! e-s opacity

  !      Preset previous values
  DATA NDEPTP/0/,TEMPP/.0/,PGP/.0/,AHEP/10./,RLAMP/-1./,ONE/1./

  !.

  !      Check if same model and wavelength

  TAU1 = TAU(1)

  !       Recalculate continuous opacity, optical depth scale and flux

  !      Planck function
  FRE = 2.997927E+18 / RLAMB
  CALL PLANCK (TEMP,FRE,BNU,NDEPTH)

  !      Scaling factors
  DO J=2,NDEPTH-1
     ALQ(J) = LOG(TAU(J+1)/TAU(J))
  ENDDO
  ALQ(1) = ALQ(2)
  ALQ(NDEPTH) = ALQ(NDEPTH-1)
  TAU(1) = EXP(LOG(TAU(2))-ALQ(2))

  !      Opacity
  WCJ = RLAMB
  DO J=1,NDEPTH
     TTJ=TEMP(J)
     PGJ=PG(J)
     PEJ=PE(J)
     OPC(J)=OPAC(TTJ,PGJ,PEJ,WCJ,CAPJ,SIGJ,J,NOPTYP,1)
     TEG(J)=OPC(J)/OP(J)*TAU(J)*ALQ(J)
     CAP(J)=CAPJ
     SIG(J)=SIGJ
  ENDDO

  !      Optical depth
  TAUN(1)=0.5*TAU(1)*OPC(1)/OP(1)
  IF (SIG(1).GT.CAP(1)) TAUN(1)=2.*TAUN(1)
  CALL INGRAT (NDEPTH,ONE,TEG,TAUN)

  !      Calculate continuum flux
  CALL AVLOL (BNU,CAP,SIG,TAUN,CONFL,NDEPTH,ONE)
  CONFLP=CONFL


  !      Check for continuum only calculation

  IF (NW == 0) RETURN

  !      Line optical depth scale calculated assuming 
  !      constant continuous opacity

  !     Cycle over wavelength
  DO I=1,NW

     !      Calculate line plus continuum optical depth
     DO J=1,NDEPTH
        TEG(J) = (RLMU(I,J)+CAP(J)+SIG(J))/OP(J)*TAU(J)*ALQ(J)
        CAC(J) = (RLMU(I,J)+CAP(J))
     ENDDO
     TAUN(1)=0.5*TAU(1)*(RLMU(I,1)+CAP(1)+SIG(1))/OP(1)
     IF (SIG(1).GT.CAC(1)) TAUN(1)=2.*TAUN(1)
     CALL INGRAT (NDEPTH,ONE,TEG,TAUN)

     !      Calculate line flux
     CALL AVLOL (BNU,CAC,SIG,TAUN,FLUX(I),NDEPTH,ONE)

     !      Continuum optical depth where line depth is unity
     DO J=2,NDEPTH
        IF(TAUN(J) > 1.0 .AND. TAUN(J-1) < 1.0) THEN
           TAUL(I)=TAU(J-1)+(TAU(J)-TAU(J-1))* &
                   (1.-TAUN(J-1))/(TAUN(J)-TAUN(J-1))
        ENDIF
     ENDDO

  ENDDO

  !     Reset tau(j=1)
  TAU(1) = TAU1

END SUBROUTINE INTEG_AVLO
