SUBROUTINE INTEG_FEAUTRIER

  !     +     
  !     integ_feautrier
  !     
  !     
  !     =    Calculates the surface fluxes and the continuum optical depths
  !     where the line optical depth is approximately unity
  !     -

  !  Global variables
  USE SPC_GLOBAL

  !  Double precision version
  ! IMPLICIT REAL*8 (A-H,O-Z)
  IMPLICIT NONE

  !  Wavelength interval for continuum calculations
  REAL, PARAMETER :: WCRES = 500.  ! resolution in continuum sepctrum
  REAL, PARAMETER :: WCINT = 1. / WCRES
  INTEGER, PARAMETER :: KWC   = INT ( KW / WCRES )

  !  Local temp variables for opacity interface (real)
  INTEGER  :: JD      ! Depth index
  INTEGER  :: IWL     ! OP Wavelenth index
  REAL     :: TTJ     ! Gas temperature
  REAL     :: PGJ     ! Gas pressure
  REAL     :: PEJ     ! Electron pressure
  REAL     :: WCJ     ! Wavelength
  REAL     :: CAPJ    ! bf+ff opacity
  REAL     :: SIGJ    ! e-s opacity

  !  Local counters
  INTEGER :: I, IJ, J, N

  INTEGER :: NTH      ! Number of angles
  INTEGER :: NWC      ! Number of wavelengths

  !  Local variables for rt_feaut interface  (double precision)
  DOUBLE PRECISION :: FRE     ! frequency
  DOUBLE PRECISION :: TAUNU   ! optical depth

  !  Local arrays

  !  Opacity data
  DOUBLE PRECISION CAP, SIG, OPC, OPL, TAUN
  DIMENSION CAP(KD),SIG(KD),OPC(KD),OPL(KD),TAUN(KD)

  !  Wavelength opacity data
  DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: CAPC, SIGC, OPCC

  !  Angle-frequency data for calculating the source function
  !  Coherent scattering
  INTEGER :: NM = 4
  DOUBLE PRECISION, DIMENSION(10) :: XNU  = 0.
  DOUBLE PRECISION, DIMENSION(10) :: XA   = (/ 1., 0.,0.,0., 0.,0.,0., 0.,0.,0.  /)
  DOUBLE PRECISION, DIMENSION(10) :: XPHI = (/ 1., 0.,0.,0., 0.,0.,0., 0.,0.,0.  /)
  DOUBLE PRECISION, DIMENSION(16) :: XMU  = 0.
  DOUBLE PRECISION, DIMENSION(16) :: XB   = 0.

  !  Source function, etc.
  DOUBLE PRECISION S, BNU, JNU
  DIMENSION S(KD), BNU(KD), JNU(KD)

  DOUBLE PRECISION WC, FC, IMUC, FC2
  DIMENSION WC(KWC),FC(KWC),IMUC(KWC,KTH),FC2(KWC)

  DOUBLE PRECISION CKAP, CSIG
  DIMENSION CKAP(KWC,KD), CSIG(KWC,KD)

  DOUBLE PRECISION COST, RI, CONI
  DIMENSION COST(KTH), RI(KTH), CONI(KTH)

  !  opacity edge variables
  INTEGER, PARAMETER  :: EJ_MAX = 50
  DOUBLE PRECISION, DIMENSION(EJ_MAX) :: EJ_WL ! opacity edge wavelengths
  DOUBLE PRECISION, DIMENSION(EJ_MAX) :: EJ_ID ! opacity edge identities
  INTEGER :: EJ_N           ! number of op edges 

  !  Preset previous values
  INTEGER, SAVE :: NDEPTP = 0.
  REAL, SAVE :: TEMPP = 0. 
  REAL, SAVE :: PGP  = 0.
  REAL, SAVE :: AHEP = 0.
  REAL, SAVE :: RLAMP = 0.

  INTEGER :: I_OPWL = 0

  !  External functions
  REAL :: OPAC

  !.

  !     Local storage for cosine angles
  NTH = NTHETA
  COST(1:NTH)=COSTHETA(1:NTH)

  IF ( BUGIN ) THEN
     write (NDEBUG,*) ' INTEG_FEAUTRIER **** RLMU '
     write (NDEBUG,*) ' RLAMB ',RLAMB
     write (NDEBUG,'(1P,10E10.3)') (rlmu(i,1:ndepth),i=1,10)
  ENDIF

  !     Recalculate continuous opacity, optical depth scale and flux
  !     for line center
  FRE = 2.997927E+18 / RLAMB
  TAUNU = 2.997927E+18 / TAU_WL

  I_OPWL = -1      ! ensures OP_OP_FFBF locates WL, possibly by interpolation

  !     SINGLE LINE MODE
  IF ( NMODE == 0 ) THEN

     !     Wavelength offset in single line mode
     W = W + RLAMB

     !     CONTINUUM FLUXES
     !     continuum opacity at line centre
     !     (not ideal for very broad linesi bot affects other routines severely).
     IC = 1
     WC(IC) = RLAMB

     !     Calculate opacity
     WCJ = RLAMB
     OPL = 0
     DO J=1,NDEPTH
        TTJ=TEMP(J)
        PGJ=PG(J)
        PEJ=PE(J)
        OPC(J)=OPAC(TTJ,PGJ,PEJ,WCJ,CAPJ,SIGJ,J,NOPTYP,I_OPWL)
        CAP(J)=CAPJ
        SIG(J)=SIGJ
     ENDDO

     !     Calculate source function
     CALL RTF_FEAUT (TAUNU,TAU,TEMP,PE,OP,NDEPTH,  &
          FRE,CAP,SIG,OPL,1,   XNU,XA,XPHI,1,   XMU,XB,NM,   TAUN,S,JNU)

     !     Calculate continuum flux 
     CALL TRANSFER (TAUN,S,NDEPTH, COST,CONI,NTH, CONFL)

     !     Save as constant values
     FC(1) = CONFL
     NWC = 1
     FLUXC = CONFL


     !     Check for continuum only calculation
     IF (NW == 0) THEN
        !     Store specific continuum intensities
        IF (NTH > 0) THEN
           INTENC(1,1:NTH)=CONI(1:NTH)
        ENDIF
        W = W - RLAMB
        RETURN
     ENDIF


     !     LINE FLUXES

     !     Cycle over wavelength
     DO I=1,NW
        OPL(1:NDEPTH) = RLMU(I,1:NDEPTH)

        !     Calculate source function
        CALL RTF_FEAUT (TAUNU,TAU,TEMP,PE,OP,NDEPTH, & 
             FRE,CAP,SIG,OPL,1,   XNU,XA,XPHI,1,   XMU,XB,NM,   TAUN,S,JNU)

        !     Calculate line flux
        CALL TRANSFER (TAUN,S,NDEPTH, COST,RI,NTH, FLUX(I))

        !     Specific line intensities
        IF (NTH > 0) THEN
           INTENS(I,1:NTH)=RI(1:NTH)
           INTENC(I,1:NTH)=CONI(1:NTH)
        ENDIF

        !     Continuum optical depth where line depth is unity
        DO J=2,NDEPTH
           IF (TAUN(J) > 1.0 .AND. TAUN(J-1) <= 1.0) THEN
              TAUL(I)=TAU(J-1)+(TAU(J)-TAU(J-1)) * (1.-TAUN(J-1))/(TAUN(J)-TAUN(J-1))
              EXIT
           ENDIF
        ENDDO

        !     Debug fluxes            
        IF ( BUGIN ) THEN
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** OPL, OPC, W = ',W
           write (NDEBUG,'(1P,10E10.3)') opl(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') opc(1:ndepth)
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** TAU, S, J, TAUL'
           write (NDEBUG,'(1P,10E10.3)') taun(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') s(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') jnu(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') taul(i)
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** TAU,T,P,OP'
           write (NDEBUG,'(1P,10E10.3)') tau(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') temp(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') pe(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') op(1:ndepth)
        ENDIF

     ENDDO

     !     Remove wavelength offset in single line mode
     W = W - RLAMB




     !     SPECTRUM SYNTHESIS
  ELSE

     !     CONTINUUM FLUXES

     !     Locate opacity edges in large syntheses
     EJ_N = 0
     IF ( W(1) > 0.1 .AND. W(NW)/W(1) > 1.1 ) THEN
        CALL OP_EDGEFIND ( W(1), W(NW), EJ_MAX, EJ_WL, EJ_ID, EJ_N )
        write (*,'(I4,A)' ) EJ_N, ' Opacity Edges Located in Synthetic Spectrum'
     ENDIF


     IC = 2
     WC(1) = W(1)
     WC(2) = W(1) * ( 1 + WCINT )
     DO WHILE ( WC(IC) < W(NW) )

        !     check for opacity edges
        IF ( EJ_N > 0 ) THEN
           DO IJ = 1, EJ_N
              IF ( WC(IC-1) < EJ_WL(IJ) .AND. EJ_WL(IJ) < W(IC-1) * ( 1 + WCINT ) ) THEN
                 WC(IC+1) = EJ_WL(IJ) * (1.-WCINT*1.E-7) 
                 WC(IC+2) = EJ_WL(IJ) * (1.+WCINT*1.E-7)
                 IC = IC+2
              ENDIF
           ENDDO
        ENDIF

        !     insert normal continuum wavelength
        IC = IC + 1
        WC(IC) = WC(IC-1) * ( 1 + WCINT )

     ENDDO
     IC = IC + 1
     WC(IC) = W(NW)
     NWC = IC

     ALLOCATE(CAPC(NDEPTH,NWC))
     ALLOCATE(SIGC(NDEPTH,NWC))


     !     Cycle over wavelengths
     DO IC = 1,NWC
        FRE = 2.997927E+18 / WC(IC)

        !     Continuous opacity
        !     write(*,*) 'integ feaut', WC(IC), IC
        !     Calculate opacity
        WCJ = WC(IC)
        OPL = 0.
        DO J=1,NDEPTH
           TTJ=TEMP(J)
           PGJ=PG(J)
           PEJ=PE(J)
           OPC(J)=OPAC(TTJ,PGJ,PEJ,WCJ,CAPJ,SIGJ,J,NOPTYP,I_OPWL)
           CAP(J)=CAPJ; CAPC(J,IC)=CAPJ
           SIG(J)=SIGJ; SIGC(J,IC)=SIGJ
           OPL(J)=0.
        ENDDO

        !     Calculate source function
        CALL RTF_FEAUT (TAUNU,TAU,TEMP,PE,OP,NDEPTH,  &
             FRE,CAP,SIG,OPL,1,   XNU,XA,XPHI,1,   XMU,XB,NM,   TAUN,S,JNU)

        !     Calculate continuum flux 
        CALL TRANSFER (TAUN,S,NDEPTH, COST,CONI,NTH, CONFL)

        !     Store continuum fluxes (and intensities)
        FC(IC) = CONFL
        IF (NTH > 0) THEN
           IMUC(IC,1:NTH)=CONI(1:NTH)
        ENDIF

     ENDDO


     !     Interpolate continuum fluxes (and intensities) onto LINE grid
     !     Currently uses linear interpolation: we did try splines but without
     !     too much success. 

     IF ( BUGIN ) THEN
        write (NDEBUG,*) ' INTEG_FEAUTRIER **** NWC ',NWC
        write (NDEBUG,*) ' INTEG_FEAUTRIER **** WC '
        write (NDEBUG,'(10F10.3)') wc(1:nwc)
        write (NDEBUG,*) ' INTEG_FEAUTRIER **** FC '
        write (NDEBUG,'(1P,10E10.3)') fc(1:nwc)
        write (NDEBUG,*) ' INTEG_FEAUTRIER **** IMUC '
        do i = 1,nth
           write (NDEBUG,'(1P,10E10.3)') imuc(1:nwc,n)
        ENDDO
     ENDIF

     DO I = 1,NW
        CALL MTH_D_TABINT ( NWC,WC,FC,1,W(I),FLUXC(I) ) 
     ENDDO

     IF (NTH > 0) THEN
        DO N=1,NTH
           FC(1:NWC)=IMUC(1:NWC,N)
           DO I = 1,NW
              CALL MTH_D_TABINT ( NWC,WC,FC,1,W(I),INTENC(I,N) ) 
           ENDDO
        ENDDO
     ENDIF



     !     LINE FLUXES


     !     Cycle over wavelength
     DO I=1,NW

        !     Frequency
        FRE = 2.997927E+18 / W(I)


        !     Continuum + Line opacity        

        !  Line opacity
        OPL(1:NDEPTH) = RLMU(I,1:NDEPTH)

        !     Calculate opacity
        WCJ=W(I)
        DO J = 1,NDEPTH
        !   TTJ=TEMP(J)
        !   PGJ=PG(J)
        !   PEJ=PE(J)
        !   OPC(J)=OPAC(TTJ,PGJ,PEJ,WCJ,CAPJ,SIGJ,J,NOPTYP,I_OPWL)
        !   CAP(J)=CAPJ
        !   SIG(J)=SIGJ
        !  experiment with intrepolating the continuous opacity
           CALL MTH_D_TABINT ( NWC,WC,CAPC(J,:),1,W(I),CAP(J) ) 
           CALL MTH_D_TABINT ( NWC,WC,SIGC(J,:),1,W(I),SIG(J) ) 
        ENDDO

        IF ( BUGIN ) THEN
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** OPL, OPC '
           write (NDEBUG,'(1P,10E10.3)') opl(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') opc(1:ndepth)
        ENDIF

        !     Calculate source function
        CALL RTF_FEAUT (TAUNU,TAU,TEMP,PE,OP,NDEPTH,  &
             FRE,CAP,SIG,OPL,1,   XNU,XA,XPHI,1,   XMU,XB,NM,   TAUN,S,JNU)

        !     Calculate line flux
        CALL TRANSFER (TAUN,S,NDEPTH, COST,RI,NTH, FLUX(I))

        !     Store specific line intensities
        IF (NTH > 0) THEN
           INTENS(I,1:NTH)=RI(1:NTH)
        ENDIF


        !     Continuum optical depth where line depth is unity
        DO J=2,NDEPTH
           IF(TAUN(J) > 1.0 .AND. TAUN(J-1) < 1.0) THEN
              TAUL(I)=TAU(J-1)+(TAU(J)-TAU(J-1)) * (1.-TAUN(J-1))/(TAUN(J)-TAUN(J-1))
           ENDIF
        ENDDO

        !     Debug fluxes            
        IF ( BUGIN ) THEN
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** OPL, OPC '
           write (NDEBUG,'(1P,10E10.3)') opl(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') opc(1:ndepth)
           write (NDEBUG,*) ' INTEG_FEAUTRIER **** TAU, S, J '
           write (NDEBUG,'(1P,10E10.3)') taun(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') s(1:ndepth)
           write (NDEBUG,'(1P,10E10.3)') jnu(1:ndepth)
        ENDIF

     ENDDO

     DEALLOCATE (CAPC)
     DEALLOCATE (SIGC)

  ENDIF


  !     debug statements
  IF ( BUGIN ) THEN       
     WRITE (NDEBUG,*) ' INTEG_FEAUTRIER **** W,F/FC,F,FC '
     DO I = 1,NW
        WRITE (NDEBUG,'(2F10.3,1P,2E11.3)') W(I),FLUX(I)/FLUXC(I),FLUX(I),FLUXC(I)
     ENDDO
  ENDIF

END SUBROUTINE INTEG_FEAUTRIER
