SUBROUTINE LHEI

  !+
  !=      Calculates line opacity for individual helium diffuse lines
  !-

  !       Double precision version

  !    Public constants
  USE SPC_CONSTANTS

  !   Global variables
  USE SPC_GLOBAL
  USE SPC_CONTROL

  !   Implicit typing
  IMPLICIT REAL*8 (A-H,O-Z)

  !       BLEND  temporary store for opacity
  !       OPLINE temporary storage for quadratic interpolation
  !       EQUIL  will contain the ratio of the i to (i-1) ionization stage
  !              with EQUIL(1)=1
  !       NBL    contains indexes of blending components

  DIMENSION EQUIL(5)

  DOUBLE PRECISION :: CLAM2  ! c. lambda^2 in appropriate units
  DOUBLE PRECISION :: XABUND ! scaled abundance (alpha/mu)
  DOUBLE PRECISION :: ABUNDP ! saved value of abundance

  DATA EQUIL(1)/1.0/,TEMPP/0.0/

  !.

  !       Checks if line opacity can be scaled

  IF(NDEPTH==NDEPTP.AND. &
       NW==NWP       .AND. &
       NSP==NSPP     .AND. &
       RLAMB==RLAMBP .AND. &
       TEMP(1)==TEMPP.AND. &
       PG(1)==PGP    .AND. &
       VT==VTP       .AND. &
       .NOT. QUB__QSTRAT(NSP)  ) THEN


     !      Scale line opacity if only an abundance change

     RABUND=ABUND/ABUNDP
     DO J=1,NW
        RLMU(J,1:NDEPTH)=RABUND*RLMU(J,1:NDEPTH)
     ENDDO
     ABUNDP=ABUND
     RETURN

  ENDIF


  !       Calculates constant in line opacity

  CLAM2=CLOPAC*RLAMB*RLAMB
  XABUND = ABUND / AMU



  !       Cycle over depth

  DO I=1,NDEPTH

     !      Extract abundances if stratified

     IF (QUB__QSTRAT(NSP)) XABUND = QUB__ABSTRAT(NSP,I)/QUB__AMU(I)


     !       Boltzmann equation

     U_J = TAP_PARFN(NSP,0,TEMP(I),PE(I)/10.)
     FEX=GFHE(IDHE)/U_J*EXP(-CEVTOT*EXCHE(IDHE)/TEMP(I))


     !       Ionization equilibria


     BT=CSAHAE*(TEMP(I)**2.5)/PE(I)
     DO J=2,3
        U_K = U_J
        U_J = TAP_PARFN(NSP,J-1,TEMP(I),PE(I)/10.)
        EQUIL(J)=BT*U_J/U_K*EXP(-CEVTOT*RION(J-1,NSP)/TEMP(I))
     ENDDO


     !       FION is the CONST*(fractional population of the lower level)
     !                        *(stimulated emission correction)

     FION=CLAM2*XABUND*FEX*(1.-EXP(-CSTIME/(TEMP(I)*RLAMB)))/ &
          (EQUIL(1)+EQUIL(2)*(1.+EQUIL(3)))


     !      Calculate line opacity

     CALL HEIDIF( TEMP(I),PE(I),QUB__VTRB(I) )

     DO J=1,NW
        RLMU(J,I)=FION*WORK(J)
     ENDDO

     !       Debug output

     IF(BUGLN) THEN
        WRITE(NDEBUG,900)I,(RLMU(J,I),J=1,NW)
     ENDIF

  ENDDO


  !       Reset variable for checking on next entry

  NDEPTP=NDEPTH
  NWP=NW
  NSPP=NSP
  RLAMBP=RLAMB
  TEMPP=TEMP(1)
  PGP=PG(1)
  VTP=VT
  ABUNDP=ABUND


  !       Debug output

900 FORMAT(' ** LHEI **',5X,'DEPTH=',I5//50(1P,10E12.5/))

END SUBROUTINE LHEI
