SUBROUTINE LMETAL

  !+
  !=      Calculates metal line opacity
  !       for individual lines and close blends
  !-
  !       Double precision version

!    Public constants
      USE SPC_CONSTANTS

!    Global variables
      USE SPC_GLOBAL

  IMPLICIT REAL*8 (A-H,O-Z)

  !  maximum number of components that may contribute to a blend
  INTEGER KBL
  PARAMETER (KBL = 10)

  !       BLEND  temporary store for opacity
  !       OPLINE temporary storage for quadratic interpolation
  !       NBL    contains indexes of blending components
  !       EQUIL  will contain the ratio of the i to (i-1) ionization stage
  !              with EQUIL(1)=1
  !       UZ     temporary stoage for partition functions

  DIMENSION BLEND(500),NBL(KBL),OPLINE(KBL),EQUIL(5),UZ(5)
  LOGICAL INTERP

  DOUBLE PRECISION :: CLAM2  ! c. lambda^2 in appropriate units
  DOUBLE PRECISION :: XABUND ! scaled abundance (alpha/mu)
  DOUBLE PRECISION :: ABUNDP ! saved value of abundance


  DATA EQUIL(1)/1.0/,TEMPP/0.0/

  !.

  !       INTER=.TRUE.  - blending contribution is interpolated
  !             .FALSE. - blending contribution is recalculated

  INTERP=.FALSE.


  !       Calculates constant in line opacity

  CLAM2=CLOPAC*RLAMB*RLAMB
  XABUND = ABUND / AMU

  !       Identify transition

  DO  I=1,KL
     NLINE=I
     IF(NSP==ISP(I).AND.ABS(RLAMB-WAVE(I))<WTOL) EXIT
     NLINE=0
  ENDDO


  !       Error message if line is not found

  IF (NLINE==0) THEN
     WRITE(NWRITE,200) NSP,RLAMB
200  FORMAT ('LMETAL: ',I3,F9.2,' line not found: opacity set to zero') 
     RLMU = 0.0
     RLMU(1,1) = -1.0
     ABUNDP=ABUND
  ELSE


     !      Checks for close blends - maximum of KBL+1 components
     !        but only in single-line mode

     NBLEND=0
     IF (NMODE==0 .OR.NMODE==1.OR.NMODE==20.OR.NMODE==21) THEN
        DO I=1,KL                                         ! read through linelist
           IF (NSP==ISP(I).AND.NION(NLINE)==NION(I)) THEN ! compare species
              IF (ABS(ZMULT(NLINE)-ZMULT(I))<0.01 .AND.&
                   ABS(EXCIT(NLINE)-EXCIT(I))<0.01) THEN    ! compare multiplet
                 IF (ABS(WAVE(NLINE)-WAVE(I))<WTOL) THEN     ! compare wavelength
                    IF (I/=NLINE) THEN                          ! don't double count
                       IF (NBLEND<=KBL) THEN
                          NBLEND=NBLEND+1
                          NBL(NBLEND)=I
                       ENDIF
                    ENDIF
                 ENDIF
              ENDIF
           ENDIF
        ENDDO
     ENDIF


     !       Cycle over depth

     DO I=1,NDEPTH


        !      Extract abundances if stratified

        IF (QUB__QSTRAT(NSP)) XABUND = QUB__ABSTRAT(NSP,I)/QUB__AMU(I)

        !       ELNUM = Electron density times 10**-15

        ELNUM=CELNUM*PE(I)/TEMP(I)


        !       Doppler width (b) including microturbulence
        !       i.e. line opacity proportional to exp(-(delta lambda/b)**2)

        VT2 = QUB__VTRB(I) * QUB__VTRB(I)
        DOPP=RLAMB*SQRT(CDOPPT*TEMP(I)/ATW(NSP)+CDOPPV*VT2)


        !       Partition functions

        DO J = 1,MIN(5,NSP+1)
           UZ(J) = TAP_PARFN ( NSP,J-1,TEMP(I),PE(I)/10. )
        ENDDO

        !       Boltzmann equation

        FEX=GF(NLINE)/UZ(NION(NLINE))*EXP(-CEVTOT*EXCIT(NLINE)/TEMP(I))


        !       Ionization equilibria


        BT=CSAHAE*(TEMP(I)**2.5)/PE(I)
        EQUIL(4) = 0.
        EQUIL(5) = 0.
        DO J=2,MIN(5,NSP+1)
           EQUIL(J)=BT*UZ(J)/UZ(J-1)*EXP(-CEVTOT*RION(J-1,NSP)/TEMP(I))
        ENDDO


        !       FION is the CONST*(fractional population of the lower level)
        !                        *(stimulated emission correction)

        FION=CLAM2*XABUND*FEX*(1.-EXP(-CSTIME/(TEMP(I)*RLAMB)))/ &
             (EQUIL(1)+EQUIL(2)*(1.+EQUIL(3)*(1.+EQUIL(4)*(1.+EQUIL(5)))))
        DO J=1,NION(NLINE)
           FION=FION*EQUIL(J)
        ENDDO


        !       Calculate a-value in voigt function

        AVO=(EDAM(NLINE)*ELNUM+RDAM(NLINE))/DOPP


        !      Calculate line opacity of principle (or only) component


        DO J=1,NW
           VO=ABS(W(J))/DOPP
           RLMU(J,I)=FION*VOIGT(AVO,VO)/DOPP
        ENDDO


        !          Check for blend

        IF(NBLEND.NE.0) THEN


           !          Cycle over wavelength

           DO J=1,NW


              !          Initialize array

              BLEND(J)=0.0


              !           Cycle round components

              DO K=1,NBLEND

                 NBLK=NBL(K)
                 DWLBL=WAVE(NBLK)-WAVE(NLINE)
                 WTEMP=W(J)-DWLBL
                 RATBLK=GF(NBLK)/GF(NLINE)


                 !             Check for interpolation or recalculation

                 IF(INTERP.EQV..TRUE.) THEN


                    !             Find wavelengths in main component appropiate to blend

                    DO L=1,NW-2
                       IF(WTEMP.LE. W(L))GOTO 11
                    ENDDO
                    L=NW-2


                    !                  Quadratic interpolation  of
                    !                  logarithmic line opacity against wavelength

11                  OPLINE(1)=LOG(RLMU(L  ,I))
                    OPLINE(2)=LOG(RLMU(L+1,I))
                    OPLINE(3)=LOG(RLMU(L+2,I))

                    CALL POLY(W(L:L+2),OPLINE,WTEMP,BLTEMP)


                    !                  Add component to the blend

                    BLEND(J)=BLEND(J)+RATBLK*EXP(BLTEMP)

                 ELSE
                    VO=ABS(WTEMP)/DOPP
                    BLEND(J)=BLEND(J) + FION*RATBLK*VOIGT(AVO,VO)/DOPP
                 ENDIF

              ENDDO

           ENDDO


           !       Debug output

           IF(BUGLN) THEN
              WRITE(NDEBUG,900)I,(RLMU(J,I),J=1,NW)
              WRITE(NDEBUG,901)I,(BLEND(J),J=1,NW)
           ENDIF

        ENDIF


        !         Add blend to line opacity


        IF(NBLEND.NE.0) THEN
           DO J=1,NW
              RLMU(J,I)=RLMU(J,I)+BLEND(J)
           ENDDO
        ENDIF
     ENDDO

  ENDIF

  !
  !       Format statements
  !
  !
  !       Debug output
  !
900 FORMAT(' ** LINOP **',5X,'DEPTH=',I5//16X,'UNBLEND COMPONENT'/50(1P,10E12.5/))
901 FORMAT(' ** LINOP **',5X,'DEPTH=',I5//16X,'BLEND COMPONENT'/50(1P,10E12.5/))

END SUBROUTINE LMETAL
