SUBROUTINE LSYNT

  !+
  !
  !=      Calculates line opacity for synthetic spectra
  !
  !-

!    Public constants
      USE SPC_CONSTANTS

!  Global variables
      USE SPC_GLOBAL
      USE SPC_VCSDATA      !! VCS tables
      USE OP_GLOBAL, ONLY : OP_XION, OP_XUU
      USE OP_GLOBAL, ONLY : OP__ALPMU

  !  Double precision version.
  IMPLICIT REAL*8 (A-H,O-Z)


  !  Local constants.

  !     Wavelength cutoff in Doppler units for calculating metal lines
  !       Cutoff is DOPCUT*AV0*DOPP = DOPCUT*GAMMA(FWHM) Angstroms 
  !       from line centre
  REAL DOPCUT             
  PARAMETER (DOPCUT = 1.E+5) 

  !     Cut off for n.gf to be used in line opacity calculation.
  !       the cutoff used is 10 times lower than would give a 
  !       0.4% error in the profiles of very weak lines.
  REAL GFNCUT
  PARAMETER (GFNCUT = 1.E-6)


  INTEGER :: XSP ! element identifier for the current line 
  DOUBLE PRECISION :: CLAM2  ! c. lambda^2 in appropriate units
  DOUBLE PRECISION :: XABUND ! scaled local abundance - alpha(i)/amu(i)
  DOUBLE PRECISION :: XAMU   ! local mean atomic weight - amu(i)

  INTEGER :: KMIN, KMAX      ! local loopp limits for line opacity calculation
  !.


  ! write(*,*) 'LSYNT  --  NDEPTH :',ndepth


  !   Re-initialize line opacity
  RLMU=0.0
  KMAX=NW

  !   Cycle over depth
  DO I=1,NDEPTH

     ELNUM=CELNUM*PE(I)/TEMP(I)
     FIONZ=CSAHAE*(TEMP(I)**2.5)/PE(I)

     !   Microturbulent velocity
     IF (NVTSY > 1) THEN 
        VT=VTSY(IVTSY)
     ELSE
        VT = QUB__VTRB(I)
     ENDIF

     !     Local mean atomic weight
     XAMU = OP__ALPMU(I)
     KMIN=1

     ! write(*,*) 'LSYNT  --  XAMU :',xamu
     ! write(*,*) 'LSYNT  --  ABUSY :',abusy(1:nelsy)
 
     !     Cycle over lines
     DO J=1,NELSY

        !     Line opacity constant
        CLAM2=CLOPAC*WAVSY(J)*WAVSY(J)

        !     Local elemental abundance scaled
        XSP = NSPSY(J)
        XABUND = ABUSY(J)
        IF (QUB__QSTRAT(XSP))  XABUND = QUB__ABSTRAT(XSP,I)
        XABUND = XABUND / XAMU
        ! write(*,*) 'LSYNT  --  XABUND :',xabund

        !     Check type of helium line
        IDHE = 0
        IDHE2 = 0
        IF (XSP==IHE) THEN
           RLAMBT=RLAMB
           RLAMB=WAVSY(J)
           CALL HETYPE
           RLAMB=RLAMBT
        ENDIF

        IH000 = 1000
        IHE00 = 1000

        !     Hydrogen line
        IF(XSP==IH) THEN

           !         Identify transistion
           CALL LHI_ID(WAVSY(J),NLOW,NUP)

           !         Boltzmann equation
           FEXC = GFSY(J)/OP_XUU(1,1,I) &
                * EXP(-CEVTOT*EH(NLOW)/TEMP(I)) &
                * XABUND &
                * (1.0-EXP(-CSTIME/(TEMP(I)*WAVSY(J))))

           !         FHION is the CLAM2*(fractional population of the lower level)
           !                           *(stimulated emission correction)
           FRAC=CLAM2*FEXC*OP_XION(1,1,I)

           !         Change to relative wavelengths
           WAVHI=WAVSY(J)
           W=W-WAVHI

           !         Call subroutine to calculate normalised hydrogen opacity
           CALL HSTARK(TEMP(I),PE(I),NLOW,NUP)

           !         Calculate opacity and change back to absolute wavelengths
           RLMU(1:NW,I)=RLMU(1:NW,I)+FRAC*WORK(1:NW)
           W=W+WAVHI


           !     Ionized Helium line   
        ELSEIF (XSP==IHE .AND. IDHE2>0) THEN

           !         Boltzmann equation
           FEXC = GFSY(J)/OP_XUU(2,2,I) &
                * EXP(-CEVTOT*EXCSY(J)/TEMP(I)) &
                * XABUND &
                * (1.-EXP(-CSTIME/(TEMP(I)*WAVSY(J))))

           !         FRAC is the CLAM2*(fractional population of the lower level)
           !                          *(stimulated emission correction)
           FRAC=CLAM2*FEXC*OP_XION(2,2,I)

           !         Change to relative wavelengths
           WAVHEII=WAVSY(J)
           W=W-WAVHEII

           !         Call subroutine to calculate normalised opacity
           CALL HE_II(PE(I),TEMP(I))

           !         Calculate opacity and change back to absolute wavelengths
           RLMU(1:NW,I)=RLMU(1:NW,I)+FRAC*WORK(1:NW)
           W=W+WAVHEII


           !    Neutral Helium lines
        ELSEIF (XSP==IHE) THEN

           DOPP=WAVSY(J)*SQRT(CDOPPT*TEMP(I)/ATW(XSP)+CDOPPV*VT*VT)

           !         Boltzmann equation
           FEXC = GFSY(J)/OP_XUU(1,2,I) &
                * EXP(-CEVTOT*EXCSY(J)/TEMP(I)) &
                * XABUND &
                * (1.-EXP(-CSTIME/(TEMP(I)*WAVSY(J))))


           FRAC=CLAM2*FEXC*OP_XION(1,2,I)

           !         Diffuse Helium line (or one having special treatment)
           IF(IDHE>0) THEN

              !           Change to relative wavelengths
              WAVHEI=WAVSY(J)
              W=W-WAVHEI

              !           Call subroutine to calculate normalised helium opacity
              RLAMBT=RLAMB
              RLAMB=WAVSY(J)
              CALL HEIDIF( TEMP(I),PE(I),VT ) 
              RLAMB=RLAMBT

              !           Calculate opacity and change back to absolute wavelengths
              RLMU(1:NW,I)=RLMU(1:NW,I)+FRAC*WORK(1:NW)
              W=W+WAVHEI

              !         Non-diffuse Helium line, treated as Voigt profile
           ELSE                 
              AV0=(ELNUM*EDAMSY(J)+RDAMSY(J))/DOPP

              DO K=1,NW
                 V0=ABS(W(K)-WAVSY(J))/DOPP
                 RLMU(K,I)=RLMU(K,I)+FRAC*VOIGT(AV0,V0)/DOPP
              ENDDO

           ENDIF


           !       Metal line
        ELSE

           FRAC = CLAM2  &                              ! constant bundle
                * GFSY(J) &                             ! gf - value 
                * EXP(-CEVTOT*EXCSY(J)/TEMP(I)) &       ! excitation potential
                / OP_XUU(NIONSY(J),XSP,I)  &        ! partition function
                * XABUND  &                             ! reduced abundance
                * (1.0-EXP(-CSTIME/(TEMP(I)*WAVSY(J)))) & ! stimulated emission
                * OP_XION(NIONSY(J),XSP,I)           ! ion fraction

           !          ,only treat line if strong enough:
           IF (FRAC>GFNCUT) THEN

              !            .Inverse Doppler width of line (to save divides)
              DOPP=WAVSY(J)*SQRT(CDOPPT*TEMP(I)/ATW(XSP)+CDOPPV*VT*VT)
              DOPI=1.D0/DOPP

              !            ,Damping parameter
              AV0 = ELNUM*EDAMSY(J) + RDAMSY(J)  ! +VDWC(I)*WDAM(J)

              !            .compute the line cutoff limit:  sqrt(DOPCUT*FRAC*GAMMA(HWHM))
              !
              !             The line profile goes as gamma / delta lamda^2, so the cutoff
              !             in delta lambda should go as sqrt (av0). 
              !
              !             I tested various forms for the following function, and
              !             different values for DOPCUT, using several thousand lines
              !             from very weak to exceptionally strong resonance lines. 
              !
              VMAX = SQRT(FRAC*DOPCUT*AV0)

              !            .Debug statements
              !             write (*,'(i3,i5,i3,i2,f9.3,1p,9e10.2)')i,j,
              !    :             nspsy(j),nionsy(j),
              !    :             wavsy(j),GFSY(J),
              !    :             frac,abusy(j,iabsy),amu,
              !    :             edamsy(j),rdamsy(j),
              !    :             AV0, VMAX

              !            .increment the total line opacity using Voigt function
              WAVELENGTH: DO K=KMIN,KMAX
                 !  IF (W(K)<WAVSY(J)-10.0) KMIN = K ; this was truncating strong UV resoncance lines
                 IF (W(K)<WAVSY(J)-VMAX) CYCLE WAVELENGTH
                 IF (W(K)>WAVSY(J)+VMAX) EXIT WAVELENGTH
                 V0=ABS(W(K)-WAVSY(J))
                 ! IF (V0<VMAX) THEN
                    RLMU(K,I)=RLMU(K,I)+FRAC*VOIGT(AV0*DOPI,V0*DOPI)*DOPI
                 ! ENDIF
              ENDDO WAVELENGTH
              ! write (*,*) j,wavsy(j),kmin,kmax,w(kmin),vmax

           ENDIF

        ENDIF

!        IF ( BUGLN .AND. I == 17 ) & 
        IF ( BUGLN ) & 
        WRITE(NDEBUG,'(2I3,F9.3,F7.3,4F8.3,F7.2,1P,4E10.2)',IOSTAT=LINE_STATUS) &
          XSP,NIONSY(J),WAVSY(J),GFSY(J),EDAMSY(J),RDAMSY(J),WDAMSY(J),EXCSY(J),ZMLTSY(J),FRAC, &
          OP_XUU(NIONSY(J),XSP,I), OP_XION(NIONSY(J),XSP,I), XABUND

     ENDDO

  ENDDO


  !  Format statements
200 FORMAT (' ***** WRONG VALUE OF RLAMB IN LSYNT',1P,E14.6// &
            ' ***** RUN ABORTED *****')
201 FORMAT(' OK H LINE HERE ')



END SUBROUTINE LSYNT
