      SUBROUTINE OUTDIS

!+
!
!  outdis
!
!= Output spectra and results to disc
!
!-

!  Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!  Logarithmic abundances for synthesis calculations
!     DIMENSION ABUSYL(KL)

!  reduced intensities for computing limb-darkening
      REAL, DIMENSION(KTH) :: INT_RED

!  Flag for first entry
      LOGICAL FIRST
      DATA FIRST/.TRUE./

!.

!  Check for first entry
      IF (FIRST) THEN
        OPEN(UNIT=NDISC,FILE='SPECTRUM.TMP',STATUS='UNKNOWN')
        OPEN(UNIT=NDISC+1,FILE='FLAMBDA.TMP',STATUS='UNKNOWN')
        OPEN(UNIT=NDISC+2,FILE='FCONTIN.TMP',STATUS='UNKNOWN')
        FIRST=.FALSE.
        IF(NMODE >= 20) THEN
        WRITE(NDISC,*) '  Z   IZ    WL     TEFF  LG_G LG_AHE LG_GF  ',
     :    ' MULT   VT  LG_AB    EW(A) '
        WRITE(NDISC,*)
        WRITE(NDISC+1,*) '  Z   IZ    WL     TEFF  LG_G LG_AHE LG_GF  ',
     :   ' MULT   VT  LG_AB    EW(A) '
        WRITE(NDISC+1,*)
        WRITE(NDISC+2,*) '  Z   IZ    WL     TEFF  LG_G LG_AHE LG_GF  ',
     :    ' MULT   VT  LG_AB    EW(A) '
        WRITE(NDISC+2,*)
        ENDIF

 
      ENDIF


!  Isolated lines
      IF(NMODE == 0) THEN

!       Logarithmic abundance
         ABUNDL=LOG10(ABUND)+ANORM

         WRITE(NDISC,200) TEFF,LOG10(GRAV),AHE,VT
         WRITE(NDISC,201) NSP,ELEM(NSP),RLAMB,NW,SYMM
!        Write out atomic data unless hydrogen, 
!         neutral diffuse or ionized helium
         IF(NSP.NE.IH .AND. NSP.NE.IHE .AND. NLINE.NE.0) 
     :       WRITE(NDISC,202) NION(NLINE),WAVE(NLINE),GF(NLINE),
     :                        ZMULT(NLINE)
         WRITE(NDISC,203) ABUNDL,EW,CONFL

!      Profiles and optical depths
         IF (NTHETA == 0) THEN
           WRITE(NDISC,206)
           WRITE(NDISC,207) (W(IA),FLUXN(IA),TAUL(IA),IA=1,NW)
           WRITE(NDISC+1,206)
           WRITE(NDISC+1,207) (W(IA),FLUX(IA),TAUL(IA),IA=1,NW)
           WRITE(NDISC+2,206)
           WRITE(NDISC+2,207) (W(IA),FLUXC(IA),TAUL(IA),IA=1,NW)

!      Specific intensities at given angles
         ELSE
           WRITE(NDISC,230) 
           WRITE(NDISC,231) (COSTHETA(N),N=1,NTHETA)
           WRITE(NDISC+1,234) 
           WRITE(NDISC+1,235) (COSTHETA(N),N=1,NTHETA)
           WRITE(NDISC+2,237) 
           WRITE(NDISC+2,238) (COSTHETA(N),N=1,NTHETA)
           DO IA=1,NW
             INT_RED(1:NWC-1) = (INTENS(1:NWC-1,N)/INTENC(1,N) 
     :                           - 1)/(COSTHETA(1:NWC) - 1.)
             BETA = SUM( INT_RED(1:NWC-1) ) / (NWC - 1)
             IF ( SYMM ) THEN
               NWC = NW
               WRITE(NDISC,232) W(IA),FLUXN(IA),
     :            (INTENS(IA,N)/INTENS(NWC,N),N=1,NTHETA),BETA
               WRITE(NDISC+1,236) W(IA),FLUX(IA),
     :            (INTENS(IA,N),N=1,NTHETA),TAUL(IA)
             ELSE
               WRITE(NDISC,232) W(IA),FLUXN(IA),
     :            (INTENS(IA,N)/INTENC(IA,N),N=1,NTHETA),BETA
               WRITE(NDISC+1,236) W(IA),FLUXN(IA),
     :            (INTENS(IA,N),N=1,NTHETA),TAUL(IA)
               WRITE(NDISC+2,239) W(IA),FLUXC(IA),
     :            (INTENC(IA,N),N=1,NTHETA),TAUL(IA)
             ENDIF
           ENDDO
         ENDIF
 
!  Spectrum synthesis for metals and non-diffuse helium
      ELSEIF(NMODE == 10.OR.NMODE == 11) THEN

!       Logarithmic abundances
!        DO I=1,NELSY
!          IF(ABUSY(I).GT.0.0) THEN
!            ABUSYL(I)=ANORM+LOG10(ABUSY(I))
!          ELSE
!            ABUSYL(I)=ABUSY(I)
!          ENDIF
!        ENDDO

!      Header line
         WRITE(NDISC,220) TEFF,LOG10(GRAV),AHE,VT,EW
         WRITE(NDISC,222) NW
         WRITE(NDISC+1,220) TEFF,LOG10(GRAV),AHE,VT,EW
         WRITE(NDISC+1,222) NW
         WRITE(NDISC+2,220) TEFF,LOG10(GRAV),AHE,VT,EW
         WRITE(NDISC+2,222) NW
        
!      Spectrum: normalized, fluxes, continuum and optical depths
         IF (NTHETA == 0) THEN
           WRITE(NDISC,221)
           WRITE(NDISC,225) (W(IA),FLUXN(IA),FLUX(IA),
     :                       FLUXC(IA),TAUL(IA),IA=1,NW)

!      Specific intensities at given angles
         ELSE
            WRITE(NDISC,230) 
            WRITE(NDISC,231) (COSTHETA(N),N=1,NTHETA)
            WRITE(NDISC+1,234) 
            WRITE(NDISC+1,235) (COSTHETA(N),N=1,NTHETA)
            WRITE(NDISC+2,237) 
            WRITE(NDISC+2,238) (COSTHETA(N),N=1,NTHETA)
            DO IA=1,NW
             INT_RED(1:NTHETA) = 1. - INTENS(IA,1:NTHETA)/INTENS(IA,1) 
             BETA = SUM( INT_RED(2:NTHETA-1)/(1-COSTHETA(2:NTHETA-1))) 
     :              / (NTHETA - 2)
             WRITE(NDISC,232) W(IA),FLUXN(IA),
     :              (INTENS(IA,N)/INTENC(IA,N),N=1,NTHETA),BETA
             WRITE(NDISC+1,236) W(IA),FLUX(IA),
     :              (INTENS(IA,N),N=1,NTHETA),TAUL(IA)
             WRITE(NDISC+2,239) W(IA),FLUX(IA),
     :              (INTENC(IA,N),N=1,NTHETA),TAUL(IA)
           ENDDO
         ENDIF


!  Compressed output for cycling around abundances and microturbulences
      ELSEIF(NMODE >= 20) THEN

!       Logarithmic abundance
         ABUNDL=LOG10(ABUND)+ANORM
         IF ( NLINE /= 0 ) NIONL = NION(NLINE)
         ! attempt to fix bug in H and He ion identifiers
         IF (NSP == 1) NIONL = 1
         IF (NSP == 2) NIONL = 1
         ! IF (NSP == 2 .AND. NHETYP > 0) NIONL = 1
         IF (NSP == 2 .AND. NHETYP < 0) NIONL = 2
         IF ( NLINE /= 0 ) THEN 
         WRITE(NDISC,250)NSP,NIONL,RLAMB,
     :         TEFF,LOG10(GRAV),
     :         LOG10(AHE),
     :         LOG10(GF(NLINE)),ZMULT(NLINE),VT,ABUNDL,EW
         ELSE
         WRITE(NDISC,250)NSP,NIONL,RLAMB,
     :         TEFF,LOG10(GRAV),
     :         LOG10(AHE),
     :         0.0,0.0,VT,ABUNDL,EW
         ENDIF

      ENDIF


!  Format statements

  200 FORMAT('# TEFF=',F7.0,2X,'LOG G=',F7.3,2X,'FRACT(HE)=',
     :    F6.3,2X,'VT=',F5.1)
  201 FORMAT('# NSP=',I3,5X,'ELEM=',A10,4X,'RLAMB=',0PF8.1,5X,
     :    'NW=',I3,5X,'SYMM=',L2)
  202 FORMAT('# ION=',I3,5X,'WAVE=',0PF10.2,5X,'GF=',0PF10.3,5X,
     :  'MULT=',0PF10.3)
  203 FORMAT('# ABUNDANCE=',0PF8.2,5X,'EW=',1P,E10.3,5X,
     :    'C/L RATIO=',0PF8.5)
  204 FORMAT('# EW=',1PE10.3,5X,'C/L RATIO=',0P,F8.5)

  206 FORMAT(//'# ',(6X,'WAVE',5X,'FLUX',5X,'TAUL'))
  207 FORMAT( 0P,F10.3, 1P,E11.3, 1P,E13.5 )

  220 FORMAT('# TEFF=',F7.0,2X,'LOG G=',F7.3,2X,'FRACT(HE)=',
     :    F6.3,2X,'VT=',F5.1,2X,'EW=',F7.3)
  221 FORMAT('# SYNTHETIC SPECTRUM :  F_NU'//
     :        2X,'WAVELEN ',1X,'NORMALIZED',4X,'TOTAL',4X,'CONTINUUM',
     :        2X,'OPT. DEPTH')
  222 FORMAT(' ',I6,'  = NW')
  225 FORMAT( 0P,F10.3, 1P,3E11.3, 1P,E13.5 )
  
  230 FORMAT(/,'# RELATIVE FLUX AND SPECIFIC INTENSITIES'/
     :   '    WAVE      F_NU/F_C   I_NU,MU/I_C ',86X,'BETA')
  231 FORMAT( 20X,       20F10.3 ,/ )
  232 FORMAT( 0P,F10.3, 1P,20E12.4 )
  
  234 FORMAT(/,'# TOTAL FLUX WITH SPECIFIC INTENSITIES '/
     :       '    WAVE     F_NU     I_NU,MU ',86X,'TAUL' )
  235 FORMAT( 17X,       20F10.3 ,/ )
  236 FORMAT( 0P,F10.3, 1P,20E12.4 )

  237 FORMAT(/,'# CONTINUUM FLUX WITH SPECIFIC INTENSITIES '/
     :       '    WAVE     F_C,NU  I_C,NU,MU',86X,'TAUL' )
  238 FORMAT( 17X,       20F10.3 ,/ )
  239 FORMAT( 0P,F10.3, 1P,20E12.4 )

  250 FORMAT(I4,I4,0P,F10.2,F7.0,3F6.2,F7.2,2F6.2,F10.4)

      END
