DOUBLE PRECISION FUNCTION SFLUX (TAUZ,TT,WAV,ND)

  !+
  !=      Calculates surface flux
  !-

  !  Global variables
  USE SPC_GLOBAL

  !  Double precision version
  IMPLICIT NONE

  !  Arguments
  DOUBLE PRECISION, INTENT(IN), DIMENSION(*) :: TAUZ
  DOUBLE PRECISION, INTENT(IN), DIMENSION(*) :: TT
  DOUBLE PRECISION, INTENT(IN) :: WAV
  INTEGER, INTENT(IN) :: ND

  !  Local variables.
  INTEGER :: I, J, JINT
  DOUBLE PRECISION :: A, TJI, YJI, TINT
  LOGICAL :: EXTRAP
  DOUBLE PRECISION, DIMENSION(18) :: Y, Z

  DATA Y(1)/.0200156/,Y(2)/.07876/,Y(3)/.13003/,Y(4)/.21664/, &
       Y(5)/.29906/,Y(6)/.44327/,Y(7)/.57633/,Y(8)/.80664/,Y(9)/1.0106/, &
       Y(10)/1.355/,Y(11)/1.6579/,Y(12)/2.1711/,Y(13)/2.6265/
  DATA Y(14)/3.4129/,Y(15)/4.2382/,Y(16)/6.198/,Y(17)/8.54/, &
       Y(18)/11.173/,Z(1)/4.4085/,Z(2)/3.9624/,Z(3)/4.8436/,Z(4)/4.3172/, &
       Z(5)/5.5634/,Z(6)/4.7446/,Z(7)/5.3222/,Z(8)/4.2679/,Z(9)/3.9545/ 
  DATA Z(10)/2.9419/,Z(11)/2.2942/,Z(12)/1.5341/,Z(13)/.95637/, &
       Z(14)/.53585/,Z(15)/.27515/,Z(16)/.070044/,Z(17)/.002669/, &
       Z(18)/.000449/

  !.

  !       Debug output

  IF(BUGIN) THEN
     WRITE(NDEBUG,900)WAV,ND,(TAUZ(J),TT(J),J=1,ND)
  ENDIF


  !       Initialize variables

  SFLUX=0.0
  A=(3.972E8)/(WAV*WAV*WAV)
  EXTRAP=.FALSE.


  !      CYCLE AROUND DEPTH POINTS

  JINT = 2
  DO I=1,18


     !       Linear interpolation in optical depth

     DO J=JINT,ND
        IF (TAUZ(J) > Y(I)) THEN
           JINT=J-1
           TJI = TT(JINT)
           YJI = TAUZ(JINT)
           TINT= TJI + (Y(I)-YJI)*(TT(J)-TJI)/(TAUZ(J)-YJI)
           GOTO 10
        ENDIF
     ENDDO


     !       Extrapolation required

     EXTRAP=.TRUE.
     JINT=ND-2


     !       Debug output

     IF(BUGIN) THEN
        WRITE(NDEBUG,901)I,SFLUX
     ENDIF

     CALL POLY(TAUZ(JINT:JINT+2),TT(JINT:JINT+2),Y(I),TINT)


     !       Calculate flux

10   SFLUX=SFLUX+Z(I)*(A/(EXP((+1.439E8)/(WAV*TINT))-1.0))
     !       write (*,'(i2,1p4e10.2)') i,wav,tauz(i),tint,sflux

  ENDDO


  !       Debug output

  IF(BUGOT.AND.EXTRAP) THEN
     WRITE(NDEBUG,902)SFLUX
  ENDIF


  !       Renormalize flux

  SFLUX=SFLUX*(2.0E-2)
  RETURN


  !       Debug format statements

900 FORMAT(//' ** SFLUX **',5X,'WAV=',F10.1,5X,'ND=',I10/ &
         25X,'TAU',8X,'TEMP'/30(1P,E28.5,E12.5/))
901 FORMAT(16X,' EXTRAPOLATION NEEDED IN FLUX FOR POINTS BEYOND I=', &
   I5,5X,'PRESENT FLUX VALUE=',1PE10.3)
902 FORMAT(16X,' FINAL FLUX VALUE=',1P,E10.3)

END FUNCTION SFLUX
