MODULE SPC_CONTROL

!+
!
!  Module:
!     SPC_CONTROL
!
!  Purpose:
!     Provide input/output stram numbers and debugging control flags
!     ** Warning -- this may be obsolescent - only used in 2 routines **
!
!  Language:
!     Fortran95
! 
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     15-FEB-2000: CSJ
!        original version adapted from SPECTRUM main program
!     29-AUG-2012: CSJ
!        converted to f90 
!
!  Use:
!     USE SPC_CONTROL 
!
!-


!         Type of calculation
!         -------------------
!           IOMODE               Defines i/o method
!           NMODE                Defines type of calculation
!           NOPTYP               Defines continuous opacity sources
!           NSOURCE              Defines treatment of source function
!                           1    Planck function
!                           2    Avrett-Loeser scheme
!                           3    Feautrier scheme
!           NHEDAT               Defines source of helium data
!                           1      Barnard, Cooper, Shamey et al..
!                           2      Beauchamp et al.
!           NHETYP               Defines identity of Helium Line
!           NHESTARK             Defines source of Stark broadening data
!                           0      Standard files supplemented by constant S&SB
!                           1      Standard files + variable D+S-B
!                           2      Variable D+S-B


      INTEGER IOMODE,NMODE,NOPTYP,NSOURCE,NHEDAT,NHESTARK,NHETYP


!  Input/Output streams
!
!           NREAD                Input stream from terminal
!           NWRITE               Output stream to terminal
!           NDISC                Disc output for equivalent widths
!           NDIPSO               Output stream for DIPSO output
!           NDEBUG               Output stream for Debug output
!           NSCAN                Output stream for synthesis data
!           NATOM                Input stream for atomic data
!           NSYNT                Input stream for synthesis data
!           NHYD                 Input stream for hydrogen data
!           NHEL                 Input stream for neutral helium data
!           NHEIB                Input stream for Beauchamp helium data
!           NHEIS                Input stream for helium Stark data
!           NHEII                Input stream for ionized helium data
!           NMODEL               Input stream for model atmosphere
!           NFILT                Input stream for filter functions
!           NPEACH               Input stream for Peach C and N opacity tables

      INTEGER, PARAMETER :: NREAD  =  5 
      INTEGER, PARAMETER :: NWRITE =  6 
      INTEGER, PARAMETER :: NDISC  =  7 
      INTEGER, PARAMETER :: NDIPSO =  8 
      INTEGER, PARAMETER :: NSCAN  =  9 
      INTEGER, PARAMETER :: NATOM  = 11 
      INTEGER, PARAMETER :: NSYNT  = 12 
      INTEGER, PARAMETER :: NHYD   = 13 
      INTEGER, PARAMETER :: NHEL   = 14 
      INTEGER, PARAMETER :: NHEII  = 15 
      INTEGER, PARAMETER :: NHEIB  = 16 
      INTEGER, PARAMETER :: NHEIS  = 17 
      INTEGER, PARAMETER :: NMODEL = 20 
      INTEGER, PARAMETER :: NFILT  = 21 
      INTEGER, PARAMETER :: NPEACH = 21 
      INTEGER, PARAMETER :: NDEBUG = 90


!  Debug flags
!
      LOGICAL :: BUGMN = .FALSE. !  Main program
      LOGICAL :: BUGWH = .FALSE.  !  Subroutine WHOLE
      LOGICAL :: BUGIN = .FALSE.  !  Subroutine INTEG and SFLUX
      LOGICAL :: BUGLN = .FALSE.  !  Line opacity routines
      LOGICAL :: BUGOP = .FALSE.  !  Continuous opacity routines
      LOGICAL :: BUGAT = .FALSE.  !  Atomic data input
      LOGICAL :: BUGMD = .FALSE.  !  Model atmosphere input
      LOGICAL :: BUGOU = .FALSE.  !  Output subroutines
      LOGICAL :: BUGOT = .FALSE.  !  Other subroutines

!
END MODULE SPC_CONTROL


