      SUBROUTINE SPC_SCAN (XMIN, XMAX)

!+
!     
!     sp_scan
!     
!=    Sets up datafile for spectrum synthesis 
!
!     10.vi.2021 -- compilation gives lots of warnings about
!     data-type conversion. This is a very old routine (Liz Conlon) and
!     use of structures might make it work more efficiently.
!     Certainly need to think about what data types should be used.
!
!-

!  Global variables
      USE SPC_GLOBAL
      USE SPC_VCSDATA      !! VCS tables

!     Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!     Subroutine arguments (given).
      DOUBLE PRECISION :: XMIN, XMAX !  wavelength limits for synthesis


!     Arrays used in this subroutine:
!     WL(KW) -   Grid over all hydrogen/helium/metal lines
!     Not sorted in wavelength
      REAL, DIMENSION(:), ALLOCATABLE ::  WL
      REAL, DIMENSION(:,:), ALLOCATABLE :: LLIST


!     Nominal resolution
      DOUBLE PRECISION, PARAMETER :: RESOLUTION = 1000000


!     Parameters for wavelength distribution in line profile
!     = WSCALE * ( WRATIO**((NG-1)/2) )
      INTEGER :: NGMET=31    !51   !     metal lines  : +/- 2.7 A
      INTEGER, PARAMETER :: NGHEI=71       !     helium lines : +/-24.7 A
      REAL :: WRATIO=1.50    ! 1.25 
      REAL, PARAMETER :: WSCALE=0.01 

!.

!     Announce subprogram
      WRITE (*,'(/,A,2F9.2)') ' SPC_SCAN ', XMIN, XMAX

!     Initialize metal line counter
      NWSY=0

!     Increase line widt4ddh if ntheta > 1 to allow for radial velocity
!     shifts
      IF ( NTHETA > 1 ) THEN 
        NGMET = 51
        WRATIO = 1.25
      ENDIF

!     If wavelength of metal line falls within wavelength range then
!     store all its atomic data

      IL = 1
      DO 
        IF (WAVE(IL+1) < 1.0) EXIT
        IL = IL + 1
      ENDDO

!     Setup temporary arrays
      ALLOCATE ( LLIST (IL,8) )

!     NELSY is counter for metal lines to be included
      NELSY=0
      DO I=1,IL

        IF (WAVE(I) >= XMIN+1 .AND. WAVE(I) <= XMAX-1) THEN
          NELSY=NELSY+1   
          WAVSY(NELSY)  = WAVE(I)
          NSPSY(NELSY)  = ISP(I)
          NIONSY(NELSY) = NION(I)
          GFSY(NELSY)   = GF(I)
          EDAMSY(NELSY) = EDAM(I) 
          RDAMSY(NELSY) = RDAM(I)
          EXCSY(NELSY)  = EXCIT(I)
          ZMLTSY(NELSY) = ZMULT(I)

          LLIST(NELSY,1) = WAVE(I)
          LLIST(NELSY,2) = ISP(I)
          LLIST(NELSY,3) = NION(I)
          LLIST(NELSY,4) = GF(I)
          LLIST(NELSY,5) = EDAM(I) 
          LLIST(NELSY,6) = RDAM(I)
          LLIST(NELSY,7) = EXCIT(I)
          LLIST(NELSY,8) = ZMULT(I)
        ENDIF

      ENDDO

!     Write total no. of metal lines found to the screen
      WRITE (*,'(A,F8.2,A,F8.2,A,I6)') 
     :     ' Metal     lines found: ',XMIN,' - ',XMAX,' = ',NELSY

!     Sort linelist with HEAPSORT on wavelength
      IF ( NELSY > KL ) THEN
        write (*,*) 'using HEAPSORT'
        CALL NR_HEAPSORT ( NELSY, 8, 1, LLIST(1:NELSY,1:8) )
        write (*,*) 'finished HEAPSORT'

        WAVSY(1:NELSY)  = LLIST(1:NELSY,1)
        NSPSY(1:NELSY)  = LLIST(1:NELSY,2)
        NIONSY(1:NELSY) = LLIST(1:NELSY,3)
        GFSY(1:NELSY)   = LLIST(1:NELSY,4)
        EDAMSY(1:NELSY) = LLIST(1:NELSY,5)
        RDAMSY(1:NELSY) = LLIST(1:NELSY,6)
        EXCSY(1:NELSY)  = LLIST(1:NELSY,7)
        ZMLTSY(1:NELSY) = LLIST(1:NELSY,8)

        write (*,*) 'unpacked HEAPSORT output',NELSY
        write (*,'(10F8.2)') WAVSY(1:10),WAVSY(NELSY-9:NELSY)

!     Sort complete set of atomic data into the correct order
!     this is desparately inefficient -- much faster means to do this. 
      ELSE
        SORT = 1
        DO WHILE ( SORT == 1 )
          SORT = 0
          DO J=1,NELSY-1
            IF (WAVSY(J) > WAVSY(J+1)) THEN
              TEMP1 = WAVSY(J)
              WAVSY(J) = WAVSY(J+1)
              WAVSY(J+1) = TEMP1

              TEMP2=NSPSY(J)
              NSPSY(J)=NSPSY(J+1)
              NSPSY(J+1)=TEMP2

              TEMP3=NIONSY(J)
              NIONSY(J)=NIONSY(J+1)
              NIONSY(J+1)=TEMP3

              TEMP4=GFSY(J)
              GFSY(J)=GFSY(J+1)
              GFSY(J+1)=TEMP4

              TEMP5=EDAMSY(J)
              EDAMSY(J)=EDAMSY(J+1)
              EDAMSY(J+1)=TEMP5

              TEMP6=RDAMSY(J)
              RDAMSY(J)=RDAMSY(J+1)
              RDAMSY(J+1)=TEMP6

              TEMP7=EXCSY(J)
              EXCSY(J)=EXCSY(J+1)
              EXCSY(J+1)=TEMP7

              TEMP8=ZMLTSY(J)
              ZMLTSY(J)=ZMLTSY(J+1)
              ZMLTSY(J+1)=TEMP8

              SORT = 1

            ENDIF
          ENDDO
        ENDDO

      ENDIF


!     Basic grid over selected wavelength range every XINT Angstroms
!     These values are also used to define the contiunuum frequencies
!     ICOUNT counts the number of wavelength points
      XINT   = 5.0
      XRANGE = XMAX - XMIN
      NXMIN  = XRANGE / XINT + 1


      ALLOCATE ( WL  (KW) )

      DO M = 1,NXMIN
        WL(M)  = XMIN + (M-1) * XINT 
      ENDDO
      ICOUNT = NXMIN
      WL(ICOUNT) = XMAX


!     Line grid over all metal lines found.
!     First over lower half of line and then over upper half.


! this may break when NELSY is too large; the number of wavelength 
! points inserted is NELSY * NGMET -- which is a lot. 

!     Cycle over lines
      DO J=1,NELSY

!     define optimum resolution for current line
        DSMALL=WAVSY(J)/RESOLUTION

        ICOUNT=ICOUNT+2
        WL(ICOUNT-1)=DSMALL*INT(WAVSY(J)/DSMALL)
        WL(ICOUNT)=WL(ICOUNT-1)+DSMALL
        NGLI=(NGMET-1)/2
        IF(NSPSY(J) == 2) NGLI=(NGHEI-1)/2

!     Negative wavelengths
        WTEMP=WAVSY(J)
        XMINT = XMIN
        IF (J > 1) XMINT = MAX(XMIN,(WAVSY(J)+WAVSY(J-1))*0.5)
        DO M=1,NGLI
          WTEMP=WTEMP-WSCALE*(WRATIO**M)
          IF(WTEMP >= XMINT) THEN
            ICOUNT=ICOUNT+1
            IF ( ICOUNT > KW ) THEN
               print*,'icount too large',icount,kw
               print*,'processing line',j,nelsy
               stop
            ENDIF
            WL(ICOUNT)=DSMALL*INT(WTEMP/DSMALL)
          ENDIF
        ENDDO

!     Positive wavelengths
        WTEMP=WAVSY(J)
        XMAXT = XMAX
        IF (J < NELSY) XMAXT = MIN(XMAX,(WAVSY(J)+WAVSY(J+1))*0.5)
        DO M=1,NGLI
          WTEMP=WTEMP+WSCALE*(WRATIO**M)
          IF(WTEMP <= XMAXT) THEN
            ICOUNT=ICOUNT+1
            IF ( ICOUNT > KW ) THEN
               print*,'icount too large',icount,kw
               print*,'processing line',j,nelsy
               stop
            ENDIF
            WL(ICOUNT)=DSMALL*INT(WTEMP/DSMALL)
          ENDIF
        ENDDO

      ENDDO
      IWL=ICOUNT

!     Debug output
      IF(BUGAT) THEN
        WRITE(NDEBUG,900)
        WRITE(NDEBUG,901)IWL
        WRITE(NDEBUG,902)(WL(I),I=1,IWL)
      ENDIF


!     Set up H and He grid if necessary - program searches for lines

!     Hydrogen lines
      NELSYM = NELSY
      DO NS = 1,3
        DO N  = 1,NH(NS)
          IF (XMIN < WH(N,NS) .AND. XMAX > WH(N,NS)) THEN
            NELSYP=NELSY
            CALL DP_HGRID(MAX(WH(N,NS)-100.0,XMIN),
     :           MIN(WH(N,NS)+100.0,XMAX), NS, N+1 )
            IF(NELSY /= NELSYP) THEN
              DO I=IWL+1,IWL+IWORK(1)
                WL(I)=WORK(I-IWL)
              ENDDO
              IWL=IWL+IWORK(1)
            ENDIF   
          ENDIF
        ENDDO
      ENDDO
      
      NHLINES = NELSY-NELSYM
      WRITE (*,'(A,F8.2,A,F8.2,A,I6)') 
     :     ' Hydrogen  lines found: ',XMIN,' - ',XMAX,' = ',NHLINES
      
!     Diffuse helium lines
      NELSYP=NELSY
      CALL HESEARCH(XMIN,XMAX)
      IF(NELSYP.NE.NELSY) THEN
        DO I=IWL+1,IWL+IWORK(1)
          WL(I)=WORK(I-IWL)
        ENDDO
        IWL=IWL+IWORK(1)
      ENDIF
      
      NHEILINES = NELSY-NELSYM-NHLINES
      WRITE (*,'(A,F8.2,A,F8.2,A,I6)') 
     :     ' Helium  I lines found: ',XMIN,' - ',XMAX,' = ',NHEILINES
      
!     Ionized helium lines
      NELSYP=NELSY
      CALL HE2SEARCH(XMIN,XMAX)
      IF(NELSYP /= NELSY) THEN
        DO I=IWL+1,IWL+IWORK(1)
          WL(I)=WORK(I-IWL)
        ENDDO
        IWL=IWL+IWORK(1)
      ENDIF

      NHEIILINES = NELSY-NELSYM-NHLINES-NHEILINES
      WRITE (*,'(A,F8.2,A,F8.2,A,I6)') 
     :     ' Helium II lines found: ',XMIN,' - ',XMAX,' = ',NHEIILINES
      


!     Sort linelist with HEAPSORT on wavelength
      IF ( NELSY > KL ) THEN

        LLIST(1:NELSY,1) = WAVSY(1:NELSY)
        LLIST(1:NELSY,2) = NSPSY(1:NELSY) 
        LLIST(1:NELSY,3) = NIONSY(1:NELSY) 
        LLIST(1:NELSY,4) = GFSY(1:NELSY)
        LLIST(1:NELSY,5) = EDAMSY(1:NELSY)
        LLIST(1:NELSY,6) = RDAMSY(1:NELSY)
        LLIST(1:NELSY,7) = EXCSY(1:NELSY) 
        LLIST(1:NELSY,8) = ZMLTSY(1:NELSY)

        CALL NR_HEAPSORT ( NELSY, 8, 1, LLIST(1:NELSY,:) )

        WAVSY(1:NELSY)  = LLIST(1:NELSY,1)
        NSPSY(1:NELSY)  = LLIST(1:NELSY,2)
        NIONSY(1:NELSY) = LLIST(1:NELSY,3)
        GFSY(1:NELSY)   = LLIST(1:NELSY,4)
        EDAMSY(1:NELSY) = LLIST(1:NELSY,5)
        RDAMSY(1:NELSY) = LLIST(1:NELSY,6)
        EXCSY(1:NELSY)  = LLIST(1:NELSY,7)
        ZMLTSY(1:NELSY) = LLIST(1:NELSY,8)


!     Sort complete set of atomic data into the correct order
      ELSE
        SORT = 1
        DO WHILE ( SORT == 1 ) 
          SORT = 0
          DO J=1,NELSY-1
            IF (WAVSY(J) > WAVSY(J+1)) THEN
              TEMP1 = WAVSY(J)
              WAVSY(J) = WAVSY(J+1)
              WAVSY(J+1) = TEMP1

              TEMP2=NSPSY(J)
              NSPSY(J)=NSPSY(J+1)
              NSPSY(J+1)=TEMP2

              TEMP3=NIONSY(J)
              NIONSY(J)=NIONSY(J+1)
              NIONSY(J+1)=TEMP3

              TEMP4=GFSY(J)
              GFSY(J)=GFSY(J+1)
              GFSY(J+1)=TEMP4

              TEMP5=EDAMSY(J)
              EDAMSY(J)=EDAMSY(J+1)
              EDAMSY(J+1)=TEMP5

              TEMP6=RDAMSY(J)
              RDAMSY(J)=RDAMSY(J+1)
              RDAMSY(J+1)=TEMP6

              TEMP7=EXCSY(J)
              EXCSY(J)=EXCSY(J+1)
              EXCSY(J+1)=TEMP7

              TEMP8=ZMLTSY(J)
              ZMLTSY(J)=ZMLTSY(J+1)
              ZMLTSY(J+1)=TEMP8

              SORT = 1

            ENDIF
          ENDDO
        ENDDO
      ENDIF

!     Use first wavelength to define nominal wavelength in interval
      DSMALL=WL(1)/RESOLUTION

      NWSY=1
      WSY(1)=XMIN


!     Cycle IWL time through wavelengths to ensure that all wavelengths
!     are found
      DO I=1,IWL

        ISMALL=0
        WSMALL=XMAX

!     Search for smallest remaining wavelength in grid WL
        DO J=1,IWL
          IF(WL(J) <= WSMALL) THEN
            ISMALL=J
            WSMALL=WL(J)
          ENDIF
        ENDDO

        IF(ABS(WSY(NWSY)-WSMALL) > DSMALL) THEN
          NWSY=NWSY+1
          WSY(NWSY)=WSMALL
          WL(ISMALL)=2.0*XMAX
        ELSE
          WL(ISMALL)=2.0*XMAX
        ENDIF

      ENDDO


!     Put in point at XMAX if necessary
      IF(ABS(WSY(NWSY)-XMAX) >= DSMALL) THEN
        NWSY=NWSY+1
        WSY(NWSY)=XMAX
      ENDIF




!     Write total no. of grid points to the screen
      WRITE(NWRITE,'(A,I8)') ' Number of wavelengths in grid:', NWSY


!     If total no. of points exceeds KW send error message
!     to the screen, then abort programme
      IF(NWSY > KW) THEN
        WRITE(NWRITE,*)' ** Too many wavelengths ** '
        WRITE(NWRITE,*)' Reduce wavelength range '
        WRITE(NWRITE,*)' OR increase line_threshold '
        STOP
      ENDIF


!     Debug output
      IF (BUGAT) THEN
        WRITE(NDEBUG,905)
        WRITE(NDEBUG,901)NWSY
        WRITE(NDEBUG,902)(WSY(I),I=1,NWSY)
        WRITE(NDEBUG,903)      
        WRITE(NDEBUG,904)(J,NSPSY(J),NIONSY(J),WAVSY(J),GFSY(J),
     :       EDAMSY(J),RDAMSY(J),EXCSY(J),J=1,NELSY)
      ENDIF

!     Release temporary arrays
      DEALLOCATE ( LLIST )
      DEALLOCATE ( WL  )




!     Format statements

!     Screen output formats
!200  FORMAT (' ** INPUT START AND END WAVELENGTHS ** ') 
!202  FORMAT(' LINE NO.',2X,'SPEC',2X,'WAVE')
!203  FORMAT (I7,I7,5X,F7.2)
!205  FORMAT(F10.2)
!206  FORMAT(7F10.2)

!     NSCAN output
!300  FORMAT (I5,5X,A)
!301  FORMAT(7F10.2)
!310  FORMAT(2I3,F9.2,1P,3E9.2,0P,F8.3,F8.2,7F6.2)

!     Debug output
 900  FORMAT (' *** METAL LINE GRID *** ')
 901  FORMAT (I3)
 902  FORMAT (7F10.3)
 903  FORMAT (' LINE NO.',2X,'SPEC',2X,'LEVEL',4X,'WAVE',7X,'GF'
     1     ,8X,'ELECT',7X,'RAD',8X,'EXCIT ')
 904  FORMAT (I6,I7,I6,0P,F13.3,1P,4E12.3)
 905  FORMAT (' *** FINAL GRID *** ')
      

      END
