      SUBROUTINE transfer (tau,s,nd, mu,i,nm, f)

!+
!
!=    calculate the emergent intensity  (i_nu)  and  flux (f_nu)
!     as a formal solution of the radiative transfer equation 
!     given the source function (s_nu) as a function of optical depth (tau_nu)
!     for the case of a semi-infinite plane-parallel atmosphere.
!     i is calculated for a given obliquity (mu = cos(theta)).
!
!     D.Mihalas, Stellar Atmospheres, 2nd ed. (2-52), (8-11)
!
!     06.11.00  Reduced nq from 48 to 24 and tested.
!               Reduced upper tau from 100 to 10 and tested.
!               both changes had negligible effect
!-

!  Global declarations.
      IMPLICIT   NONE

!  Local constants.
      INTEGER    kd                               ! maximum number of depths
      PARAMETER (kd=201)
      INTEGER    nq                               ! order of G. quadrature
      PARAMETER (nq=24)
      
! Global variables.
      REAL*8     tt(kd),  ss(kd), amu             ! inputs copied to local common
      INTEGER    n
      COMMON /source_block/ tt, ss, amu, n

!  Subroutine arguments.
      INTEGER    nd, d                            ! number of depth points
      REAL*8     tau(*)                           ! optical depth
      REAL*8     s(*)                             ! source function
      REAL*8     mu(*)                            ! cosine angle of radiation
      REAL*8     i(*)                             ! specific intensity (d=1)
      INTEGER    nm, j                            ! number of angles
      REAL*8     f                                ! emergent flux

!  Local variables.
      REAL*8     wgts(nq), absc(nq)               ! weights and abscissae for Gaussian qudrature
      INTEGER    d1, d2, no 
      REAL*8     f_integral, i_integral           ! integrand functions

!.

      no = nq
      d1 = 1
      d2 = 1
      DO d = 1,nd
        tt(d) = tau(d)
        ss(d) = s(d)
        IF (tt(d).LE.10.0) d2 = max (d,d2)
      ENDDO
      n = nd

!  get quadrature abscissae and weights
      CALL nr_gauleg (tau(d1), tau(d2), absc, wgts, no)

!  integrate flux 
      f = 0.0
      DO d = 1, no
        f = f + wgts(d) * f_integral(absc(d))
      ENDDO
      f = 2 * f 

!  integrate intensities
      IF (nm .GT. 0) THEN
      DO j = 1, nm
      i(j) = 0.0
        DO d = 1, no
          amu = mu(j)
          i(j) = i(j) + wgts(d) * i_integral(absc(d)) / amu
        ENDDO
      ENDDO
      ENDIF

      END
