      SUBROUTINE VCS(NE,TT,UPPER,DEL,PROF,NDEL)

!+
!=      Calculates Vidal, Cooper and Shamey profiles 
!        Data for H alpha to H delta taken from Kurucz
!             for H epsilon calculated from Vidal et al code -
!             thes are unreliable for Ne>10**16 and Te<10**4
!-

!  Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!  Subroutine arguments
      DOUBLE PRECISION NE
      DOUBLE PRECISION TT
      INTEGER UPPER
      DOUBLE PRECISION DEL(*)
      DOUBLE PRECISION PROF(*)
      INTEGER NDEL

!  Local variables and arrays
      DOUBLE PRECISION NEL
      DOUBLE PRECISION S(6,17,40)
      DOUBLE PRECISION TTV(6)
      DOUBLE PRECISION NEV(17)
      DOUBLE PRECISION ALPHAV(40)
      DOUBLE PRECISION PRALP(40)

      DATA LINEP/0/,SMALL/1.0E-30/

!.

!       Logarithmic variables

      NEL=LOG10(NE)
      TTL=LOG10(TT)


!       Set up array sizes and initialize grid points

      IF(UPPER.NE.7) THEN
        ITT=6
        INE=17
        IALPHA=40
      ELSE
        ITT=5
        INE=14
        IALPHA=39
      ENDIF

      TTV(1)=LOG10(2500.)
      RATIO=LOG10(2.0)
      DO I=2,ITT
       TTV(I)=TTV(I-1)+RATIO
      ENDDO

      NEV(1)=10.0
      RATIO=0.5
      DO I=2,INE
       NEV(I)=NEV(I-1)+RATIO
      ENDDO

      ALPHAV(1)=-3.0
      RATIO=0.2
      DO I=2,IALPHA
       ALPHAV(I)=ALPHAV(I-1)+RATIO
      ENDDO


!       Debug output

      IF(BUGLN) THEN
        WRITE(NDEBUG,900)NEL,TTL,UPPER,ITT,INE,IALPHA,
     *                   (TTV(I),I=1,ITT),
     *                   (NEV(I),I=1,INE),
     *                   (ALPHAV(I),I=1,IALPHA)
      ENDIF


!       Define line

      LINE=UPPER-2


!      Check for same line

      IF(LINE.NE.LINEP)THEN

        LINEP=LINE

        IF(LINE.LE.4)THEN


!      Open stream NHYD for unformatted read

          OPEN(UNIT=NHYD,STATUS='OLD',FORM='FORMATTED',
     *         FILE='BALMER')
          DO IL=1,LINE
            READ(NHYD,101)(((S(I,J,K),I=1,6),K=1,40),J=1,17)
  101       FORMAT(////(6E13.0))
          ENDDO
          CLOSE(UNIT=NHYD)

        ELSE


!       Open stream NHYD for formatted read

          OPEN(UNIT=NHYD,STATUS='OLD',FORM='FORMATTED',
     *         FILE='EPSILON')
          READ(NHYD,100)((RUBB,(S(J,K,L),L=1,IALPHA),J=1,ITT),K=1,INE)
  100     FORMAT(24X,F12.0)
          CLOSE(UNIT=NHYD)

!         Change to logarithmic format
          DO I=1,ITT
            DO J=1,INE
              DO K=1,IALPHA
                S(I,J,K)=LOG10(S(I,J,K))
              ENDDO
            ENDDO
          ENDDO

        ENDIF

      ENDIF


!       Debug output

      IF(BUGLN) THEN
        WRITE(NDEBUG,901)(((S(I,J,K),I=1,6),K=1,40),J=1,17)
      ENDIF


!       Mean field

      FO=1.25E-9*NE**.66666667                                         


!       Find position in density and Temperature grid

      DO JTT=2,ITT
        IF(TTL.LE.TTV(JTT)) GOTO 10
      ENDDO
      JTT=ITT

   10 DO JNE=2,INE
        IF(NEL.LE.NEV(JNE)) GOTO 11
      ENDDO
      JNE=INE


!       Temperature and density interpolation
!
!         This code has been modified to prevent extrapolation 
!         outside the limits of the grid. This problem which caused severe
!         failure in low-density atmospheres. Ideally, this low
!         density regime should be replaced with (eg) a Voigt profile.
!         At very high densities, the profiles became step-like. 
!
   11 CONTINUE
      W1=(TTV(JTT)-TTL)/(TTV(JTT)-TTV(JTT-1))
      W2=(NEV(JNE)-NEL)/(NEV(JNE)-NEV(JNE-1))
      IF (NEL.LT.NEV(1)) W2 = 1.0             ! prevent low-density extrapolation
      IF (NEL.GT.NEV(INE)) W2 = 0.0           ! prevent high-density extrapolation

      DO J=1,IALPHA
        S1=W1*S(JTT-1,JNE-1,J)+(1.0-W1)*S(JTT,JNE-1,J)
        S2=W1*S(JTT-1,JNE  ,J)+(1.0-W1)*S(JTT,JNE  ,J)
        PRALP(J)=W2*S1+(1.0-W2)*S2
      ENDDO


!       Interpolation in Alpha

      DO I=1,NDEL

        DELALP=LOG10((ABS(DEL(I))+SMALL)/FO)

        IF (DELALP.LE.ALPHAV(1)) THEN

!       Small Alpha

          PROF(I)=(10.0**PRALP(1))/FO                                         

        ELSE


!       Large Alpha

          DO JALPHA=2,IALPHA
            IF(DELALP.LE.ALPHAV(JALPHA)) GOTO 12
          ENDDO
          JALPHA=IALPHA


!       Alpha interpolation


   12     W1=(ALPHAV(JALPHA)-DELALP)/(ALPHAV(JALPHA)-ALPHAV(JALPHA-1))
          S1=W1*PRALP(JALPHA-1)+(1.0-W1)*PRALP(JALPHA)
          PROF(I)=(10.0**S1)/FO

        ENDIF

      ENDDO

!       Debug output


      IF(BUGLN) THEN
        WRITE(NDEBUG,902)(DEL(I),I=1,NDEL),(PROF(I),I=1,NDEL)
      ENDIF
      RETURN


!       Debug format statements

  900 FORMAT(' ***** VCS ***** DENSITY AND TEMPERATURES'/
     *1P,2E12.4,4I5/(1P,10E12.4))
  901 FORMAT(' ***** VCS ***** PROFILES'/
     *(1P,6E13.5))
  902 FORMAT(' ***** VCS ***** WAVELENGTHS AND LINE OPACITY'/
     *(1P,10E12.5))
      END                                                               
