      SUBROUTINE VCS2(NLO,NUP,NE,TT,DEL,PROF,NDEL)
!+
!
!=      Calculates Vidal, Cooper and Smith profiles 
!       (1970, JQSRT, 10, 1011)
!
!-

!  Global variables
      USE SPC_VCSDATA      !! VCS tables
      USE SPC_CONTROL      !! i/o streams etc

!  Double precision version
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
!     IMPLICIT NONE

!  Subroutine arguments
      INTEGER           NLO, NUP, NDEL
      DOUBLE PRECISION  NE, TT, DEL(*), PROF(*)

!  Local constants
      DOUBLE PRECISION SMALL
      PARAMETER (SMALL=1.D-30)

!  Local variables
      DOUBLE PRECISION NEL, TTL
      INTEGER          LINE
      DOUBLE PRECISION FO
      DOUBLE PRECISION W1, W2, W3, S1, S2, S3
      INTEGER          JT, JN, JA
      DOUBLE PRECISION PRALP(pmp), DELALP

      INTEGER          NLX
      SAVE             NLX

!.

      IF (BUGLN) 
     :    WRITE(NDEBUG,905) NLO,NUP,NE,TT,NDEL

!  Logarithmic variables
      NEL=LOG10(NE)
      TTL=LOG10(TT)

!  Check data is available
      DATA NLX/0/
      IF(NLX.NE.NLO) THEN
         CALL VCS_READ (NLO)
         NLX = NLO
      ENDIF

!  Identify line (assumes table complete from nup=nlo+1 to nup=nlo+nline)
      LINE = NUP-NLO

!  Check line OK
      IF (NLO.NE.NL(LINE).OR.NUP.NE.NU(LINE)) THEN
         WRITE (*,*) ' VCSREAD : Inconsistent line indices ',
     :               NLO,NUP,NL(LINE),NU(LINE),WH(NLO,LINE)
      ENDIF
      
!  Mean field
      FO=1.25E-9*NE**.66666667                                         

!  Find position in density and temperature grid
      JT = INT((TTL-LOG_T0(LINE))/LOG_T_INC(LINE)) + 2
      JT = MAX(MIN(JT,MT(LINE)),2)
      JN = INT((NEL-LOG_NE0(LINE))/LOG_NE_INC(LINE)) + 2
      JN = MAX(MIN(JN,MNE(LINE)),2)

!  Temperature and density interpolation
!
!         This code has been modified to prevent extrapolation 
!         outside the limits of the grid. This problem which caused severe
!         failure in low-density atmospheres. Ideally, this low
!         density regime should be replaced with (eg) a Voigt profile.
!         At very high densities, the profiles became step-like. 
!
!
      W1=(TTL-(LOG_T0(LINE)+(JT-2)*LOG_T_INC(LINE)))/LOG_T_INC(LINE)
      W2=(NEL-(LOG_NE0(LINE)+(JN-2)*LOG_NE_INC(LINE)))/LOG_NE_INC(LINE)
!      IF (NEL.LT.LOG_NE0(1)) W2 = 0.0                          ! prevent low-density extrapolation
!      IF (NEL.GT.LOG_NE0(1)+(JN-1)*LOG_NE_INC(LINE)) W2 = 1.0  ! prevent high-density extrapolation

      W1=MAX(0.0D0, MIN(1.0D0, W1))
      W2=MAX(0.0D0, MIN(1.0D0, W2))
      DO J=1,MP(LINE)
        S1=(1.0-W1)*SVCS(JT-1,JN-1,J,LINE)+W1*SVCS(JT,JN-1,J,LINE)
        S2=(1.0-W1)*SVCS(JT-1,JN  ,J,LINE)+W1*SVCS(JT,JN  ,J,LINE)
        PRALP(J)=(1.0-W2)*S1+W2*S2
      ENDDO

!  Interpolation in Alpha
      DO I=1,NDEL
        DELALP=LOG10((ABS(DEL(I))+SMALL)/FO)

!     Small Alpha
        IF (DELALP.LE.LOG_ALPHA0(LINE)) THEN
          PROF(I)=(10.0**PRALP(1))/FO                                         

!     Large Alpha
        ELSE
          IF (DELALP.GT.0.) THEN
          JA=INT((DELALP-LOG_ALPHA0(LINE))/LOG_ALPHA_INC(LINE)) + 2
          ELSE
          JA=-INT((LOG_ALPHA0(LINE)-DELALP)/LOG_ALPHA_INC(LINE)) + 2
          ENDIF
          JA=MAX(MIN(JA,MP(LINE)),2)

!     Alpha interpolation
          W3=(DELALP-(LOG_ALPHA0(LINE)+(JA-2)*LOG_ALPHA_INC(LINE)))
     >        /LOG_ALPHA_INC(LINE)
          W3=MAX(0.0D0, MIN(1.0D0, W3))
          S3=(1.0-W3)*PRALP(JA-1)+W3*PRALP(JA)
          PROF(I)=(10.0**S3)/FO

        ENDIF
!        IF (BUGLN) 
!     :    WRITE(NDEBUG,900) I,DEL(I),DELALP,JA,W3,S3,PROF(I)
      ENDDO

!   Debug output
      IF(BUGLN) THEN
        WRITE(NDEBUG,901) PRALP(1:MP(LINE))
        WRITE(NDEBUG,902)(DEL(I),I=1,NDEL),(PROF(I),I=1,NDEL)
      ENDIF
      RETURN


! Debug format statements
 900  FORMAT(' ***** VCS ***** DEL,...'/
     *(I3,1P,2E12.5,I3,3E12.5))
 901  FORMAT(' ***** VCS ***** PRALP'/
     *(1P,10E12.5))
 902  FORMAT(' ***** VCS ***** WAVELENGTHS AND LINE OPACITY'/
     *(1P,10E12.5))
 905  FORMAT(' ***** VCS ***** INPUT'/(2I3,1P,2E12.5,I3))

      END                                                               
