      SUBROUTINE WHOLE

!+
!=      Main subroutine
!
!       Sets up wavelength scale (WAVEG)
!       Calculates line opacity (LINOP)
!       Calculate optical depths and fluxes (INTEG)
!       Calculates normalised fluxes and equivalent width (EQUIV)
!       Outputs equivalent widths and line profiles
!               to terminal and LP (OUTFOR)
!               in DIPSO format (OUTDIP)
!
!
!       now only cvalled by "EWABUND"
!       all NMODE references reducndant, NMODE must be 0 for INTEG
!-

!  Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!.

      
!  Generate wavelength scale

   !  Spectrum synthesis calculation
      IF(NMODE.EQ.10.OR.NMODE.EQ.11) THEN
        NW=NWSY
        SYMM=.FALSE.
        DO  I=1,NW
          W(I)=WSY(I)
        ENDDO

   !  OTHER METAL CALCULATIONS
      ELSE
          CALL WAVEG
      ENDIF

   !  Debug output
      IF(BUGWH) THEN
        WRITE(NDEBUG,901)NW,WAVMAX,(W(I),I=1,NW)
      ENDIF



!  CALCULATES LINE OPACITY FOR METALS

      IF (NMODE.EQ.11) WRITE (*,*) 'CALCULATE LINE OPACITIES'
      CALL LIN_OP

!     abort if line not found
      IF ( RLMU(1,1) < 0. ) RETURN

   !  Debug output
      IF(BUGWH) THEN
        WRITE(NDEBUG,900)NW,NDEPTH,
     *                   ((RLMU(I,J),I=1,NW),J=1,NDEPTH)
      ENDIF



!  Calculate optical depth scale and fluxes

      IF (NMODE.EQ.11) WRITE (*,*) 'CALCULATE RADIATIVE TRANSFER'
      IF ( NMODE == 1 ) THEN
         NMODE = 0 
         CALL INTEG
         NMODE = 1
      ELSE
         CALL INTEG
      ENDIF


!  Calculate equivalent width and normalised fluxes

      IF (NMODE.EQ.11) WRITE (*,*) 'NORMALIZE AND OUTPUT'
      CALL EQUIV


   !  Iteration to correct abundance
      IF(NMODE.EQ.1) THEN
        RETURN

   !  Standard output
      ELSE

      ! Output to terminal
      ! WRITE (*,*) 'Output to terminal'
        CALL OUTTER

      ! Output to disc
      ! WRITE (*,*) 'Output to disc'
        CALL OUTDIS

      ! Output to DIPSO
      ! WRITE (*,*) 'Output to dipso'
        CALL OUTDIP

      ENDIF



!  Debug Format Statements
  900 FORMAT(//' ** WHOLE **',5X,'NW=',I5,5X,'NDEPTH=',I5//
     *       16X,'LINE OPACITY'/1500(1P,10E12.5/))
  901 FORMAT(//' ** WHOLE **',5X,'NW=',I5,5X,'WAVMAX=',F10.3//
     *       16X,'WAVELENGTHS'/50(1P,E12.5/))


      END
