SUBROUTINE FLXCON(NEWTEMP_IT,KEY_OPBF)

  !+
  !
  !      Calculates monochromatic and integrated flux and some 
  !      integrals for temperature correction
  !      Continuum only
  !
  !-

  USE LTE_PRECISIONS
  USE LTE_WLGRID
  USE LTE_MODEL
  USE STN_PARAMETERS
  USE STN_CONSTANTS
  USE STN_INTGRA
  USE STN_FLUXES
  USE STN_ODFDATA

  IMPLICIT NONE

  !.

  INTEGER, INTENT(IN) :: NEWTEMP_IT
  CHARACTER(LEN=12), INTENT(IN) :: KEY_OPBF 

  !  Local variables.

  INTEGER :: I, K, J

  REAL :: EN(ND)      ! Electron densities     ne(tau)
  REAL :: CAP(ND)
  REAL :: SIGM(ND)                                                 
  REAL :: OPA
  REAL :: FREQ

  !! debugging check:
  REAL(KIND=QQ), ALLOCATABLE, DIMENSION (:) :: TAU_NU   ! local optical depth


  INTEGER :: N, NSTEPS

  REAL(KIND=QQ) :: W 
  REAL :: STEPWT, SUMWT

  !!  External function declaration
  REAL, EXTERNAL :: FREWGT

  !! Allocate radiative arrays:

  ALLOCATE(BNU(ND))   
  ALLOCATE(SNU(ND))   
  ALLOCATE(JNU(ND))   
  ALLOCATE(FNU(ND))   

  ALLOCATE(FNUC(ND))  
  ALLOCATE(JNUC(ND))  
  ALLOCATE(FNUL(ND))  
  ALLOCATE(JNUL(ND))  

  ALLOCATE(CAC(ND))
  ALLOCATE(tau_nu(ND))

  !! Allocate interpolation coefficients (for LINOP)

  ALLOCATE(CO1(ND))
  ALLOCATE(CO2(ND))
  ALLOCATE(CO3(ND))
  ALLOCATE(CO4(ND))
  ALLOCATE(INEJ(ND))
  ALLOCATE(ITJ(ND))


  NU_REFWL = C / REFWL *  1.e+8 

  !  Calculate frequencies.      
  NU(LA:LL) = C / WL(LA:LL) * 1.E+8

  !  Initialize all variables and integrals      
  SIGM(1:ND) = SIG(1:ND)
  B    = 0.0
  FLX  = 0.0
  TNS  = 0.0
  KAPP = 0.0
  KAPJ = 0.0
  KAPF = 0.0
  GRAD = 0.0

  !  Boltzmann factors and electron densities.
  !  Thomson scattering.
  EN(1:ND)    = P_E(1:ND) / (BK * TEMP(1:ND))

  !********************************
  !  Loop over frequencies
  DO I = LA,LL

     W = FREWGT(i,nu,la,ll)
     FREQ = nu(i)

     !     Planck function.
     CALL BNUE(temp,FREQ,BNU,ND)
     !!*     to be tested....
     !!*       CALL RTF_BNU (temp,ND, nu(i),1, BNU,ND)

     !     Continuous opacities.
     DO K=1,ND
        KAPPA_NU(K)=0.0
        SIG(K)   =0.0
        CAC(K) = 0.0
        OPA = 0.0
        CALL STN_OPAC (KEY_OPBF,K,I,TEMP(K),P_TOTAL(K),P_E(K),WL(I),OPA,CAC(K),SIG(K))
     ENDDO

     !    Continuum source function (Feautrier)
     CALL FLX_FEAUT (NU(I),TAU_NU,KAPPA_NU,JNUC)
     DO K=1,ND-1
        IF (TAU_NU(K) < 5.) THEN
           CALL FLX_PHI (TAU_NU,K,FNUC(K))  
        ENDIF
        IF (TAU_NU(K) >= 5. .OR. FNUC(K) < 0.) THEN
           IF (K > 1) THEN  
              FNUC(K) = 0.5*((BNU(K)-BNU(K-1))/(TAU_NU(K)-TAU_NU(K-1)) &
                   &  +(BNU(K+1)-BNU(K))/(TAU_NU(K+1)-TAU_NU(K)))*(4./3.)
           ELSE
              FNUC(1) = 0.5*(BNU(K)/TAU_NU(K) &
                   & +(BNU(K+1)-BNU(K))/(TAU_NU(K+1)-TAU_NU(K)))*(4./3.)
           ENDIF
        ENDIF
     ENDDO
     FNUC(ND) = (BNU(ND)-BNU(ND-1))/(TAU_NU(ND)-TAU_NU(ND-1))*4./3.

     !! Continuum only!


     DO K=1, ND
        FNU(K)  = FNUC(K)
        JNU(K)  = JNUC(K)
        B(K)    = B(K)   + W*BNU(K)
        FLX(K)  = FLX(K) + W*FNUC(K)
        TNS(K)  = TNS(K) + W*JNUC(K)
        CAP(K)  =             CAC(K)
        KAPP(K) = KAPP(K)+ W* CAP(K)*BNU(K)
        KAPJ(K) = KAPJ(K)+ W* CAP(K)*JNUC(K)
        KAPF(K) = KAPF(K)+ W*(CAP(K)+sig(K))*FNUC(K)
     ENDDO

     FNUE(I) = FNU(1)

  ENDDO

  !  end of frequency loop
  !***********************************

  DO K = 1,ND                                           
     GRAD(K)= PI/C/KAPPA(K)*KAPF(K)                                   
     SIG(K) = SIGM(K)                                               
  ENDDO

  !! deallocate arrays:

  DEALLOCATE(BNU)   
  DEALLOCATE(SNU)   
  DEALLOCATE(JNU)   
  DEALLOCATE(FNU)   
  DEALLOCATE(FNUC)  
  DEALLOCATE(JNUC)  
  DEALLOCATE(FNUL)  
  DEALLOCATE(JNUL)  
  DEALLOCATE(CO1)
  DEALLOCATE(CO2)
  DEALLOCATE(CO3)
  DEALLOCATE(CO4)
  DEALLOCATE(INEJ)
  DEALLOCATE(ITJ)

  DEALLOCATE(CAC)
  DEALLOCATE(tau_nu)


CONTAINS


  subroutine flx_feaut (flx_nu,flx_tau,flx_kap,jnu)


    ! This subroutine provides an interface to the standard rtf_feaut 
    ! subroutine. It is required because this version of flxos uses a
    ! "real(kind=qq)" to define the variable precision, whilst "rtf" 
    ! assumes double precision throughout. 

    !  sb arguments
    REAL(KIND=QQ)               :: flx_nu
    REAL(KIND=QQ), DIMENSION(*) :: flx_tau,flx_kap,jnu

    !  local copies for interface to rtf

    !      INTEGER    nd                              ! declare atmosphere structure
    REAL*8                 :: nuzero
    REAL*8, DIMENSION(100) :: tauz, tt, pe, opz

    REAL*8                 :: fnu               ! declare opacities at freq
    REAL*8, DIMENSION(100) :: opc, ops, opl     ! for which solution reqd.

    REAL*8, DIMENSION(100) :: rnu, a, phi       ! function frequency structure

    REAL*8, DIMENSION(100) :: ss, jj, taunu                    ! output structures

    real*8, dimension(10)  :: XNU = 0.
    real*8, dimension(10)  :: XA = (/1.0,(0.0, i=2,10)/)
    real*8, dimension(10)  :: XPHI = (/1.0,(0.0,i=2,10)/)
    real*8, dimension(16)  :: XMU = 0.
    real*8, dimension(16)  :: XB = 0.

    integer :: NM = 4

    !   copy into local arrays for passing to rtf

    nuzero      = nu_refwl
    tauz(1:nd)  = tau(1:nd)
    tt(1:nd)    = temp(1:nd)
    pe(1:nd)    = p_e(1:nd)
    opz(1:nd)   = kappa(1:nd)
    fnu         = flx_nu
    opc(1:nd)   = cac(1:nd)
    ops(1:nd)   = sig(1:nd)
    opl(1:nd)   = kappa_nu(1:nd)


    CALL RTF_FEAUT ( nuzero, tauz, tt, pe, opz, nd, &
         fnu, opc, ops, opl, 1, &
         XNU, XA, XPHI, 1, &
         XMU, XB, NM, &
         taunu, ss, jj )

    !   copy back local arrays for passing to rtf


    tau_nu(1:nd) = taunu(1:nd)
    snu(1:nd) = ss(1:nd)
    jnu(1:nd) = jj(1:nd)

  end subroutine flx_feaut



  subroutine flx_phi  ( tau_nu, k, fnu )

! This subroutine provides an interface to the standard rtf_phi
! subroutine. It is required because this version of flxos uses a
! "real(kind=qq)" to define the variable precision, whilst "rtf" 
! assumes double precision throughout. 

    !  sb arguments
    REAL(KIND=QQ), DIMENSION(*) :: tau_nu
    INTEGER        :: k
    REAL(KIND=QQ)  :: fnu

    !  local definitions

    !    Import
    DOUBLE PRECISION t(100)                 ! t
    DOUBLE PRECISION f(100)                 ! f(t)
    INTEGER n                             ! dimension of f(t)
    DOUBLE PRECISION tau                  ! PHI evaluation 
    !    Export
    DOUBLE PRECISION phi                  ! PHI integral

    ! Convert variable types

    t(1:nd) = tau_nu(1:nd)
    f(1:nd) = snu(1:nd)
    tau     = tau_nu(k)

    CALL RTF_PHI ( t, f, nd, tau, phi  )

    fnu = phi

  end subroutine flx_phi


END SUBROUTINE FLXCON
