
  subroutine ngacc(t_in, t_1, t_2, t_3, t_new, ND) 

!! Second-order NG acceleration, applied to the temperature correction
!! (Temkor), after 4 temperature iterations 
!!
!! History:
!!    19-March-2004 (NTB)

    implicit none 

!  Subroutine arguments.
!   input:
      integer :: ND    ! Number of depth points in atmopshere
      real :: t_in(ND) ! last temperature
      real :: t_1(ND), t_2(ND), t_3(ND) ! previous iterates
!   output:
      real :: t_new(ND)

!  Local variables.
      integer :: j    ! Depth in atmosphere

      real :: a1, b1, b2, c1, c2  ! coefficients
      real :: wt                  ! weight, can be varied
      real :: d0, d1, d2
      real :: a, b

! Initialize values

      a1 = 0.0
      b1 = 0.0
      b2 = 0.0
      c1 = 0.0
      c2 = 0.0
      wt = 1.0   ! this value may vary

! Loop through atmosphere, to calculate coefficients:

      do j = 1,ND
        d0 = t_in(j) - t_1(j)
        d1 = t_in(j) - 2*t_1(j) + t_2(j)
        d2 = t_in(j) - t_1(j) - t_2(j) + t_3(j)

        a1 = a1 + wt*d1*d1
        b1 = b1 + wt*d1*d2
        b2 = b2 + wt*d2*d2
        c1 = c1 + wt*d0*d1
        c2 = c2 + wt*d0*d2
      end do

! construct coefficients
     
      a = (b2*c1 - b1*c2)/(b2*a1 - b1*b1)
      b = (a1*c2 - b1*c1)/(b2*a1 - b1*b1)

      do j = 1,ND
        t_new(j) = (1-a-b)*t_in(j) + a*t_1(j) + b*t_2(j)
      end do      
 
  end subroutine ngacc       
