
  subroutine STERNE_QUB
!!+
!! 
!! Name:
!!     STERNE_QUB
!!
!! Purpose:
!!     Reformat a STERNE model atmosphere for input to SPECTRUM
!!     and for input to DIPSO (ie the flux distribution)
!!
!! Language:
!!     Fortran 90
!!
!! Description:
!!
!!     This code operates on ouput from the line-blanketed version 
!!
!! Authors:
!!     CSJ: C.S.Jeffery (CCP7)
!!
!! History:
!!     21-JAN-1993 (CSJ): original
!!     21-MAR-1996 (CSJ): adapted to ensure composition data always present
!!
!! Bugs:
!!
!!-

    implicit none

!!  i/o file names and headers

      CHARACTER(len=40) :: FROOT
      CHARACTER(len=40) :: FSTERNE
      CHARACTER(len=40) :: FSPECTRUM
      CHARACTER(len=40) :: FDIPSO
      CHARACTER(len=79) :: SPHEAD

!!  Model description

      integer, parameter :: KD=50 ! Maximum onumber of zones in model atmosphere

      real :: TEFF      ! Effective temperature
      real :: GRAV      ! Surface gravity 

      integer :: NDEPTH ! Number of layers in model atmosphere
      real :: TAU(KD)   ! Optical depth grid for model atmospheres
      real :: TT(KD)    ! Temperature as function of optical depth
      real :: GRAD(KD)  ! dPr / d tau ....
      real :: PG(KD)    ! Gas pressure ....
      real :: PE(KD)    ! Electron pressure ....
      real :: OP(KD)    ! Continuous opacity ....


      real :: ALPHA(40)  ! Atmospheric composition (homogeneous)

      real :: AB(10,KD)  ! Depth dependent composition

      integer :: NW     ! Number of wavelength points in spectrum
      real :: W(10000)   ! Wavelengths
      real :: F(10000)   ! Astrophysical fluxes
      real :: B(10000)   ! Blocking factor

!! local variables
   
      integer :: I, J, JL, IDUMMY, K
      real :: AH, AHE, ACN, AO, ANE, AMG, AAL
      real :: ASI, AS, ACA, AFE, SUM, OVR
      real :: AC, AN

!! Read in the converged model structure and flux distribution

!!   Open the input model
      OPEN  (1,FILE='STN_MODEL',STATUS='OLD')

!!   Read the model structure
      NDEPTH = KD
      READ  (1,'(F6.0,F5.0)') TEFF, GRAV
      READ  (1,*) (ALPHA(J),J=1,10)
      READ  (1,*) (ALPHA(J),J=11,20)
      READ  (1,*) (ALPHA(J),J=21,30)
      READ  (1,*) (ALPHA(J),J=31,40)
      READ  (1,'(10F8.0)') (TT(J),J=1,NDEPTH)
      READ  (1,'(10E10.0)') (GRAD(J), J=1,NDEPTH)  
      READ  (1,'(10F8.0)') (OP(J),J=1,NDEPTH)
      READ  (1,'(10F8.0)') (PG(J),J=1,NDEPTH)
      READ  (1,'(10F8.0)') (PE(J),J=1,NDEPTH)
      READ  (1,'(5E15.0)') (TAU(J),J=1,NDEPTH)
      !READ  (1,'(5F14.0)') (TT(J),J=1,NDEPTH)
      !READ  (1,'(5E10.0)') (GRAD(J), J=1,NDEPTH)  
      !READ  (1,'(5F12.0)') (OP(J),J=1,NDEPTH)
      !READ  (1,'(5F12.0)') (PG(J),J=1,NDEPTH)
      !READ  (1,'(5F12.0)') (PE(J),J=1,NDEPTH)
      !READ  (1,'(5E15.0)') (TAU(J),J=1,NDEPTH)

!!   Read the depth dependent composition
      READ (1,*) IDUMMY
      READ (1,104) ((AB(I,J),I=1,10),J=1,50)
104   FORMAT (F6.0,5X,F6.0,4X,8E11.0)

      READ (1,*) 
      READ (1,107) ((AB(I,J),I=1,10),J=1,50)
107   FORMAT (10E11.0)

      READ (1,*) 
      READ (1,107) ((AB(I,J),I=1,10),J=1,50)

      READ (1,*) 
      READ (1,107) ((AB(I,J),I=1,10),J=1,50)

!!   Read the flux distribution
      READ (1,*) NW
      READ (1,105) (W(K),F(K),B(K),K=1,NW)
105   FORMAT (F8.0,E12.0,F6.0)

!!   Close the input
      CLOSE (1)

!! Process the model structure

!!   Take antilogs of the model
      GRAV = 10.**GRAV
      IF (OP(1).LT.0.) THEN
         DO J = 1,NDEPTH
           OP(J) = 10.**OP(J)
         ENDDO
      ENDIF
      DO J = 1,NDEPTH
         PG(J) = 10.**(PG(J))
         PE(J) = 10.**(PE(J))
      ENDDO

!!   Sort out the composition data
      IF (ALPHA(1)+ALPHA(2).GT.2.0) THEN
         DO I = 1,7
            ALPHA(I) = ALPHA(I) / 10.
         ENDDO
      ENDIF

!!   all abundances are fractional
!!   defaults: ACN=0 means C and N take log abundances
      AH  = ALPHA(1)
      AHE = ALPHA(2)
      ACN = ALPHA(6) + ALPHA(7)

!!   check C and N abundances, 
!!   IF zero, adopt solar values * metallicity factor
!!   ratios for sum over metals = 1. from Anders and Grevesse (1988)
                        AC  = ALPHA(6)
!      IF (AC.LT.0.0001) AC  = 10.**(ALC-3.440)
                        AN  = ALPHA(7)
!      IF (AN.LT.0.0001) AN  = 10.**(ALN-3.950)

!!$!!   then scale remaining metals to solar ratios
!!$      AO  = 10.**(ALPMET(1)-3.07)
!!$      ANE = 10.**(ALPMET(3)-3.91) ! use Mg as reference
!!$      AMG = 10.**(ALPMET(5)-4.42)
!!$      AAL = 10.**(ALPMET(6)-5.53)
!!$      ASI = 10.**(ALPMET(7)-4.45)
!!$      AS  = 10.**(ALPMET(9)-4.79)
!!$      ACA = 10.**(ALPMET(13)-5.64)
!!$      AFE = 10.**(ALPMET(19)-4.33)

!!   
      AO  = ALPHA(8)
      ANE = ALPHA(10)
      AMG = ALPHA(12)
      AAL = ALPHA(13)
      ASI = ALPHA(14)
      AS  = ALPHA(16)
      ACA = ALPHA(20)
      AFE = ALPHA(26)


!!   Conservation of numbers
      SUM = AH  + AHE + AC  + AN  + AO  + ANE + AMG &
                    & + AAL + ASI + AS  + ACA + AFE
      OVR = SUM - 1.0
      IF (AH.GT.AHE) AH  = AH  - OVR
      IF (AHE.GT.AH) AHE = AHE - OVR


!!$!! Report the model 
!!$
!!$      WRITE (*,*)
!!$      WRITE (*,'(1X,A,F8.1)') 'Teff  = ', TEFF
!!$      WRITE (*,'(1X,A,F8.1)') 'log g = ', LOG10(GRAV)
!!$      WRITE (*,'(1X,A,F8.5)') 'nH    = ', AH
!!$      WRITE (*,'(1X,A,F8.5)') 'nHe   = ', AHE
!!$      WRITE (*,'(1X,A,F8.5)') 'nC+nN = ', ACN
!!$      WRITE (*,*)


!! Write out the model for SPECTRUM

      OPEN  (2,FILE='STN_q',STATUS='UNKNOWN')
      WRITE (2,'(F12.5,1PE12.5,0P3F12.5)') TEFF, GRAV, AHE, AH, ACN
!!     IF (ACN.GT.0.0001) THEN
         WRITE (2,'(1P5E12.5)') AC ,AN ,AO ,ANE,AMG
         WRITE (2,'(1P5E12.5)') AAL,ASI,AS ,ACA,AFE
!!     ENDIF

      DO J = 1,NDEPTH
!        WRITE (2,'(1P5E12.5)') TAU(J),TT(J),PG(J),PE(J),OP(J)
         WRITE (2,'(1P5E14.7)') TAU(J),TT(J),PG(J),PE(J),OP(J)
      ENDDO
      CLOSE (2)


!! Write out the spectrum for DIPSO

!!   Create some header lines
      WRITE (SPHEAD,'(''STERNE: Teff='',F6.0,'' log g='',F3.1, &
           & '' H='',F4.2,'' He='',F4.2,'' CN='',F7.5)') &
           & TEFF, LOG10(GRAV), AH, AHE, ACN

!!   Write the file - SPECTRUM format 2
      OPEN  (2,FILE='STN_s',STATUS='UNKNOWN')
      WRITE (2,'(A)') SPHEAD
      WRITE (2, *)
      WRITE (2,'(I10)') NW
      WRITE (2,'(F10.1,E12.5)') (W(K),F(K),K=1,NW)
      CLOSE (2)


!! Write out the temp distribution:

!!   Create some header lines
      WRITE (SPHEAD,'(''STERNE: Teff='',F6.0,'' log g='',F3.1, &
           & '' H='',F4.2,'' He='',F4.2,'' CN='',F7.5)') &
           & TEFF, LOG10(GRAV), AH, AHE, ACN

      OPEN  (2,FILE='STN_t',STATUS='UNKNOWN')
      WRITE (2,'(A)') SPHEAD
      WRITE (2, *)
      WRITE (2,'(I10)') KD
      WRITE (2,'(1P2E12.5)') (TAU(J),TT(J),J=1,KD)
      CLOSE (2)


  end subroutine STERNE_QUB 
