SUBROUTINE STN_HYDRO (KEY_OPBF) 

  !+
  !
  !   HYDRO
  !
  !   Numerical integration of the hydrostatic equation
  !
  !-

  USE LTE_WLGRID
  USE LTE_MODEL
  USE STN_PARAMETERS
  USE STN_CONSTANTS
  USE STN_SWITCHES
  USE STN_INTGRA

  IMPLICIT NONE

  CHARACTER(LEN=12), INTENT(IN) :: KEY_OPBF 

  INTEGER I, K
  REAL :: TAU1, PP1, T1, GRD1
  INTEGER :: JDEPTH
  REAL :: TAU2, PP2, T2, GRD2
  REAL :: TAU3, PP3, T3, GRD3 
  REAL :: TAU4, PP4, T4, GRD4
  REAL :: PPE

  REAL :: CAPP, RBF, SIGM

  REAL :: A, Q         ! used in construction of the optical depth scale, 
  ! will recover from the optical depth scale

  REAL :: step, H
  REAL :: F1, F2, F3, F4

  REAL :: PA, kFB, PC
  REAL :: PAA, PBB, FBB, PCC, F5

!.

  !! Set boundary conditions:

  IF (LM == 1 .and. IGREY == 0) THEN
     GRAD =  SIGMA*TEFF**4/C
     ! Outer boundary condition
     P_total(1)  = (GRAV*WM/0.6655E-24-GRAD(1))*tau(1)
     IF (P_total(1) < 0.0) P_total(1)=abs(P_total(1))
  ELSE
     GRAD(1) = MIN(GRAV/kappa(1)*GRMAX, GRAD(1))
     ! Outer boundary condition
     ! Form 1: dP/dtau = g/kappa - g_rad	 
     P_total(1) = (GRAV/kappa(1)-GRAD(1))*tau(1)
     ! Form 2: d(P-Prad)/dtau = (g-g_rad)/kappa
     !       P_total(1) = ((GRAV-GRAD(1))/kappa(1))*2.*tau(1) +  (SIGMA*temp(1)**4)/C/3.
     ! Form 3: d(P-Prad)/dtau = g/kappa
     !       P_total(1) = (GRAV/kappa(1)*2.*tau(1)) + (SIGMA*temp(1)**4)/C/3.
  ENDIF


  !  The integration is initialized with a four-point Runge-Kutta procedure
  K = 1    
  H = 1.0

  CAPP = 0.0
  RBF = 0.0
  SIGM = 0.0   
  TAU1 = tau(K)
  PP1  = P_total(K)
  T1   = temp(K)
  JDEPTH = K 
  
  A = LOG10(tau(1))
  Q = LOG10(tau(2)/tau(1))
  IF (PP1 < 0.0) THEN
     write (*,*) 'stn_hydro, pp<0', K, PP1, GRAV,WM,0.6655E-24,-GRAD(1),tau(1)
     PP1 = -PP1
  ENDIF

  !write(*,*) 'STN_HYDRO:',jdepth,tau1,t1,pp1
  
  CALL STN_PRESS (K,T1,PP1,PPE)
  IF (PPE < 0.0) WRITE (*,*) 'stn_hydro, pe<0', K, PPE,PP1
  CALL STN_OPAC (KEY_OPBF,K,0,T1,PP1,PPE,REFWL,CAPP,RBF,SIGM)

  
  Kappa(K) = CAPP
  GRAD(K) = MIN(GRAV/Kappa(K)*GRMAX, GRAD(K))
  GRD1    = GRAD(K)
  P_e(K) = PPE
  sig(K)  = SIGM

  !write(*,*) 'STN_HYDRO:',k,t1,pp1,ppe,capp,rbf,sigm,grad(k)


  DO K=1,3  
     JDEPTH = K + 1
     F1    = TAU1*ALQ(K)*TENLOG*(GRAV/CAPP-GRD1)
     step = (K-1)+H/2.
     PP2   = P_total(K)+H/2.*F1
     TAU2  = 10.**(A+Q*step)
     T2    = temp(K)+(TAU2-tau(K))/(tau(K+1)-tau(K))*(temp(K+1)-temp(K))

     IF (PP2 < 0.0) THEN
        WRITE (*,*) 'stn_hydro pp<0', K+1, PP2
        PP2 = -PP2
     ENDIF

     CALL STN_PRESS    (K+1,T2,PP2,PPE)
     IF (PPE < 0.0) then
        write (*,*) 'stn_hydro pe<0', K+1, PPE,PP2
        PPE = -PPE
     ENDIF
     CALL STN_OPAC     (KEY_OPBF,K+1,0,T2,PP2,PPE,REFWL,CAPP,RBF,SIGM)

     GRD2  = GRAD(K)+(TAU2-tau(K))/(tau(K+1)-tau(K))* &
          (GRAD(K+1)-GRAD(K))
     GRD2  = MIN(GRAV/CAPP*GRMAX, GRD2)
     F2    = TAU2*ALQ(K)*TENLOG*(GRAV/CAPP-GRD2)
     TAU3  = TAU2
     PP3   = P_total(K)+H/2.*F2
     T3    = T2
     GRD3  = GRD2

     IF (PP3 < 0.0) THEN
        write (*,*) 'stn_hydro', K+1, PP3
        PP3 = -PP3
     ENDIF
     CALL STN_PRESS    (K+1,T3,PP3,PPE)
     IF (PPE < 0.0) THEN
        write (*,*) 'hydro', K+1, PPE,PP3
        PPE = -PPE
     END IF
     CALL STN_OPAC     (KEY_OPBF,K+1,0,T3,PP3,PPE,REFWL,CAPP,RBF,SIGM)

     GRD3  = MIN(GRAV/CAPP*GRMAX, GRD2)
     F3    = TAU3*ALQ(K)*TENLOG*(GRAV/CAPP-GRD3)
     TAU4  = TAU(K+1)
     PP4   = P_total(K)+H*F3
     T4    = temp(K+1)

     IF (PP4 < 0.0) write (*,*) 'hydro pp<0', K+1, PP4
     CALL STN_PRESS    (K+1,T4,PP4,PPE)
     IF (PPE < 0.0) write (*,*) 'hydro pe<0', K+1, PPE, PP4
     CALL STN_OPAC     (KEY_OPBF,K+1,0,T4,PP4,PPE,REFWL,CAPP,RBF,SIGM)

      !! to avoid undefined part of the P_e array, add
     P_e(K+1) = PPE

     sig(K+1) = SIGM
     Kappa(K+1) = CAPP
     GRAD(K+1) = MIN(GRAV/Kappa(K+1)*GRMAX, GRAD(K+1))
     GRD4  = GRAD(K+1)
     F4    = TAU4*ALQ(K)*TENLOG*(GRAV/CAPP-GRD4)
     P_total(K+1) = P_total(K)+H/6.*(F1+2.*F2+2.*F3+F4)

    !  write(*,*) 'STN_HYDRO:',k+1,p_e(k+1),kappa(k+1),grad(k+1),p_total(k+1)

     
  ENDDO


  !  The Runge-Kutta integration for the first four points is finished,
  !  integration is continued using Hammings' predictor-corrector method.
  !  IF four points P_total(1), P_total(2), P_total(3), P_total(4) are already Known then the
  !  following point can be predicted, and then iteratively improved

  !  Predictor for fifth point
  K = 5
  JDEPTH = 5
  F2 = TAU(2)*ALQ(2)*TENLOG*(GRAV/Kappa(2)-GRAD(2))
  F3 = TAU(3)*ALQ(3)*TENLOG*(GRAV/Kappa(3)-GRAD(3))
  F4 = TAU(4)*ALQ(4)*TENLOG*(GRAV/Kappa(4)-GRAD(4))
  PA = P_total(1) +1.333333*H*(2.*F4-F3+2.*F2)

  IF ( PA < 0.0) PA = (2.0*PP4)-(0.1*PP4)
  CALL STN_PRESS    (K,temp(K),PA,PPE)
  IF (PPE < 0.0) write (*,*) 'hydro pe<0', K, PPE,PA
  CALL STN_OPAC     (KEY_OPBF,K,0,temp(K),PA,PPE,REFWL,CAPP,RBF,SIGM)

  GRAD(5) = MIN(GRAV/CAPP*GRMAX, GRAD(K))
  KFB      = tau(K)*ALQ(K)*TENLOG*(GRAV/CAPP-GRAD(K))

  !  Corrector for fifth point
  PC      = 0.125*(9.*P_total(4)   -P_total(2)   +3.*H*(KFB+2.*F4-F3))
  P_total(K)   = PC+0.07438*(PA-PC)

  IF (P_total(K) < 0.0) write (*,*) 'hydro, pp<0', K, P_total(K)      
  CALL STN_PRESS    (K,temp(K),P_total(K),PPE)
  IF (PPE < 0.0) write (*,*) 'hydro, pe<0', K, PPE, P_total(K)
  CALL STN_OPAC     (KEY_OPBF,K,0,temp(K),P_total(K),PPE,REFWL,CAPP,RBF,SIGM)

  Kappa(K) = CAPP
  IF (GRAD(K) < 0.0) then
     GRAD(K) = GRAV/Kappa(K)*GRMAX
  ELSE
     GRAD(K) = MIN(GRAV/Kappa(K)*GRMAX, GRAD(K))
  ENDIF

  P_e(K) = PPE
  sig(K)  = SIGM
  F5      = tau(K)*ALQ(K)*TENLOG*(GRAV/CAPP-GRAD(K))

  DO K = 6, ND
     JDEPTH = K

     !  Predictor for point K
     F2  = F3
     F3  = F4
     F4  = F5
     PAA = P_total(K-4)+1.333333*H*(2.*F4-F3+2.*F2)

     !  Corrector for point K
     PBB =  PAA-0.9256198*(PA-PC)

     IF (PBB < 0.0) write (*,*) 'hydro 1, pb<0', K, PBB, PAA, PA, PC   
     IF ( PBB < 0.0) PBB = (2.0*P_e(K-1))-(0.1*P_e(K-1))   
     CALL STN_PRESS    (K,temp(K),PBB,PPE)
     IF (PPE < 0.0) write (*,*) 'hydro 1, pe<0', K, PPE,PBB
     CALL STN_OPAC     (KEY_OPBF,K,0,temp(K),PBB,PPE,REFWL,CAPP,RBF,SIGM)

     IF (GRAD(K) < 0.0) then
        GRAD(K) = GRAV/CAPP*GRMAX
     ELSE
        GRAD(K) = MIN(GRAV/CAPP*GRMAX, GRAD(K))
     ENDIF

     FBB     = tau(K)*ALQ(K)*TENLOG*(GRAV/CAPP-GRAD(K))
     PCC     = 0.125*(9.*P_total(K-1)-P_total(K-3)+3.*H*(FBB+2.*F4-F3))
     P_total(K)   = PCC+0.07438*(PAA-PCC)

     IF (P_total(K) < 0.0) write (*,*) 'hydro 2, pp<0', K, P_total(K),PCC,PAA, PCC      
     CALL STN_PRESS    (K,temp(K),P_total(K),PPE)
     IF (PPE < 0.0) write (*,*) 'hydro 2, pe<0', K, PPE,P_total(K)
     CALL STN_OPAC     (KEY_OPBF,K,0,temp(K),P_total(K),PPE,REFWL,CAPP,RBF,SIGM)

     Kappa(K) = CAPP
     IF (GRAD(K) < 0.0) then
        GRAD(K) = GRAV/CAPP*GRMAX
     ELSE
        GRAD(K) = MIN(GRAV/CAPP*GRMAX, GRAD(K))
     ENDIF

     P_e(K) = PPE
     IF (P_e(K) < P_e(K-1)) write(*,*) 'hydro, pe_K<pe_K-1', K, P_e(K), P_e(K-1)
     sig(K)  = SIGM
     F5 = tau(K)*ALQ(K)*TENLOG*(GRAV/CAPP-GRAD(K))

     PA      = PAA
     PC      = PCC

  ENDDO

  !  Now calculate the geometrical depth and mass scale
  DO K = 1,ND
     DENS(K) = WM*(P_total(K)-P_e(K))/(BK*temp(K))
     XINT(K) = tau(K)*ALQ(K)*TENLOG/(Kappa(K)*DENS(K))
  ENDDO
  DIST(1)  = tau(1) *0.5/(Kappa(1) *DENS(1) )
  CALL INGRAT( 1,ND,H,DIST,XINT)


END SUBROUTINE STN_HYDRO
