SUBROUTINE STN_OUTMODEL (KEY_OPBF,KEY_OPBB)

!+
!   STN_OUTMODEL
!
!  Output current model - and update regularly
!
!-

  USE LTE_WLGRID
  USE LTE_MODEL
  USE STN_PARAMETERS
  USE STN_SWITCHES
  USE STN_INTGRA
  USE STN_COMPOSITION
  USE OP_GLOBAL
  
  IMPLICIT NONE
    
  CHARACTER(LEN=12), INTENT(IN) :: KEY_OPBF 
  CHARACTER(LEN=12), INTENT(IN) :: KEY_OPBB 
  
  !  Local variables.
  REAL, DIMENSION(LL) ::  FNUX
  
  INTEGER :: I, J
  INTEGER :: NSTRAT
  
  REAL :: FEH
  
  CHARACTER(LEN=2) :: SYMBOL   !  chemical symbol
  INTEGER :: IZ                !  atomic number
  INTEGER :: STATUS = 0        !  inherited error status

  INTEGER, DIMENSION(8) :: DATE_VALS

  !.

  !homogeneous models:
  NSTRAT = 0
  
  FEH = OP_ALPNN(26,ND)/MAX(OP_ALPNN(1,ND),0.00001)
  ! divide by solar abund
  FEH = FEH/0.318E-04
  FEH = LOG10(FEH)
  
  OPEN(9,FILE='STN_MOD',STATUS='UNKNOWN',FORM='FORMATTED')

  ! origin of the model 
  WRITE(9,*) 'model.origin  Sterne_', STN__VERSION 
  WRITE(9,*) 'model.label   ', STN__LABEL
  CALL DATE_AND_TIME ( VALUES = DATE_VALS )
  WRITE(9,'(A,I4,2I2.2)') ' model.date    ',DATE_VALS(1:3)
  WRITE(9,'(A,2(I2.2,'':''),I2.2)') ' model.time    ', DATE_VALS(5:7)
  WRITE(9,*)

  ! model parameters
  WRITE(9,'(A13,F6.0)') ' model.teff  ', TEFF
  WRITE(9,*) 'units.teff  kelvin'
  WRITE(9,'(A16,F5.2)') ' model.gravity  ', GLG
  WRITE(9,*) 'units.gravity  log_cgs'
  WRITE(9,*) 
  WRITE(9,*) 'model.ndepth  ',ND
  WRITE(9,*) 'model.nelems  ',NZ
  WRITE(9,*) 'model.nstrat  ',NSTRAT, ' ! current default '
  WRITE(9,*) 
  WRITE(9,*) 'model.frache ', ALPHA(2,ND)
  WRITE(9,*) 'model.hebyh  ', ALPHA(2,ND)/MAX(ALPHA(1,ND),0.00001)
  WRITE(9,*) 'model.febyh  ', FEH
  WRITE(9,*) 'model.albyhe ', 0.0, ' ! not evaluated '
  WRITE(9,*)

  ! opacity parameters
  WRITE(9,*) 'opacity.bf_source  ',KEY_OPBF
  WRITE(9,*) 'opacity.bb_source  ',KEY_OPBB
  WRITE(9,*) 'opacity.bb_int  ',BB_DVC
  WRITE(9,*) 'opacity.vturb   ',VMIC
  WRITE(9,*) 'units.vturb      cm/s'
  WRITE(9,*)

  ! printout composition assuming this is a homegeneous model
  ! and in any case for all elements which are not stratified
  DO IZ=1,NZ
     CALL TAP_ESYMB ( IZ, SYMBOL, STATUS )
     IF ( IZ <= 2 ) &  
          WRITE(9,'(A17,I4,F8.4,2A3)') &
          ' model.abundance ',IZ, (ALPHA(IZ,ND)),'  !',SYMBOL
     IF ( IZ >  2 ) &  
          WRITE(9,'(A17,I4,1PE12.4,2A3)') &
          ' model.abundance ',IZ, (ALPHA(IZ,ND)),'  !',SYMBOL
  ENDDO
  WRITE(9,*) 'units.abundance  number_fraction'
  WRITE(9,*)

  ! print model structure
  WRITE(9,*) 'model.lambda_ref  ', REFWL
  WRITE(9,*) 'units.lambda_ref  Angstrom'
  WRITE(9,*)
  WRITE(9,*) 'structure.tau_ref  '
  WRITE(9,'(5E15.7)')  (TAU(J),J=1,ND)
  WRITE(9,*) 'units.tau_ref  cm'
  WRITE(9,*)
  WRITE(9,*) 'structure.opac_ref  '
  WRITE(9,'(10F8.4)')  (LOG10(KAPPA(J)), J = 1,ND)            
  WRITE(9,*) 'units.opac_ref  log   cm^-1 '
  WRITE(9,*)
  WRITE(9,*) 'structure.temp  '
  WRITE(9,'(10F9.1)')  (TEMP(J), J = 1,ND)    
  WRITE(9,*) 'units.temp  kelvin'
  WRITE(9,*) 
  WRITE(9,*) 'structure.Pgas  '
  WRITE(9,'(10F8.4)')  (LOG10(P_TOTAL(J)), J = 1,ND)              
  WRITE(9,*) 'units.pgas  log dyn'
  WRITE(9,*) 'structure.Pel  '
  WRITE(9,'(10F8.4)')  (LOG10(P_e(J)), J = 1,ND)             
  WRITE(9,*) 'units.pel  log dyn'
  WRITE(9,*)
  WRITE(9,*) 'structure.grad  '
  WRITE(9,'(10E10.3)') (GRAD(J), J = 1,ND)            
  WRITE(9,*) 

  ! printout stratified elements if required
  ! next line needs to identify which elements are stratified
  IF (NSTRAT /= 0) THEN
     WRITE(9,*) 'structure.abundance H  '
     WRITE(9,*) (OP_ALPNN(1,J),J=1,ND)
     WRITE(9,*) 'structure.abundance He '
     WRITE(9,*) (OP_ALPNN(2,J),J=1,ND)
     WRITE(9,*) 'units.abundance  number_fraction'
     WRITE(9,*)
  ENDIF

  ! print spectrum 
  WRITE(9,*) 'spectrum.length  ', LL
  WRITE(9,*) 'spectrum.wave  '
  WRITE(9,'(10F9.1)') (WL(J),J=LA,LL)
  WRITE(9,*) 'units.wave  Angstrom'
  DO J=LA,LL
     FNUX(J) = 2.99793E18*FNUE(J)/WL(J)/WL(J)
  END DO
  WRITE(9,*) 'spectrum.flux  '
  WRITE(9,'(10E12.5)') (FNUX(J),J=LA,LL)
  WRITE(9,*) 'units.flux  erg/cm^2/A'
  
  CLOSE (9)

END SUBROUTINE STN_OUTMODEL
