  SUBROUTINE STN_STAND

 !+
 !! Sets up optical depth scale, and initial temperature values for a 
 !! grey atmosphere  
 !! Adapted from Sterne subroutine STAND
 !-

    USE LTE_MODEL
    USE STN_PARAMETERS
    USE STN_SWITCHES
    USE OP_GLOBAL

    IMPLICIT NONE

 !! atw: mean atomic masses 
 !! hm: mass of hydrogen, sigma (units???)

      REAL, PARAMETER :: HM = 1.6732E-24
      REAL, PARAMETER :: SIGMA =  5.6705E-05
      REAL, DIMENSION(7) :: ATW = (/1.00801,4.0028,6.939,9.013,10.812,12.01161,14.0073/)

      INTEGER :: I
      INTEGER :: N1, N2
      REAL :: P1, P2, P3, Q1, Q2, Q3

 !.

 !! Mean molecular weight of stellar material in grams.
      WM = 0.0
      DO I = 1,7
        WM = WM+HM*OP_ALPNN(I,ND)*ATW(I)/ATW(1)
      ENDDO 

 !! Surface gravity (real value).
      GRAV = 10.0**GLG

 !! Construct the optical depth scale (log tau)
 !! The initial (log) value at the surface is P1. The first 10 (log) 
 !! steps are incremented by 0.35 (Q1), the next 10 by 0.25 (Q2). 
 !! The corresponding initial value for these steps is P2=-5.
 !! The remaining steps are incremented by Q3=0.10. The corresponding 
 !! initial value is P3=-2.3.
 !! The spacing between steps is saved for future use as alq()

      N1 = 10
      P1 = -6.0 
      Q1 =  0.35
      N2 = 19
      P2 = -5.0  
      Q2 =  0.25
      P3 = -2.3  
      Q3 =  0.10

      DO I=1,N1
         TAU(I) = 10.**(P1+Q1*(I-1))
         ALQ(I) = Q1
      ENDDO
      DO I=N1+1,N2
         TAU(I) = 10.**(P2+Q2*(I-1))
         ALQ(I) = Q2
      ENDDO
      DO I = N2+1,ND
         TAU(I) = 10.**(P3+Q3*(I-1))
         ALQ(I) = Q3
      ENDDO

 !! Set up the initial temperature values for a grey atmosphere (if used).
 !! This equation is taken from Mihalas, equation 3-16, with the approximate 
 !! solution for the Hopf function taken from the table on p.72 (0.577351).
 !! Only using the surface value (tau=0) of the function to obtain approximate 
 !! initial temperatures. 

      IF (IGREY == 0) THEN
          TEMP(1:ND) = SQRT(SQRT(0.75*TEFF**4*(TAU(1:ND)+0.577351)))
      ENDIF

  END SUBROUTINE STN_STAND
