LOGICAL FUNCTION KEY_CONTAINS (STRING,VALUE,OPTIONS)

  !+
  !   Program:
  !      KEY_CONTAINS
  !
  !   Purpose:
  !=     Compare a test string (X) of arbitrary length with a given value (Y)
  !      return T if Y contains X, F if not
  !
  !   Options:
  !             : (default) case ignored, match only up to first blank character
  !      C      : Require case matching
  !
  !   Input:
  !      STRING  - CHARACTER*(*)  - the string to be tested
  !      VALUE   - CHARACTER*(*)  - the value it should have
  !      OPTIONS - CHARACTER*(*)  - options for key matching (OPTIONAL)
  !
  !-

  !  Global type declaration
  IMPLICIT NONE

  ! Subroutine argumemnts (given):        
  CHARACTER*(*), INTENT(IN)           ::  STRING
  CHARACTER*(*), INTENT(IN)           ::  VALUE
  CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS

  !.

  ! Default return value
  KEY_CONTAINS = .FALSE.

  ! apply options if any are present
  IF ( PRESENT (OPTIONS) ) THEN
  ! convert string and value to lower case unless 'C' set
  IF ( SCAN (OPTIONS,'C') == 0 ) THEN
     CALL STR_LCASE (STRING)
     CALL STR_LCASE (VALUE)
  ENDIF
  ENDIF

  ! test if string is contained by finding string position
  IF ( INDEX(STRING, VALUE) /=0 ) KEY_CONTAINS = .TRUE.
  
END FUNCTION KEY_CONTAINS
