LOGICAL FUNCTION KEY_MATCH (STRING,VALUE,OPTIONS)

  !+
  !   Program:
  !      KEY_MATCH
  !
  !   Purpose:
  !=     Compare the start of a test string of arbitrary length with a given value
  !      return T of they match up to the value given, F if not
  !
  !   Options:
  !             : (default) case ignored, match only up to first blank character
  !      C      : Require case matching
  !
  !   Input:
  !      STRING  - CHARACTER*(*)  - the string to be tested
  !      VALUE   - CHARACTER*(*)  - the value it should have
  !      OPTIONS - CHARACTER*(*)  - options for key matching (OPTIONAL)
  !
  !-

  !  Global type declaration
  IMPLICIT NONE

  ! INCLUDE 'str_par.f'

  ! Function result (returned):
  ! LOGICAL KEY_MATCH

  ! Subroutine argumemnts (given):        
  CHARACTER*(*), INTENT(IN)           ::  STRING
  CHARACTER*(*), INTENT(IN)           ::  VALUE
  CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS

  ! Local variables:
  INTEGER :: STRING_LENGTH ! length of test  string
  INTEGER :: VALUE_LENGTH  ! length of test value 
  INTEGER :: COMP_LENGTH   ! maximum string length for valid comparison

  !.

  ! Default return value
  KEY_MATCH = .FALSE.

  ! apply options if any are present
  IF ( PRESENT (OPTIONS) ) THEN
  ! convert string and value to lower case unless 'C' set
  IF ( SCAN (OPTIONS,'C') == 0 ) THEN
     CALL STR_LCASE (STRING)
     CALL STR_LCASE (VALUE)
  ENDIF
  ENDIF

  ! find actual length of string and value 
  CALL STR_LEN (STRING, STRING_LENGTH)
  CALL STR_LEN (VALUE, VALUE_LENGTH) 

  ! check matched string variables have non-zero length
  COMP_LENGTH = MIN (STRING_LENGTH, VALUE_LENGTH) 
  IF ( COMP_LENGTH == 0 ) THEN 
     WRITE (*,*) '** key_match **: attempting to match null string'
     RETURN
  ENDIF

  ! Apply test and return result
  IF ( STRING (1:COMP_LENGTH) == VALUE (1:COMP_LENGTH) ) &
       KEY_MATCH = .TRUE. 

END FUNCTION KEY_MATCH
