SUBROUTINE TAP_ABINI( TAP_ABUND, STATUS )

!+
!  Name:
!     TAP_ABINI

!  Purpose:
!=    Initialise the elemental abundances.

!  Language:
!     Fortran 90

!  Invocation:
!     CALL TAP_ABINI( TAP_ABUND, STATUS )

!  Description:
!     The elemental abundances used by the Theoretical Astrophysics
!     library routines are initialised. This is done by providing a
!     DOUBLE PRECISION FUNCTION TAP_ABUND as an argument, which is then
!     used to initialise the internal array of abundances. This allows
!     the user to use their own abundance data, if required.
!
!     In order to use TAP_ABINI, the routine TAP_ABUND must be declared
!     EXTERNAL in the calling routine.

!  Arguments:
!     TAP_ABUND = REAL (Given)
!        The abundance initialisation function. The FUNCTION TAP_ABUND
!        has one argument of type INTEGER which is the atomic number, Z,
!        of a given element. The FUNCTION will then return a value for
!        the logarithm abundance relative to hydrogen, where the
!        logarithm hydrogen abundance is 12.00. Where no abundance data
!        are available, or the atomic number is out of range, TAP_ABUND
!        returns the value TAPC__DNULL. The FUNCTIONS TAP_ABMET and
!        TAP_ABSOL may be used as templates for user code.
!     STATUS = INTEGER (Given and Returned)
!        The global status.

!  Authors:
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     9-JUL-1992 (PCTR):
!        Original version.
!     08-JAN-2002 (CSJ):
!        conversion to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-
      
!  Global Constants:
      USE SAE_PAR          ! Standard SAE constants
      USE TAP_PAR          ! TAP_ public constants

!  Global Variables:
      USE TAPC_CMN         ! TAP_ chemical global data

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      DOUBLE PRECISION :: TAP_ABUND

!  Status:
      INTEGER :: STATUS             ! Global status

!  Local Variables:
      INTEGER :: ZELEM              ! Element atomic number, Z

!.

!  Check inherited global status.
      IF ( STATUS .NE. SAI__OK ) RETURN

!  Loop to initialise the abundance array.
      DO ZELEM = 1, TAP__NELEM
         TABUND( ZELEM ) = REAL( TAP_ABUND( ZELEM ) )
      END DO

      END SUBROUTINE
