SUBROUTINE TAP_ERREP( TAP_ROUTN, STATUS )

!+
!  Name:
!     TAP_ERREP

!  Purpose:
!=    Report an error message associated with a TAP status value.

!  Language:
!     Fortran 90

!  Invocation:
!     CALL TAP_ERREP( TAP_ROUTN, STATUS )

!  Description:
!     The TAP error codes provided in TAP_ERR are searched for the 
!     given status value. If successful, the given routine TAP_ROUTN
!     is used to deliver the associated error message text. If 
!     unsuccessful, the standard SAE values in SAE_PAR are searched,
!     If successful, an appropriate error message is delivered via 
!     TAP_ROUTN. If unsuccessful, an error message to this effect is 
!     constructed, including the given status value, and delivered.

!  Arguments:
!     TAP_ROUTN = SUBROUTINE (Given)
!        The name of a subroutine used to report the error message
!        associated with the given status value.
!     STATUS = INTEGER (Given)
!        The global status.

!  Authors:
!     PCTR: Paul Rees (STARLINK)
!     CSJ: SImon Jeffery (Armagh Observatory)
!     {enter_new_authors_here}

!  History:
!     11-JAN-1993 (PCTR):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Converted to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE SAE_PAR          ! Standard SAE error codes
      USE TAP_PAR          ! TAP public constants
      USE TAP_ERR          ! TAP error codes

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Status:
      INTEGER :: STATUS             ! Global status

!  External References:
      EXTERNAL TAP_ROUTN         ! Error reporting routine

!  Local Variables:
      INTEGER :: ISTAT              ! Local status
      INTEGER :: TEXLEN             ! Length of TEXT

      CHARACTER * ( TAP__SZMSG ) :: TEXT ! Error message text

!.

!  Check through the known TAP error status values using TAP_ERMES
!  and deliver the associated error message using TAP_ROUTN.
      IF ( STATUS .EQ. SAI__OK ) THEN
         ISTAT = TAP__ERROR
         CALL TAP_ERMES( TEXT, TEXLEN, STATUS )
         CALL TAP_ROUTN( ' ', TEXT( : TEXLEN ), ISTAT )
      ELSE
         CALL TAP_ERMES( TEXT, TEXLEN, STATUS )
         CALL TAP_ROUTN( ' ', TEXT( : TEXLEN ), STATUS )
      END IF

      END
