SUBROUTINE TAP_ERROR ( TAP_ROUTN, NAME, TEXT, STATUS )

!+
!  Name:
!     TAP_ERROR

!  Purpose:
!=    Report an error message.

!  Language:
!     Fortran 90

!  Invocation:
!     CALL TAP_ERROR ( TAP_ROUTN, NAME, TEXT, STATUS )

!  Description:
!     Use the given subroutine to deliver an error message to the user.
!     The error message has two parts: NAME and TEXT. NAME is
!     recommended to be the name of the routine in which TAP_ERROR is
!     called. TEXT is the error message text. The status value must
!     not be set to SAI__OK, but to some error value.

!  Arguments:
!     TAP_ROUTN = SUBROUTINE (Given)
!        The name of a subroutine used to report the error message
!        associated with the given status value.
!     NAME = CHARACTER * ( * ) (Given)
!        TAP error message name.
!     TEXT = CHARACTER * ( * ) (Given)
!        TAP error message text.
!     STATUS = INTEGER (Given)
!        The global status.

!  Authors:
!     PCTR: Paul Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     11-JAN-1993 (PCTR):
!        Original version.
!     8-JAN-2002 (CSJ)
!        Converted to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE SAE_PAR           ! Standard SAE error codes

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      CHARACTER * ( * ) :: NAME
      CHARACTER * ( * ) :: TEXT

!  Status:
      INTEGER :: STATUS             ! Global status

!  External References:
      EXTERNAL TAP_ROUTN         ! Error reporting routine

!.

!  Report the error message using the given routine.
      CALL TAP_ROUTN ( NAME, TEXT, STATUS )

      END
