SUBROUTINE TAP_ESYMB ( IZ, SYMBOL, STATUS )

!+
!  Name:
!     TAP_ESYMB

!  Purpose:
!=    Return the element symbol.

!  Language:
!     Fortran 90

!  Invocation:
!     CALL TAP_ESYMB ( IZ, SYMBOL, STATUS )

!  Description:
!     The element symbol associated with the given atomic number is
!     returned. If the atomic number is not valid, i.e. out of range,
!     the symbol string is returned empty and a status value of
!     TAP__BADZ is returned.

!  Arguments:
!     IZ = INTEGER (Given)
!        The atomic number.
!     SYMBOL = CHARACTER * ( * ) (Returned)
!        The associated element symbol.
!     STATUS = INTEGER (Given and Returned)
!        The global status.

!  Authors:
!     PCTR: Paul Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     14-JAN-1993 (PCTR):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Converted to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-


!  Global Constants:
      USE SAE_PAR          ! Standard SAE constants
      USE TAP_PAR          ! TAP public constants
      USE TAP_ERR          ! TAP error codes

!  Global Variables:
      USE TAPC_CMN, ONLY : TACSYM         ! TAP chemical global data

!        TACSYM( TAP__NELEM ) = CHARACTER * ( 2 ) (Given)
!           Table of element symbols.
      
!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing


!  Arguments Given:
      INTEGER :: IZ

!  Arguments Returned:
      CHARACTER * ( * ) :: SYMBOL

!  Status:
      INTEGER :: STATUS             ! Global status

!  External References:
      EXTERNAL TAP_REP           ! TAP error reporting

!.

!  Check inherited global status.
      IF ( STATUS .NE. SAI__OK ) RETURN

!  Check the given atomic number.
      IF ( ( IZ .LT. 1 ) .OR. ( IZ .GT. TAP__NELEM ) ) THEN
         STATUS = TAP__BADZ
         CALL TAP_ERROR( TAP_REP, 'TAP_ESYMB', &
                         'Atomic number is out of range.', STATUS )
         SYMBOL = ' '
      ELSE
         SYMBOL = TACSYM( IZ )
      END IF
      
      END
