SUBROUTINE TAP_REP( NAME, TEXT, STATUS )

!+
!  Name:
!     TAP_REP

!  Purpose:
!=    Report an error message from TAP.

!  Language:
!     Fortran 90

!  Invocation:
!     CALL TAP_REP( NAME, TEXT, STATUS )

!  Description:
!     An error message is built in the form:
!
!        !! <NAME>: <TEXT>
!
!     or
!
!        !! <TEXT>
!
!     if NAME is not defined. The resulting string is stripped of any
!     non-printable characters and delivered to the user on the 
!     standard output logical unit.

!  Arguments:
!     NAME = CHARACTER * ( * ) (Given)
!        TAP error message name.
!     TEXT = CHARACTER * ( * ) (Given)
!        TAP error message text.
!     STATUS = INTEGER (Given)
!        The global status.

!  Authors:
!     PCTR: Paul Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     11-JAN-1993 (PCTR):
!        Original version.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE TAP_PAR                ! TAP public constants

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      CHARACTER * ( * ) :: NAME
      CHARACTER * ( * ) :: TEXT

!  Status:
      INTEGER :: STATUS

!  External References:
      INTEGER CHR_LEN            ! String length

!  Local Variables:
      INTEGER IOSTAT             ! Fortran I/O status
      INTEGER LENGTH             ! String length

      CHARACTER * ( TAP__SZMSG ) STRING ! Output string

!.

!  Initialise STRING and append the value of NAME.
      STRING = '!! ' // NAME

!  Append the value of temp.
      LENGTH = LEN_TRIM ( NAME )

      IF ( LENGTH .GT. 0 ) THEN
         STRING( 4+LENGTH : ) = ': ' // TEXT
      ELSE
         STRING( 4 : ) = TEXT
      END IF

      CALL CHR_CLEAN( STRING )
      LENGTH = LEN_TRIM ( STRING )
      WRITE( *, '( 1X, A )', IOSTAT=IOSTAT ) STRING( : LENGTH )

      END
