MODULE TAPM_PAR

!+
!  Name:
!     TAPM_PAR

!  Purpose:
!=    Theoretical Astrophysics Library public mathematical constants.

!  Language:
!     Fortran 95

!  Type of Module:
!     Global contants module

!  Description:
!     This file defines the mathematical global constants used by the 
!     Theoretical Astrophysics Library. All values were generated using 
!     the MAPLE symbolic computation package (Char, B.W., Geddes, K.O., 
!     Gonnet, G.H., Monagan, M.B., and Watt, S.M., 1988, Watcom 
!     Publications, Waterloo, Ontario, Canada). All values are accurate
!     to 34 decimal digits and are declared as DOUBLE PRECISION constants.

!  Authors:
!     PCTR: P.C.T. Rees (STARLINK)
!     CSJ: C.S.Jeffery (St Andrews)
!     {enter_new_authors_here}

!  History:
!     14-JAN-1992 (PCTR):
!        Original version.
!     04-AUG-1992 (CSJ):
!        Inclusion of Euler's constant.
!     08-JAN-2001 (CSJ):
!        Conversion to f95 MODULE
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Pi.
!  TAPM__PI = PI
      DOUBLE PRECISION, PARAMETER ::TAPM__PI = 3.141592653589793238462643383279503D+00

!  Pi/2.
!  TAPM__PIB2 = PI / 2.0D+00
      DOUBLE PRECISION, PARAMETER ::TAPM__PIB2 = 1.570796326794896619231321691639752D+00

!  2Pi.
!  TAPM__2PI = 2.0D+00 * PI
      DOUBLE PRECISION, PARAMETER ::TAPM__2PI = 6.283185307179586476925286766559006D+00

!  1/2Pi.
!  TAPM__1B2PI = 1.0D+00 / ( 2.0D+00 * PI )
      DOUBLE PRECISION, PARAMETER ::TAPM__1B2PI = 1.591549430918953357688837633725144D-01

!  4Pi.
!  TAPM__4PI = 4.0D+00 * PI
      DOUBLE PRECISION, PARAMETER ::TAPM__4PI = 1.256637061435917295385057353311801D+01

!  1/4Pi.
!  TAPM__1B4PI = 1.0D+00 / ( 4.0D+00 * PI )
      DOUBLE PRECISION, PARAMETER ::TAPM__1B4PI = 7.957747154594766788444188168625718D-02

!  Pi^2.
!  TAPM__PISQ = PI**2
      DOUBLE PRECISION, PARAMETER ::TAPM__PISQ = 9.869604401089358618834490999876152D+00

!  Root Pi.
!  TAPM__SQTPI = SQRT( PI )
      DOUBLE PRECISION, PARAMETER ::TAPM__SQTPI = 1.772453850905516027298167483341145D+00

!  Log Pi.
!  TAPM__LGPI = LOG10( PI )
      DOUBLE PRECISION, PARAMETER ::TAPM__LGPI = 4.971498726941338543512682882908990D-01

!  Ln Pi.
!  TAPM__LNPI = LOG( PI )
      DOUBLE PRECISION, PARAMETER ::TAPM__LNPI = 1.144729885849400174143427351353059D+00

!  Degrees to radians: Pi/180.
!  TAPM__DTOR = PI / 180.0D+00
      DOUBLE PRECISION, PARAMETER ::TAPM__DTOR = 1.745329251994329576923690768488613D-02

!  Radians to degrees: 180/Pi.
!  TAPM__RTOD = 180.0D+00 / PI
      DOUBLE PRECISION, PARAMETER ::TAPM__RTOD = 5.729577951308232087679815481410517D+01

!  Arc seconds to radians: Pi/(180*3600).
!  TAPM__ASTOR = PI / ( 180.0D+00 * 3600.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__ASTOR = 4.848136811095359935899141023579480D-06

!  Radians to arc seconds: (180*3600)/Pi.
!  TAPM__RTOAS = ( 180.0D+00 * 3600.0D+00 ) / PI
      DOUBLE PRECISION, PARAMETER ::TAPM__RTOAS = 2.062648062470963551564733573307786D+05

!  Hours to radians: Pi/12.
!  TAPM__HTOR = PI / 12.0D+00
      DOUBLE PRECISION, PARAMETER ::TAPM__HTOR = 2.617993877991494365385536152732919D-01

!  Radians to hours: 12/Pi.
!  TAPM__RTOH = 12.0D+00 / PI
      DOUBLE PRECISION, PARAMETER ::TAPM__RTOH = 3.819718634205488058453210320940344D+00

!  Seconds time to radians: Pi/(12*3600).
!  TAPM__STOR = PI / ( 12.0D+00 * 3600.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__STOR = 7.272205216643039903848711535369220D-05

!  Radians to hours time: (12*3600)/Pi.
!  TAPM__RTOS = ( 12.0D+00 * 3600.0D+00 ) / PI
      DOUBLE PRECISION, PARAMETER ::TAPM__RTOS = 1.375098708313975701043155715538524D+04

!  e.
!  TAPM__E = E
      DOUBLE PRECISION, PARAMETER ::TAPM__E = 2.718281828459045235360287471352662D+00

!  Log e.
!  TAPM__LGE = LOG10( E )
      DOUBLE PRECISION, PARAMETER ::TAPM__LGE = 4.342944819032518276511289189166051D-01

!  e^Pi.
!  TAPM__EPI = E**PI
      DOUBLE PRECISION, PARAMETER ::TAPM__EPI = 2.314069263277926900572908636794854D+01

!  gamma (Euler's constant).
!  TAPM__GAMMA = LIM (M -> INF.) [ 1 + 1/2 + 1/3 + 1/4 + ... + 1/M - ln M ]
      DOUBLE PRECISION, PARAMETER ::TAPM__GAMMA = 0.5772156649015328606065120900824024D+00

!  ln gamma.
!  TAPM__LNGAM = LOG ( GAMMA )
      DOUBLE PRECISION, PARAMETER ::TAPM__LNGAM = -0.5495393129816448223376617688029078D+00


!  Root 2.
!  TAPM__SQT2 = SQRT( 2.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__SQT2 = 1.414213562373095048801688724209698D+00

!  Log 2.
!  TAPM__LG2 = LOG10( 2.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__LG2 = 3.010299956639811952137388947244931D-01

!  Ln 2.
!  TAPM__LN2 = LOG( 2.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__LN2 = 6.931471805599453094172321214581766D-01

!  Root 3.
!  TAPM__SQT3 =  SQRT( 3.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__SQT3 = 1.732050807568877293527446341505872D+00

!  Log 3.
!  TAPM__LG3 = LOG10( 3.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__LG3 = 4.771212547196624372950279032551155D-01

!  Ln 3.
!  TAPM__LN3 = LOG( 3.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__LN3 = 1.098612288668109691395245236922526D+00

!  Root 10.
!  TAPM__SQT10 = SQRT( 10.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__SQT10 = 3.162277660168379331998893544432719D+00

!  Ln 10.
!  TAPM__LN10 = LOG( 10.0D+00 )
      DOUBLE PRECISION, PARAMETER ::TAPM__LN10 = 2.302585092994045684017991454684364D+00

!.

END MODULE
