SUBROUTINE SFIT_LOADGRID  ( GRDFILE, &
     NWMAX,NTMAX,NGMAX,NHMAX, &
     NWMOD,NTMOD,NGMOD,NHMOD, &
     WMOD, TMOD, GMOD, HMOD, &
     FMOD, CMOD )

!+
!  Program:
!     sfit_loadgrid
!
!  Purpose:
!     Load up a partition tree-index grid of models.
!!
!  Language:
!     Fortran 90
! 
!  Reference:
!
!  Authors:
!     C. Winter (CWR) (Armagh Observatory)
!
!  History:
!     27-Jan-2005: CWR - Adaptation from SFIT_RDBIN
!     26-Oct-2001: CSJ - Original SFIT_RDBIN subroutine
!
!-

      USE mod_model_list
      USE mod_tetra_list


! Implicit declaration
      IMPLICIT NONE

      CHARACTER(LEN=256), EXTERNAL :: STR_APPEND


! Subroutine Arguments.
      CHARACTER :: GRDFILE*(*)        ! filename for model grid
      
      INTEGER   :: NWMAX             ! maximum number of wavelengths
      INTEGER   :: NTMAX             ! maximum number of temperatures
      INTEGER   :: NGMAX             ! maximum number of gravities
      INTEGER   :: NHMAX             ! maximum number of helium abundances 

      INTEGER   :: NWMOD             ! number of wavelengths
      INTEGER   :: NTMOD             ! number of temperatures 
      INTEGER   :: NGMOD             ! number of gravities
      INTEGER   :: NHMOD             ! number of helium abundances

      REAL      :: TMOD(NTMAX)       ! temperatures in grid
      REAL      :: GMOD(NGMAX)       ! gravities in grid
      REAL      :: HMOD(NHMAX)       ! helium abundances in grid
      REAL      :: WMOD(NWMAX)       ! wavelengths in model

      REAL      :: FMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! emergent fluxes
      REAL      :: CMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! continuum fluxes
!      REAL      :: AMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! normalized fluxes

      INTEGER   :: STAT

      REAL, DIMENSION(2001) :: TEST


! Read in the index.

      print *, "Reading grid index"

      OPEN (1, FILE=STR_APPEND(GRDFILE, ".idx"), STATUS='OLD', FORM='UNFORMATTED')
      !!! Read grid dimensions
      READ (1) NWMOD

      !!! Read grid axes
      !READ  (1) TMOD(1:NTMOD)
      !READ  (1) GMOD(1:NGMOD)
      !READ  (1) HMOD(1:NHMOD)
      READ  (1) WMOD(1:NWMOD)

      !!! Load model list and tetrahedra list
      CALL MODLIST_LOAD(1, MODSLIST)
      CALL TETRALIST_LOAD(1, TETRALIST)
      CLOSE (1)

      !!! Hackish approach to remembering the grid filename
      CALL MODLIST_OPEN_GRID(GRDFILE)

      print *, "Models: ", MODLIST_LENGTH(MODSLIST)
      print *, "Tetrahedra: ", TETRALIST_LENGTH(TETRALIST)

! Read in the wavelength grid from the models file.

      OPEN(UNIT=26, FILE=GRDFILE, ACCESS="DIRECT", RECL=4*NWMOD, &
           FORM="UNFORMATTED", IOSTAT=STAT)

      IF(STAT .NE. 0) THEN
         PRINT *, "sfit_loadgrid: Unable to open model grid ", GRDFILE
         STOP
      ELSE
         READ (UNIT=26, REC=1) WMOD(1:NWMOD)
      ENDIF

      CLOSE(26) 


!  Report grid

      WRITE (*,'(/'' Binary grid:'')')
      !WRITE (*,'(4x,''NMOD '',I5)') NTMOD*NGMOD*NHMOD
      WRITE (*,'(4x,''WMOD '',I5,10F10.2)') NWMOD,WMOD(1),WMOD(NWMOD)
      !WRITE (*,'(4x,''TMOD '',I5,10F10.2,3(/10F10.2))') NTMOD,TMOD(1:NTMOD)
      !WRITE (*,'(4x,''GMOD '',I5,10F10.2)') NGMOD,GMOD(1:NGMOD)
      !WRITE (*,'(4x,''AMOD '',I5,2X,10F10.4)') NHMOD,HMOD(1:NHMOD)
      WRITE (*,*)

      WRITE (11,'(/'' Binary grid:'')')
      !WRITE (11,'(4x,''NMOD '',I5)') NTMOD*NGMOD*NHMOD
      WRITE (11,'(4x,''WMOD '',I5,10F10.2)') NWMOD,WMOD(1),WMOD(NWMOD)
      !WRITE (11,'(4x,''TMOD '',I5,10F10.2,3(/10F10.2))') NTMOD,TMOD(1:NTMOD)
      !WRITE (11,'(4x,''GMOD '',I5,10F10.2)') NGMOD,GMOD(1:NGMOD)
      !WRITE (11,'(4x,''AMOD '',I5,2X,10F10.4)') NHMOD,HMOD(1:NHMOD)
      WRITE (11,*)

    END SUBROUTINE SFIT_LOADGRID
