MODULE SFIT_SYNDATA

  !+
  !
  !  Module:
  !     sfit_syndata
  !
  !  Purpose:
  !=    Holders for global variables containing parameters required to generate a synthetic spectrum
  !     using SPEC_SYNTH
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     17-FEB-2000: CSJ
  !        original version
  !
  !  Prerequisites:
  !     Module SFIT_XFIT_DIMS
  !
  !-



  !  Holders for model atmosphere
  CHARACTER(LEN=256) :: SYN__FOLDER   ! folder containing model atmospheres (input)
  CHARACTER(LEN=256) :: SYN__FILE     ! filename for model atmosphere (input)
  CHARACTER(LEN=128) :: SYN__TITLE    ! title for mdodel atmosphere (from model?)

  REAL      :: SYN__TEFF          ! Teff for model   (from model?)
  REAL      :: SYN__LOGG          ! log g for model  (from model?)
  REAL      :: SYN__VTURB = 5.0   ! microturbulent velocity (input)
  REAL      :: SYN__VSINI = 0.0   ! vsini for model  (input)
  REAL      :: SYN__VRAD = 0.0    ! v_rad for model  (input)
  REAL      :: SYN__BETA = 1.5    ! limb darkening coefficient  (input)


  !  Holders for linelist filename
  CHARACTER(LEN=256) :: SYN__LINES = 'LINES' ! filename for model atmosphere (input)

  !  Identifiers and fixed values for abundances
  INTEGER, PARAMETER        :: NAMAX = 90  ! max number of abundances
  ! INTEGER                   :: NABU    ! actual number of species

  INTEGER                   :: SYN__NPAR ! Number of abundances + 1  
  INTEGER, DIMENSION(NAMAX) :: SYN__ELID ! element identifiers
  REAL,    DIMENSION(NAMAX) :: SYN__ABUN ! synth abundances
  REAL,    DIMENSION(NAMAX) :: SYN__DABU ! synth varaibles

  !  Wavelength range for fitting spectrum
  REAL     :: SYN__WMIN, SYN__WMAX


  ! Variables controlling spectrum options : default values
  INTEGER :: SPC__NMODE = 11  ! type of calculation
  INTEGER :: SPC__NOPAC = 0  ! continnuum opacity sources
  INTEGER :: SPC__NSOURCE = 3  ! treatment of source function
  INTEGER :: SPC__NTHETA = 1  ! number of angles for intensities
  INTEGER :: SPC__NHEDAT = 2  ! source of helium data
  INTEGER :: SPC__NHESTARK = 1  ! source of HeI Stark broadening data
  !.

END MODULE SFIT_SYNDATA
