SUBROUTINE SFIT_WRBIN  ( GRDFILE, &
     NWMAX,NTMAX,NGMAX,NHMAX, &
     NWMOD,NTMOD,NGMOD,NHMOD, &
     WMOD, TMOD, GMOD, HMOD, &
     FMOD, CMOD )

  !+
  !  Program:
  !     sfit_wrbin
  !
  !  Purpose:
  !=    Write a grid of model spectra from a binary file
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Reference:
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     26-Oct-2001: CSJ
  !
  !-

  ! Implicit declaration
  IMPLICIT NONE


  ! Subroutine Arguments.
  CHARACTER :: GRDFILE*(*)        ! filename for model grid

  INTEGER   :: NWMAX             ! maximum number of wavelengths
  INTEGER   :: NTMAX             ! maximum number of temperatures
  INTEGER   :: NGMAX             ! maximum number of gravities
  INTEGER   :: NHMAX             ! maximum number of helium abundances 

  INTEGER   :: NWMOD             ! number of wavelengths
  INTEGER   :: NTMOD             ! number of temperatures 
  INTEGER   :: NGMOD             ! number of gravities
  INTEGER   :: NHMOD             ! number of helium abundances

  REAL      :: TMOD(NTMAX)       ! temperatures in grid
  REAL      :: GMOD(NGMAX)       ! gravities in grid
  REAL      :: HMOD(NHMAX)       ! helium abundances in grid
  REAL      :: WMOD(NWMAX)       ! wavelengths in model

  REAL      :: FMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! emergent fluxes
  REAL      :: CMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! continuum fluxes


  ! Temporary local arrays to aoid segmentation faults
  REAL, ALLOCATABLE :: FLOC(:)
  REAL, ALLOCATABLE :: CLOC(:)

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance
  !. 

  ! Check data arrays
!  WRITE (*,*) NTMOD, NGMOD, NHMOD, NWMOD
!  WRITE (*,*) NTMAX, NGMAX, NHMAX, NWMAX
!  WRITE (*,*) TMOD(1:NTMOD)
!  WRITE (*,*) GMOD(1:NGMOD)
!  WRITE (*,*) HMOD(1:NHMOD)
!  WRITE (*,*) WMOD(1:10),"....",WMOD(NWMOD-9:NWMOD)
!  DO NH = 1,NHMOD 
!    DO NG = 1,NGMOD
!      DO NT = 1,NTMOD
!        write (*,*) nh,ng,nt
!        WRITE (*,*) FMOD(1:3,NT,NG,NH),"...",FMOD(NWMOD-2:NWMOD,NT,NG,NH)
!        WRITE (*,*) CMOD(1:3,NT,NG,NH),"...",CMOD(NWMOD-2:NWMOD,NT,NG,NH)
!      END DO
!    END DO
!  END DO

  !  Open binary file
  OPEN  (1,FILE=GRDFILE,STATUS='UNKNOWN',FORM='UNFORMATTED')

  !  Write grid dimensions
  WRITE (1) NTMOD, NGMOD, NHMOD, NWMOD

  !  Write grid axes
  WRITE (1) TMOD(1:NTMOD)
  WRITE (1) GMOD(1:NGMOD)
  WRITE (1) HMOD(1:NHMOD)
  WRITE (1) WMOD(1:NWMOD)

  ALLOCATE ( FLOC(NWMOD), CLOC(NWMOD) )


  !  Write models
  DO NH = 1,NHMOD 
    DO NG = 1,NGMOD
      DO NT = 1,NTMOD

        ! copy to local array to avoid segmentation faults
        FLOC =  FMOD(1:NWMOD,NT,NG,NH)
        CLOC =  CMOD(1:NWMOD,NT,NG,NH)

        WRITE (1) FLOC
        WRITE (1) CLOC
      END DO
    END DO
  END DO

  DEALLOCATE ( FLOC, CLOC )

  !  Close binary file
  CLOSE (1)

  !  Report grid saved
  WRITE (*,'(/'' Binary grid saved:'')')
  WRITE (*,'(4x,''NMOD '',I5)') NTMOD*NGMOD*NHMOD

END SUBROUTINE SFIT_WRBIN








