      MODULE TFIT_MODELS

!+
!
!  Module:
!     tfit_models
!
!  Purpose:
!=    Holders for global variables containing model fluxes
!     to be used by programme tfit
!
!  Language:
!     Fortran 90
! 
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     27-May-1999: CSJ
!        original version
!
!-

      INTEGER, PARAMETER :: NWMAX = 1500  ! maximum number of wavelengths
      INTEGER, PARAMETER :: NTMAX =   40  ! maximum number of temperatures
      INTEGER, PARAMETER :: NGMAX =   10  ! maximum number of gravities
      INTEGER, PARAMETER :: NHMAX =    5  ! maximum number of abundances

      INTEGER   :: NWMOD             ! number of wavelengths in model
      INTEGER   :: NTMOD             ! number of temperatures in model
      INTEGER   :: NGMOD             ! number of gravities in model
      INTEGER   :: NHMOD             ! number of abundances in model

      REAL      :: TMOD(NTMAX)       ! temperatures in grid
      REAL      :: GMOD(NGMAX)       ! gravities in grid
      REAL      :: HMOD(NHMAX)       ! abundances in grid
      REAL      :: WMOD(NWMAX)       ! wavelengths in model

      REAL      :: FMOD(NWMAX,NTMAX,NGMAX,NHMAX) ! fluxes in model
      REAL      :: CMOD(NWMAX,NTMAX,NGMAX,NHMAX) ! continuum fluxes in model (dummy)
      REAL      :: AMOD(NWMAX,NTMAX,NGMAX,NHMAX) ! residual fluxes in model (dummy)
      CHARACTER :: MODTITLE*128      ! title for model grid
      CHARACTER :: MODFILE*256       ! filename for model spectra
!.
      END MODULE TFIT_MODELS
