INTEGER FUNCTION XFT_SEG_FBIN2 ( NW, NT, NG, NH, SEG )

  !+  
  !
  !  Programme:
  !     xft_seg_fbin
  !
  !  Purpose:
  !=    Extract a segment of the binned fluxed spectrum 
  !
  !  Method: 
  !     The start corner of the required segment is defined by the
  !     subroutine argument. The shape of the required segment is 
  !     defined automatically by the array segment. The values are
  !     obtained from the global variable FINT.
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     09-AUG-2002: CSJ
  !        new subroutine
  !
  !-


  ! Global constants:
  USE SFIT_BIN_TGH_2, ONLY : FBIN2


  ! Global typing
  IMPLICIT NONE


  !    Model grid pointers:
  INTEGER, INTENT(IN) ::  NW, NT, NG, NH   !

  !    Model grid segment: (assumed shape)
  REAL, INTENT(INOUT), DIMENSION(:,:,:,:) :: SEG

  !    Model segmeng shape
  INTEGER, DIMENSION(4) :: SS, ST

  !.


  ! Recover segment shape and subtract 1
  SS = SHAPE ( SEG )
  ST = SS - 1

  ! Copy data
  SEG = FBIN2 ( nw:nw+st(1), nt:nt+st(2), ng:ng+st(3), nh:nh+st(4) )

  ! Return function status
  XFT_SEG_FBIN2 = 0


END FUNCTION XFT_SEG_FBIN2
